package mct;


/**
 * @(#)NetletDaemon.java          December 30, 1996 Gatot Susilo
 */
import java.net.*;
import mct.admin.MobileCodeDaemon;
import mct.security.*;
import java.io.*;
import java.util.*;

/**
 * This is the main program of generic daemon for the mobile code.
 * To execute this program, you require a configuration file 
 * (i.e., netletdaemon.properties) that specifies the behaviour of the daemon.<br>
 * Now you can install mobile code at the first start up of executing mct.NetletDaemon; 
 * therefore you can install the first VMC, Provisioning Mobile Code,
 * etc to resemble a certain network component. To do so, you have to specify
 * the name of the mobile code's class in the configuration file and the location
 * the classes should be specified in the environment variable CLASSPATH.
 * See an example of the configuration file that is also included in the distribution.<p>
 * To simulate a different network component with different initial VMCs and mobile codes
 * can be achieved by writing a different configuration file (properties) and different
 * VMCs, mobile codes, etc.<p>
 * The daemon is instance of the MobileCodeDaemon. This daemon should be adjusted
 * for embedded systems such as routers, bridges, etc.<p>
 * <h2>Usage:</h2>
 * <h3>java mct.NetletDaemon netletdaemon.properties</h3>
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.admin.MobileCodeDaemon
 * @see     mct.security.MCDSecurityManager
 * @since   MCT1.0
 */
public class NetletDaemon extends MobileCodeDaemon {



public NetletDaemon(InetAddress tcpIp, int tcpPort, InetAddress udpIp, int udpPort, boolean flag1, boolean flag2) {
	super(tcpIp, tcpPort, udpIp, udpPort, flag1, flag2);
}
/**
   * This method runs the code associated with the class,
   * installs the jar file(s) and reads in any properties
   * that are to be associated with the running threads.
   * Tony White: I have modified this in order to ensure that
   * the class name and the mobile code container key are one and 
   * the same. Previously, this was assumed but not enforced.
   *
   * @param Prop the properties to be associated with the code
   */
private void installStartupMobileCode(Properties prop) {
	// Install mobile code if any
	mct.Console.message(null, "Installing startup mobile codes...");
	String name = "";
	int i = 0;
	for (i = 0; name != null; i++) {
		name = prop.getProperty("netletdaemon.install.mobilecode." + i);
		String propertiesFile = prop.getProperty("netletdaemon.install.mobilecode.properties." + i);
		if (name != null) {
			mct.users.MCIdentifier id = new mct.users.MCIdentifier(name);
			name = id.getID();
			String jarFileName = "";
			Vector vector = new Vector();
			for (int j = 0; jarFileName != null; j++) {
				jarFileName = prop.getProperty("netletdaemon.install.mobilecode.jar." + i + "." + j);
				if (jarFileName != null)
					vector.addElement(jarFileName);
			}
			if (vector.isEmpty())
				mct.Console.warning(null, "No jar files specified for " + name);
			else {
				String jars[] = new String[vector.size()];
				vector.copyInto(jars);
				mct.Console.message(null, "Trying to install mobile code jar(s) for " + name);
				if (installMobileCodeJar(name, jars))
					mct.Console.message(null, name + "'s jars have been installed succesfully");
				else
					mct.Console.warning(null, "Failed to install jars for " + name);
		}
		mct.Console.message(null, "Trying to install mobile code " + name);
		if (installMobileCode(name, propertiesFile))
			mct.Console.message(null, name + " has been installed successfuly");
		else
			mct.Console.warning(null, "Failed to install " + name);
	}
}
if (i == 1)
	mct.Console.warning(null, "No mobile code installed");
else
	mct.Console.message(null, "Installation of mobile code completed!");
}
/**
   * Usage: java mct.NetletDaemon netletdaemon.properties
   */
public static void main(String argv[]) throws IOException {
	if (argv.length != 1) {
		System.out.println("Usage: java mct.NetletDaemon netletdaemon.properties");
		System.exit(-1);
	}
	mct.Console.message(null, "Loading configuration file " + argv[0]);
	FileInputStream fis = new FileInputStream(argv[0]);
	Properties prop = new Properties();
	prop.load(fis);
	fis.close();
	mct.Console.message(null, "Loading configuration file completed!");
	int ltcpPort = -1, ludpPort = -1;
	int tcpPort = -1, udpPort = -1;
	boolean enableSecurity = false;
	boolean tcpAsDefaultProtocol = true;
	InetAddress destHostTcp = null, destHostUdp = null;

	// get argument parameters to set up MobileCodeDaemon
	try {
		destHostTcp = InetAddress.getByName(prop.getProperty("netletdaemon.default.migration.tcp.ip", "localhost"));
		destHostUdp = InetAddress.getByName(prop.getProperty("netletdaemon.default.migration.udp.ip", "localhost"));
		ltcpPort = Integer.parseInt(prop.getProperty("netletdaemon.listen.tcp.port", "9998"));
		ludpPort = Integer.parseInt(prop.getProperty("netletdaemon.listen.udp.port", "9999"));
		tcpPort = Integer.parseInt(prop.getProperty("netletdaemon.default.migration.tcp.port", "3000"));
		udpPort = Integer.parseInt(prop.getProperty("netletdaemon.default.migration.udp.port", "3100"));
		tcpAsDefaultProtocol = prop.getProperty("netletdaemon.default.protocol", "tcp").equals("tcp");
		enableSecurity = prop.getProperty("netletdaemon.security.enable", "true").equals("true");
	} catch (Throwable e) {
		mct.Console.error(null, "Exception: " + e);
		System.exit(-1);
	}

	// Set up SecurityManager
	if (enableSecurity) {
		String root = prop.getProperty("netletdaemon.security.root", "Admin");
		Vector storageManagerUsers = new Vector();
		Vector mobileCodeManagerUsers = new Vector();
		boolean done = false;
		for (int i = 0; !done; i++) {
			String value = prop.getProperty("netletdaemon.security.user.allows.access.mobilecodemanager." + i);
			if (value == null)
				done = true;
			else
				mobileCodeManagerUsers.addElement(value);
		}
		done = false;
		for (int i = 0; !done; i++) {
			String value = prop.getProperty("netletdaemon.security.user.allows.access.storagemanager." + i);
			if (value == null)
				done = true;
			else
				storageManagerUsers.addElement(value);
		}
		if (prop.getProperty("netletdaemon.security.enable.MCDSecurityManager").equals("true")) {
			try {
				mct.Console.message(null, "Installing MCDSecurityManager...");
				System.setSecurityManager(new MCDSecurityManager(root, mobileCodeManagerUsers, storageManagerUsers));
				mct.Console.message(null, "Installing MCDSecurityManager completed !");
			} catch (SecurityException e) {
				mct.Console.error(null, "Someone has set up SecurityManager");
				System.exit(-1);
			}
		} else
			mct.Console.warning(null, "MCDSecurityManager is disabled");
	} else {
		SecurityManager smgr = System.getSecurityManager();
		mct.Console.warning(null, "Security feature is completely disabled");
		if (smgr == null)
			System.setSecurityManager(new MCDNullSecurityManager());
	}
	// Set up instance of MobileCodeDaemon
	NetletDaemon daemon = new NetletDaemon(destHostTcp, tcpPort, destHostUdp, udpPort, tcpAsDefaultProtocol, enableSecurity);

	// Set up facilitator
	if (prop.getProperty("netletdaemon.facilitator.enable", "false").equals("true")) {
		String facClass = prop.getProperty("netletdaemon.facilitator.mobilecode");
		String facProp = prop.getProperty("netletdaemon.facilitator.properties");
		daemon.installFacilitator(facClass, facProp);
	}
	// Install mobile code if any

	daemon.installStartupMobileCode(prop);

	// Now start listening the servers
	daemon.tcpListener(ltcpPort);
	daemon.udpListener(ludpPort);
}
}