package mct;


/**
 * @(#)NetletDaemon.java          December 30, 1996 Gatot Susilo
 */
import java.net.*;
import mct.users.*;
import mct.admin.MobileCodeDaemon;
import mct.security.*;
import mct.admin.MigrationInterface;
import java.io.*;
import java.util.*;

/**
 * This is the main program of generic daemon for the mobile code.
 * To execute this program, you require a configuration file 
 * (i.e., netletdaemon.properties) that specifies the behaviour of the daemon.<br>
 * Now you can install mobile code at the first start up of executing mct.NetletDaemon; 
 * therefore you can install the first VMC, Provisioning Mobile Code,
 * etc to resemble a certain network component. To do so, you have to specify
 * the name of the mobile code's class in the configuration file and the location
 * the classes should be specified in the environment variable CLASSPATH.
 * See an example of the configuration file that is also included in the distribution.<p>
 * To simulate a different network component with different initial VMCs and mobile codes
 * can be achieved by writing a different configuration file (properties) and different
 * VMCs, mobile codes, etc.<p>
 * The daemon is instance of the MobileCodeDaemon. This daemon should be adjusted
 * for embedded systems such as routers, bridges, etc.<p>
 * <h2>Usage:</h2>
 * <h3>java mct.NetletDaemon netletdaemon.properties</h3>
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.admin.MobileCodeDaemon
 * @see     mct.security.MCDSecurityManager
 * @since   MCT1.0
 */
public class RMINetletDaemon extends MobileCodeDaemon {


public RMINetletDaemon(InetAddress tcpIp, int tcpPort, InetAddress udpIp, int udpPort, boolean flag1, boolean flag2) {
	super(tcpIp, tcpPort, udpIp, udpPort, flag1, flag2);
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 * @param name java.lang.String
 * @param propertiesFileName java.lang.String
 */
public boolean installMigrationFacilitator(String name, String propertiesFileName) {
	MobileCodeObject handle = null;
	String className = (new MCIdentifier(name)).getID();
	boolean ok = installMobileCode(className, propertiesFileName);
	if (ok) {
		while ((handle = mobileCodeManager.getMobileCodeHandle(className)) == null) {
			// Put a re-try loop instead of an "if" just in case...
			Thread.yield();
		}
		mct.Console.message(this, "Waiting for Migration Facilitator to become available ....");
		while (!((mct.admin.RemoteMigrationFacilitator) handle).isAvailable()) {
			// I had to use sleep instead of yield because yield does not
			// seem to guarantee that another thread is scheduled.
			// The sleep period was chosen experimentally : long enough for
			// a task switch, not too long to slow things down.
			try {
				Thread.sleep(150);
			} catch (Exception e) {
			}
		}
		if (handle instanceof MigrationInterface) {
			((mct.admin.RMIMobileCodeManager) mobileCodeManager).setMigrationFacilitator(handle);
		} else {
			mct.Console.error(this, "Supplied Migration Facilitator does not implement MigrationInterface. No migration provided");
			return false;
		}
		mct.Console.message(this, "Migration Facilitator is installed");
		return true;
	} else {
		mct.Console.error(this, "Failed to install Migration Facilitator");
		return false;
	}
}
/**
   * This method runs the code associated with the class,
   * installs the jar file(s) and reads in any properties
   * that are to be associated with the running threads.
   * Tony White: I have modified this in order to ensure that
   * the class name and the mobile code container key are one and 
   * the same. Previously, this was assumed but not enforced.
   *
   * @param Prop the properties to be associated with the code
   */
private void installStartupMobileCode(Properties prop) {
	// Install mobile code if any
	mct.Console.message(null, "Installing startup mobile codes...");
	String name = "";
	int i = 0;
	for (i = 0; name != null; i++) {
		name = prop.getProperty("netletdaemon.install.mobilecode." + i);
		String propertiesFile = prop.getProperty("netletdaemon.install.mobilecode.properties." + i);
		if (name != null) {
			mct.users.MCIdentifier id = new mct.users.MCIdentifier(name);
			name = id.getID();
			String jarFileName = "";
			Vector vector = new Vector();
			for (int j = 0; jarFileName != null; j++) {
				jarFileName = prop.getProperty("netletdaemon.install.mobilecode.jar." + i + "." + j);
				if (jarFileName != null)
					vector.addElement(jarFileName);
			}
			if (vector.isEmpty())
				mct.Console.warning(null, "No jar files specified for " + name);
			else {
				String jars[] = new String[vector.size()];
				vector.copyInto(jars);
				mct.Console.message(null, "Trying to install mobile code jar(s) for " + name);
				if (installMobileCodeJar(name, jars))
					mct.Console.message(null, name + "'s jars have been installed succesfully");
				else
					mct.Console.warning(null, "Failed to install jars for " + name);
		}
		mct.Console.message(null, "Trying to install mobile code " + name);
		if (installMobileCode(name, propertiesFile)) {
			mct.Console.message(null, name + " has been installed successfully");
		} else
			mct.Console.warning(null, "Failed to install " + name);
	}
}
if (i == 1)
	mct.Console.warning(null, "No mobile code installed");
else
	mct.Console.message(null, "Installation of mobile code completed!");
}
/**
   * Usage: java mct.NetletDaemon netletdaemon.properties
   */
public static void main(String argv[]) throws IOException {
	if (argv.length != 1) {
		System.out.println("Usage: java mct.RMINetletDaemon netletdaemon.properties");
		System.exit(-1);
	}
	String version = System.getProperty("java.version");
	if (version == null) {
		mct.Console.warning(null, "mct.RMINetletDaemon: Sorry, I cannot ascertain the version of java!");
	}
	else if (!version.equals("1.1.6")) {
		mct.Console.warning(null,"mct.RMINetletDaemon only supported for JDK 1.1.6");
	}
	mct.Console.message(null, "Loading configuration file " + argv[0]);
	FileInputStream fis = new FileInputStream(argv[0]);
	Properties prop = new Properties();
	prop.load(fis);
	fis.close();
	mct.Console.message(null, "Loading configuration file completed!");
	int ltcpPort = -1, ludpPort = -1;
	int tcpPort = -1, udpPort = -1;
	boolean enableSecurity = false;
	boolean tcpAsDefaultProtocol = true;
	boolean rmiAsDefaultProtocol = false;
	InetAddress destHostTcp = null, destHostUdp = null;

	// get argument parameters to set up MobileCodeDaemon
	try {
		// Make console silent or using System.out
		boolean consoleOutput;
		String console = prop.getProperty("netletdaemon.console", "true");
		if (console.equals("false")) {
			consoleOutput = false;
			mct.Console.setConsole(false);
		} else
			if (console.equals("true")) {
				consoleOutput = true;
				mct.Console.setConsole(true);
			} else
				consoleOutput = false;
		// Set up debug etc output
		if (consoleOutput)
			setupConsoleOptions(prop);
		destHostTcp = InetAddress.getByName(prop.getProperty("netletdaemon.default.migration.tcp.ip", "localhost"));
		destHostUdp = InetAddress.getByName(prop.getProperty("netletdaemon.default.migration.udp.ip", "localhost"));
		ltcpPort = Integer.parseInt(prop.getProperty("netletdaemon.listen.tcp.port", "9998"));
		ludpPort = Integer.parseInt(prop.getProperty("netletdaemon.listen.udp.port", "9999"));
		tcpPort = Integer.parseInt(prop.getProperty("netletdaemon.default.migration.tcp.port", "3000"));
		udpPort = Integer.parseInt(prop.getProperty("netletdaemon.default.migration.udp.port", "3100"));
		tcpAsDefaultProtocol = prop.getProperty("netletdaemon.default.protocol", "tcp").equals("tcp");
		rmiAsDefaultProtocol = prop.getProperty("netletdaemon.default.protocol", "tcp").equals("rmi");
		enableSecurity = prop.getProperty("netletdaemon.security.enable", "true").equals("true");
	} catch (Throwable e) {
		mct.Console.error(null, "Exception: " + e);
		System.exit(-1);
	}

	// Set up SecurityManager
	if (enableSecurity) {
		String root = prop.getProperty("netletdaemon.security.root", "Admin");
		Vector storageManagerUsers = new Vector();
		Vector mobileCodeManagerUsers = new Vector();
		boolean done = false;
		for (int i = 0; !done; i++) {
			String value = prop.getProperty("netletdaemon.security.user.allows.access.mobilecodemanager." + i);
			if (value == null)
				done = true;
			else
				mobileCodeManagerUsers.addElement(value);
		}
		done = false;
		for (int i = 0; !done; i++) {
			String value = prop.getProperty("netletdaemon.security.user.allows.access.storagemanager." + i);
			if (value == null)
				done = true;
			else
				storageManagerUsers.addElement(value);
		}
		if (prop.getProperty("netletdaemon.security.enable.MCDSecurityManager").equals("true")) {
			try {
				mct.Console.message(null, "Installing MCDSecurityManager...");
				System.setSecurityManager(new MCDSecurityManager(root, mobileCodeManagerUsers, storageManagerUsers));
				mct.Console.message(null, "Installing MCDSecurityManager completed !");
			} catch (SecurityException e) {
				mct.Console.error(null, "Someone has set up SecurityManager");
				System.exit(-1);
			}
		} else
			mct.Console.warning(null, "MCDSecurityManager is disabled");
	} else {
		try {
			SecurityManager smgr = System.getSecurityManager();
			mct.Console.warning(null, "Security feature is completely disabled");
			if (smgr == null)
				System.setSecurityManager(new MCDNullSecurityManager());
		} catch (SecurityException e) {
			mct.Console.error(null, "Someone has set up SecurityManager");
			System.exit(-1);
		}
	}


	// Set up instance of MobileCodeDaemon
	RMINetletDaemon daemon = new RMINetletDaemon(destHostTcp, tcpPort, destHostUdp, udpPort, tcpAsDefaultProtocol, enableSecurity);


	// Set up facilitator
	if (prop.getProperty("netletdaemon.facilitator.enable", "false").equals("true")) {
		String facClass = prop.getProperty("netletdaemon.facilitator.mobilecode");
		String facProp = prop.getProperty("netletdaemon.facilitator.properties");
		daemon.installFacilitator(facClass, facProp);
	}
	// Set up migration facilitator
	if (prop.getProperty("netletdaemon.migrator.enable", "false").equals("true")) {
		String migFacClass = prop.getProperty("netletdaemon.migrator.mobilecode");
		String migFacProp = prop.getProperty("netletdaemon.migrator.properties");
		daemon.installMigrationFacilitator(migFacClass, migFacProp);
	}
	// Add a default migration host and target
	String hostName = "localhost";
	String mFac = "[PUBLIC]mct.admin.RemoteMigrationFacilitator";
	for (int i = 0;(mFac = prop.getProperty("netletdaemon.migration.rmi.name." + i)) != null; i++) {
		hostName = prop.getProperty("netletdaemon.migration.rmi.host." + i, hostName);
		((mct.admin.RMIMobileCodeManager) (daemon.mobileCodeManager)).setDefaultRMIMigrationParameters(hostName, mFac);
	}

	// Resolve at least one migration host in order to ensure that
	// the startup code does not try and migrate before one is up
	daemon.resolveMigrationHost(hostName, mFac);

	// Install mobile code if any

	daemon.installStartupMobileCode(prop);

	// Now start listening the servers
	if (!rmiAsDefaultProtocol) {
		daemon.tcpListener(ltcpPort);
		daemon.udpListener(ludpPort);
	}
}
/**
 * This method resolves one of the migration hosts.
 * @param hostName String for name of host
 * @param mFac String for name of migration facilitator 
 */
private void resolveMigrationHost(String hostName, String mFac) {
	boolean resolved = false;
	int maxTries = 10;
	int sleepTime = 1000;
	int tries = 0;
	while (!resolved && tries < maxTries) {
		try {
			tries++;
			String destName = ((mct.admin.RMIMobileCodeManager) mobileCodeManager).getMigrationDestinationName(hostName, mFac);
			((mct.admin.RMIMobileCodeManager) mobileCodeManager).getMigrationDestination(destName);
			resolved = true;
		} catch (Exception ex) {
			try {
				Thread.sleep(sleepTime);
			} catch (java.lang.InterruptedException ex1) {
			}
		}
	}

}
/**
 * This method allows the user access to all properties related to 
 * I/O to the console.
 *
 * @param properties java.util.Properties
 */
private static void setupConsoleOptions (Properties properties ) {
	boolean onErr, onWarn, onMsg, onDbg, onInfo, onApp;
	String trueStr = "true";
	String falseStr = "false";
	String variable = "netletdaemon.console.display.";
	onErr = properties.getProperty(variable+"errors", trueStr).equals(trueStr);
	onWarn = properties.getProperty(variable+"warnings", trueStr).equals(trueStr);
	onMsg = properties.getProperty(variable+"messages", falseStr).equals(trueStr);
	onDbg = properties.getProperty(variable+"debug", falseStr).equals(trueStr);	
	onInfo = properties.getProperty(variable+"information", falseStr).equals(trueStr);	
	onApp = properties.getProperty(variable+"application", falseStr).equals(trueStr);	
	mct.Console.setFlags(onDbg, onErr, onMsg, onWarn, onApp, onInfo);
	return;
}
}