package mct.admin;


/**
 * @(#)Instantiator.java        June 16, 1997   Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/

import java.util.Vector;
import java.util.Enumeration;
//import java.security.IdentityScope;
//import mct.security.MobileCodeVerifier;
//import java.security.PublicKey;

/**
 * The Instantiator runs as a thread that acts as a queue for mobile code that
 * is qoing to be instatiated by the daemon (i.e., MobileCodeDaemon).
 * The received mobile code is not directly stored in the StorageManager,
 * but it must be verified first. If the mobile code passes the verifier's security checks,
 * it will be stored in the StorageManager and then it is instantiated. Otherwise,
 * it will be discarded or transported to the next default daemon. 
 * The virifier is only used when the security feature is enabled.<p>
 * 
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.security.MobileCodeVerifier
 * @see     mct.security.MCDSecurityManager
 * @see     java.security.IdentityScope
 * @since   MCT1.0
 */
public class Instantiator extends Thread implements InstantiatorInterface {
	//**********
	//* Fields *
	//**********
	protected MobileCodeManager mobileCodeManager;
	protected StorageManager storageManager;
	protected Vector queue;
	protected boolean suspended = true;
	//private IdentityScope identityScope;
	//private boolean byPassVerifier=false;
	protected MobileCodeThreadGroup mcThreadGroup;



/**
   * The class can only be constructed from the same package.
   * Typically, it is instantiated by MobileCodeDaemon.
   * This ensures the visiting mobile code can not instantiate this
   * class and by pass the verifier's security checks.
   */
protected Instantiator(MobileCodeManager mobileCodeManager, StorageManager storageManager) {
	this.mobileCodeManager = mobileCodeManager;
	this.storageManager = storageManager;
	queue = new Vector();
	mcThreadGroup = new MobileCodeThreadGroup("mobile code");
	start();
}
/**
   * Add a mobile code that is going to be instantiated by the daemon.
   * The mobile code must be stored/encapsulated in the MobileCodeContainer.
   * @see   mct.admin.MobileCodeContainer
   */
public synchronized void addElement(Object mcc) {
	queue.addElement(mcc);
	if (suspended) {
		resume();
	}
}
/**
   * The class runs as a thread. It instantiates a mobile code if it finds
   * a mobile code in its queue. If the queue is empty,
   * the thread is suspended, and it will be resumed when a mobile code is
   * added in the queue by using method addElement(MobileCodeContainer).<p>
   *
   * Note: transporting the mobile code to next default destination has not
   * been added if the mobile code does not pass the verifier's secuirity checks.
   * Instead the mobile code is just discarded.
   */
public void run() {
	while (true) {
		try {
			if (queue.isEmpty()) {
				suspended = true;
				suspend();
				suspended = false;
			}
			//System.out.println ("Queue is not empty");
			MobileCodeContainer mccQ = (MobileCodeContainer) queue.firstElement();
			queue.removeElementAt(0); // Remove first element
			String mcQName = mccQ.getMobileCodeName();
			if (mobileCodeManager.verifyMobileCodeContainer(mccQ)) {
				MobileCodeContainer mcc = storageManager.getContainer(mcQName);
				if (mcc == null) {
					//	System.out.println ("Instantiator: "+mcQName+" does not exist yet");
					storageManager.putContainer(mccQ);
					mobileCodeManager.instantiate(mcQName);
				} else {
					// see if I can replace the old mobile code
					if (mobileCodeManager.replace(mcQName)) {
						mct.Console.warning(this, mcQName + " already exists, so replace it");
						storageManager.removeContainer(mcQName);
						storageManager.putContainer(mccQ);
						mobileCodeManager.instantiate(mcQName);
					}
					// nop.. you can't so just discard it
					else {
						mct.Console.warning(this, mcQName + " already exists and cannot replace it");
						mccQ = null; // discard it
					}
				}
			} else
				mccQ = null; // discard it
		} catch (Throwable e) {
			mct.Console.error(this, "Instantiator : exception occurs..");
			e.printStackTrace();
		}
	}
}
}