package mct.admin;


/**
 * @(#)Listener.java		1.0	13-Nov-1996 Gatot Susilo
 * Revision:
 *  May 24, 1997, Gatot Susilo
 *    The package is renamed to mct from pmc
 *  25-Feb-1996 Gatot Susilo
 *    I rename AgentDaemon into MobileCodeDaemon, because it does not
 *    handle only "Agent" but any MobileCode.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/

import java.net.*;
import java.io.*;
import java.util.*;
import mct.net.*;
import mct.mediator.CommunicationFacilitator;
import java.security.IdentityScope;

/**
 * The listener handles either via
 * the TCP or the UDP. It runs as a thread that will listen
 * at a certain port.
 */
class Listener extends Thread {
	//**********
	//* Fields *
	//**********

	// The port number where the Listener listens to
	int port;

	// The storage manager handler and process manager handler
	// The TCP or the UDP listener should share this handler
	private Instantiator instantiator;

	// TCP is true if it is TCP listener and vice-versa
	private boolean TCP;



/**
   * The listener constructor.
   * @param port the port number where the listener should listens to
   * @param netletManager the handler toward the netletManager
   * @param tcp true if it is TCP listener, and vice versa
   *
   */
Listener(int port, Instantiator instantiator, boolean tcp) {
	this.port = port;
	this.instantiator = instantiator;
	TCP = tcp;
}
private java.lang.reflect.Method getHandleMessageMethod() {
	// Find the method object for the handleMessage(java.net.Socket) method
	java.lang.reflect.Method method = null;
	Class[] object = new Class[1];
	try {
		object[0] = Class.forName("java.net.Socket");
		method = this.getClass().getMethod("handleMessage", object);
	} catch (java.lang.ClassNotFoundException ex) {
	} catch (java.lang.NoSuchMethodException ex) {
	};
	return method;
}
/**
 * This method was created by a SmartGuide.
 * Handle the actual message (runs in a thread separate from Listener).
 * @param socket java.net.Socket
 */
public void handleMessage(Socket socket) {
	if (socket != null) {
		Ctp ctp = null;
		try {
			// We are connected so receive the PDU
			ctp = new Ctp(socket);
			MobileCodeContainer mc = ctp.receive(instantiator);
		} catch (IOException e) {
			mct.Console.error(this, "TCP I/O error: " + e);
		} catch (CtpException ctpe) {
			//throw ctpe;
		} finally {
			try {
				ctp.close();
			} catch (IOException ex) {
			};
		}
	}
	return;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isTCP() {
	return TCP;
}
// Forever loop for the corresponding listener

public void run() {
	if (isTCP())
		runTCP();
	else
		runUDP();
}
// Forever loop for the corresponding listener
private void runTCP() {
	// Find the method object for the handleMessage(java.net.Socket) method
	java.lang.reflect.Method method = getHandleMessageMethod();
	if (method == null)
		return;
	Object[] parameters = new Object[1];
	// Create the server socket and listen
	ServerSocket serverSocket = null;
	try {
		serverSocket = new ServerSocket(port);
		mct.Console.message(this, "Waiting for TCP connections at port " + port);
		while (true) {
			parameters[0] = serverSocket.accept();
			new MethodRunnerThread(this, method, parameters, MCThreadNullExceptionHandler.getDefaultHandler());
		}
	} catch (IOException ex) {
		mct.Console.error(this, "Failed to open server socket");
	} finally {
		mct.Console.warning(this, "Closing TCP socket");
		if (serverSocket != null) {
			try {
				serverSocket.close();
			} catch (IOException ex) {
				mct.Console.error(this, "Server Socket cannot be closed");
			}
		}
	}
}
// Forever loop for the corresponding listener
private void runUDP() {
	DatagramSocket datagramSocket = null;
	Ctp ctp = null;
	try {
		datagramSocket = new DatagramSocket(port);
		mct.Console.message(this, "Waiting for UDP connections at port " + port);
		while (true) {
			// Create the UDP port and waiting for the reception
			// of the mobile code
			ctp = new Ctp(datagramSocket);
			MobileCodeContainer mc = ctp.receive(instantiator);
		}
	} catch (SocketException ex) {
		mct.Console.error(this, "Socket cannot bind to port " + port);
	} finally {
		mct.Console.warning(this, "Closing UDP socket");
		try {
			if (ctp != null)
				ctp.close();
		} catch (IOException e) {
		};
	}
}
}