package mct.admin;


/**
 * @(#)MctClassLoader.java December 6, 1996 Gatot Susilo
 *
 * Revision:
 *  May 24, 1997, Gatot Susilo
 *    - It's renamed to MctClassLoader from PmcClassLoader
 *    - Search on multiple classes mobile code problems is fixed.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/

import java.util.Hashtable;

/**
 * Each instantiated mobile code uses this class loader.
 * You have to modify this if you want to change the behaviour of
 * the class loader for your mobile code. <p>
 * Of course, the class loader can be constructed if only if it is allowed
 * by the MCDSecurityManager.
 *
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.admin.StorageManager
 * @see     mct.security.MCDSecurityManager
 * @see     mct.admin.MobileCodeContainer
 * @since   MCT1.0
 */
public class MctClassLoader extends ClassLoader {
	//**********
	//* Fields *
	//**********
	private StorageManager storageManager;
	private Hashtable cache;
	private MobileCodeContainer mcc;



//****************
//* Constructors *
//****************
/**
   * The class loader will search the class in the systemClass 
   * (i.e., CLASSPATH) first and then in the MobileCodeContainer associated to the only
   * mobile code that is going to be instantiated. Therefore, every mobile code has
   * its own name space.
   * 
   * @param storageManager the handler of storageManager
   * @param name the name of the mobile code that is going to be instantiated
   */
public MctClassLoader(StorageManager storageManager, String name) {
	this.storageManager = storageManager;
	if (storageManager == null)
		mcc = null;
	else
		mcc = this.storageManager.getContainer(mct.users.MCIdentifier.getClassFromIdentifier(name));
	cache = new Hashtable();
}
/**
   * First of all, find the class in the system and then
   * find the class in the MobileCodeContainer.<p>
   *
   * You may change the order of the search if you want to
   * make that your classes in the MobileCodeContainer are precendence
   * to ones in the system. <p>
   *
   * @see#ClassLoader class for further detail.
   *
   */
public synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
	Class c = (Class) cache.get(name);
	//System.out.println ("ClassLoader: "+name+" resolve: "+resolve);
	if (c == null) {
		try {
			c = findSystemClass(name);
		} catch (ClassNotFoundException ex) {
			if (name.startsWith("java."))
				throw new ClassNotFoundException();
			byte data[] = loadClassData(name);
			if ((data != null) && (data.length > 0)) {
				// Gatot :          c = defineClass(data, 0, data.length);
				//       c = defineClass(name, data, 0, data.length);
				c = defineClass(null, data, 0, data.length);
				cache.put(name, c);
			} else {
				System.out.println("Fail to load class " + name);
				resolve = false;
			}
		}
	}
	if (resolve)
		resolveClass(c);
	return c;
}
/*
   * Find the bytecode in the MobileCodeContainer of corresponding class.
   */
public synchronized byte loadClassData(String name)[] {
	if (mcc == null)
		return null;
	return mcc.getClass(name + ".class");
}
}