package mct.admin;


/**
 * @(#)MobileCodeContainer.java	1.0 13-Nov-1996	Gatot Susilo
 * Revision:
 *   June 2, 1997, Gatot Susilo
 *      - It implements Serializable, hence it can be shipped right away to 
 *        the next destination.
 *      - It support jar as well.
 *   May 24, 1997, Gatot Susilo
 *      The package is renamed to mct from pmc
 *   25-Feb-1997 (Gatot Susilo)
 *      I renamed NetletContainer into MobileCodeContainer,
 *      because the container can accomodate any Mobile Codes
 *      and not only Netlet.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.Hashtable;
import java.util.Properties;
import java.util.Enumeration;
import java.lang.String;
import java.io.Serializable;
import java.security.PublicKey;
import java.util.BitSet;
import mct.users.MCIdentifier;
/**
 * This class stores mobile code's bytecodes (classes) and also its parameters.
 * Therefore you don't  have to store them in the local disk.
 * The classes are stored in the JAR format and signed
 * by proper signer. On migrating the mobile code, the MobileCodeContainer is
 * serialized and shipped to the next MobileCodeDaemon. As a result, the parameters
 * can be any objects and must implement interface <b>Serializable</b>. Fail
 * to do so, the parameter may not be accessed properly.<p>
 *
 * The JAR is verified and inflated into classes that are stored in the cache
 * (dedicated space in the MobileCodeContainer). The cache is read-only, therefore
 * you cannot add/modify classes in the cache. To add or modify class(es),
 * you can add the JAR in the MobileCodeContainer and followed by calling
 * MobileCodeManager.verifyMobileCodeContainer (mcc).<br>
 * The persistent state is also added if the mobile code carries one during
 * migration. The cache and persistent state in this class are not serialized
 * during migration.<p>
 *
 * One of the class must have name "MobileCodeName" that will be instantiated
 * by the MobileCodeDaemon.<p>
 *
 * @version 1.0, July 1,1996
 * @author  Gatot Susilo
 * @see     java.io.Serializable
 * @see     mct.admin.MobileCodeContainer
 * @since   MCT1.0
 */
public class MobileCodeContainer implements Serializable {
	//**********
	//* Fields *
	//**********
	private MCIdentifier MobileCodeName;
	transient private byte persistentState[] = null;
	transient private Hashtable byteCode = null;
	transient private PublicKey signer = null;
	transient private MCTNetworkClassLoader classLoader = null;
	private Properties parameter;
	private Hashtable jar;
	// Used by the system for state-related mobile code changes,
	// i.e. bit zero is set when the mobile code migrates for the 
	// first time. Other bits will be used as necessary.
	private BitSet flags;



/**
   * Constructor of MobileCodeContainer
   * @param mobileCodeName the name of mobile code and must correspond to the main class of the mobile code
   */
public MobileCodeContainer(String mobileCodeName) {
	MobileCodeName = new MCIdentifier(mobileCodeName);
	parameter = new Properties();
	byteCode = new Hashtable();
	jar = new Hashtable();
	flags = new BitSet(8);
	clearMigrationFlag();
}
/**
 * This method was created by a SmartGuide.
 * @param identifier mct.users.MCIdentifier
 */
public MobileCodeContainer(MCIdentifier identifier) {
	MobileCodeName = identifier;
	parameter = new Properties();
	byteCode = new Hashtable();
	jar = new Hashtable();
	flags = new BitSet(8);
	clearMigrationFlag();
}
/**
 * This method was created by a SmartGuide.
 * @param name java.lang.String
 * @param jarBytes byte[]
 */
public synchronized void addJar(String name, byte[] jarBytes) {
	jar.put(name, jarBytes);
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public void clearMigrationFlag() {
	flags.clear(0);
}
/**
 * This method was created by a SmartGuide.
 * @return mct.admin.MobileCodeContainer
 * @param cloneID mct.users.MCIdentifier
 */
public MobileCodeContainer clone(MCIdentifier cloneID) {
	MobileCodeContainer mcc = new MobileCodeContainer(cloneID);
	mcc.parameter = parameter;
	mcc.jar = jar;
	mcc.signer = signer;
	mcc.byteCode = byteCode;
	mcc.persistentState = persistentState;
	return null;
}
/**
   * Get bytecode whose class name specified on <tt>name</tt>
   * @param name Class name
   * @return bytecode if the class is found in the table otherwise return null
   *
   */
public synchronized byte getClass(String name)[] {
	if (byteCode == null)
		return null;
	return (byte[]) byteCode.get(name);
}
/**
 * This method was created by a SmartGuide.
 * @return mct.admin.MCTNetworkClassLoader
 */
public MCTNetworkClassLoader getClassLoader() {
	return classLoader;
}
/**
   * Get the list of classes
   */
public synchronized Enumeration getClassNames() {
	if (byteCode == null)
		return null;
	return byteCode.keys();
}
/**
 * This method was created by a SmartGuide.
 * @return java.util.BitSet
 */
public synchronized BitSet getFlags() {
	return flags;
}
/**
   * Get the JAR table. You can add/modify the JAR in the 
   * MobileCodeContainer. You should call 
   * MobileCodeManager.verifyMobileCodeContainer to inflate the JAR
   * into bytecode (classes).
   * @return hashtable that stores the JAR. The key is jarName,
   *         and the value is byte array of JAR.
   */
public synchronized Hashtable getJarTable() {
	return jar;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean getMigrationFlag() {
	return flags.get(0);
}
/**
   * Return MobileCode's name
   */
public synchronized String getMobileCodeName() {
	return MobileCodeName.getID();
}
/**
   * Get Mobile Code's parameter
   * @param name Parameter's name
   * @return value of Netlet's parameter whose name is specified
   */
public synchronized Object getParameter(Object name) {
	return parameter.get(name);
}
/**
   * Get the list of parameter names.
   */
public synchronized Enumeration getParameterNames() {
	return parameter.keys();
}
/**
 * This method was created by a SmartGuide.
 * @return java.util.Properties
 */
public synchronized Properties getParameters() {
	return parameter;
}
/**
   * Get the list of parameter values.
   */
public synchronized Enumeration getParameterValues() {
	return parameter.elements();
}
/**
   * Get persistent state.
   */
public synchronized byte[] getPersistentState() {
	return persistentState;
}
/**
   * Get the signer's public key.
   */
protected synchronized PublicKey getSignerPublicKey() {
	return signer;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean hasMigrated() {
	return getMigrationFlag();
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean hasParameters() {
	return !parameter.isEmpty();
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean hasPersistentState() {
	return persistentState != null;
}
/**
 * This method was created by a SmartGuide.
 * @param className java.lang.String
 * @param classByteCode byte[]
 */
public void putClassByteCode(String className, byte[] classByteCode) {
	if (classByteCode != null)
		byteCode.put(className, classByteCode);
}
/**
   * Put the class table as result of inflating the JARs.
   */
protected synchronized void putClassTable(Hashtable table) {
	byteCode = table;
}
/**
   * Store Netlet's parameter
   * @param name Parameter name;
   * @param value Paramter value;
   */
public synchronized void putParameter(Object name, Object value) {
	parameter.put(name, value);
}
/**
   * Put persistent state.
   */
public synchronized void putPersistentState(byte states[]) {
	persistentState = states;
}
/**
   * Put the signer's public key.
   */
protected synchronized void putSignerPublicKey(PublicKey publicKey) {
	signer = publicKey;
}
/**
   * Remove bytecode whose class name specified on <tt>name</tt> from the table
   * @param name Class name
   * @return bytecode if the class is found in the table otherwise return null
   *
   */
public synchronized byte removeClass(String name)[] {
	if (byteCode == null)
		return null;
	return (byte[]) byteCode.remove(name);
}
/**
   * Remove Mobile Code's parameter from the table
   * @param name Parameter's name
   * @return value of Netlet's parameter whose name is specified
   */
public synchronized Object removeParameter(Object name) {
	return parameter.remove(name);
}
/**
 * This method was created by a SmartGuide.
 * @return mct.admin.MCTNetworkClassLoader
 */
public MCTNetworkClassLoader setClassLoader(MCTNetworkClassLoader classLoader) {
	return this.classLoader = classLoader;
}
/**
 * This method was created by a SmartGuide.
 * @param flags java.util.BitSet
 * @return java.util.BitSet
 */
public BitSet setFlags(BitSet flags) {
	return this.flags = flags;
}
/**
 * This method was created by a SmartGuide.
 * Only mct.admin can access this method.
 * @return boolean
 */
protected void setMigrationFlag() {
	flags.set(0);
}
/**
 * This method was created by a SmartGuide.
 * @return java.util.Properties
 * @param parameters java.util.Properties
 */
public Properties setParameters(Properties parameters) {
	if (parameters != null)
		this.parameter = parameters;
	return this.parameter;
}
}