package mct.admin;


/**
 * @(#)MobileCodeDaemon.java		1.0	13-Nov-1996 Gatot Susilo
 * Revision:
 *  May 24, 1997, Gatot Susilo
 *    The package is renamed to mct from pmc
 *  25-Feb-1996 Gatot Susilo
 *    I rename AgentDaemon into MobileCodeDaemon, because it does not
 *    handle only "Agent" but any MobileCode.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/

import java.net.*;
import java.io.*;
import java.util.*;
import mct.net.*;
import mct.users.*;
import mct.mediator.CommunicationFacilitatorInterface;
import java.security.IdentityScope;
import sun.tools.java.ClassPath;
import sun.tools.java.ClassFile;

/**
 * This is a framework to manage the visiting mobile codes in which
 * each one performs a certain task. Typically, a mobile code will
 * query/manipulate the VirtualManagedComponent of network element where
 * the MobileCodeDaemon is running.
 * <p>
 * Knowing VirtualManagedComponent varies from one network element
 * to another and a visiting mobile code do not care how the
 * managed components are implemented,hence, the mobile code daemon
 * should know how to access the managed component by extending 
 * a class <b>VirtualManagedComponent</b>.
 * <p>
 * The mobile code migration is handled by a Code Transfer Protocol
 * that can transport the data either via UDP or TCP.
 *
 * @see mct.users.VirtualManagedComponent
 * @see mct.admin.MobileCodeManager
 * @see mct.net.Ctp
 * @see mct.users.Netlet
 *<p>
 * @version 1.0 3-Nov-96
 * @author Gatot Susilo 
 */
public class MobileCodeDaemon {
	//**********
	//* Fields *
	//**********
	private Listener tcpClient = null;
	private Listener udpClient = null;
	private StorageManager storageManager = null;
	protected MobileCodeManager mobileCodeManager = null;
	private Instantiator instantiator = null;
	private MobileCodeThreadGroup mcdThreadGroup;
	private ClassPath cPath = null;
	private boolean available = false;



//****************
//* Constructors *
//****************
/**
   * The MobileCodeDaemon's constructor
   * @param tcpIP the default host destination for next migration
   * @param tcpPort the port number for the next migration
   * @param udpIp the default host destination for next migration via UDP
   * @param udpPort the port number for the next migration via UDP
   * @param managedObject the managed objects handler
   */
public MobileCodeDaemon(InetAddress tcpIp, int tcpPort, InetAddress udpIp, int udpPort, boolean tcp, boolean enableSecurity) {
	boolean byPassVerifier = !enableSecurity;
	storageManager = new StorageManager();
	mobileCodeManager = new RMIMobileCodeManager(storageManager, byPassVerifier);
	mobileCodeManager.setDefaultShippingDestination(tcpIp, tcpPort, udpIp, udpPort, tcp);
	instantiator = new Instantiator(mobileCodeManager, storageManager
	/* IdentityScope.getSystemScope(), byPassVerifier */);
	mobileCodeManager.setInstantiator(instantiator);
	mcdThreadGroup = new MobileCodeThreadGroup("resident code");
}
public void installFacilitator(String name, String propertiesFile) {
	MobileCodeObject handle = null;
	String facilitatorName = (new MCIdentifier(name)).getID();
	boolean ok = installMobileCode(facilitatorName, propertiesFile);
	if (ok) {
		while ((handle = mobileCodeManager.getMobileCodeHandle(facilitatorName)) == null) {
			// Put a re-try loop instead of an "if" just in case...
			Thread.yield();
		}
		if (handle instanceof CommunicationFacilitatorInterface) {
			mobileCodeManager.setFacilitator((CommunicationFacilitatorInterface) handle);
		} else {
			mct.Console.error(this, "Supplied Communication Facilitator does not implement CommunicationFacilitatorInterface. No messaging provided");
		}
		mct.Console.message(this, "Waiting for Communication Facilitator to become available ....");
		while (!((CommunicationFacilitatorInterface) handle).isAvailable()) {
			// I had to use sleep instead of yield because yield does not
			// seem to guarantee that another thread is scheduled.
			// The sleep period was chosen experimentally : long enough for
			// a task switch, not too long to slow things down.
			try {
				Thread.sleep(150);
			} catch (Exception e) {
			}
		}
		mct.Console.message(this, "Communication Facilitator is installed");
	} else
		mct.Console.error(this, "Failed to install Communication Facilitator");
}
/**
   * install default mobile code that resides on the daemon.
   * The class is typically from local system, not stored in the StorageManager.
   * Use absolute name of mobile code. 
   *
   * May 98 : Code is now instantiated on a separate thread, not on the 
   *	      MobileCodeDaemon's thread.
   *
   * @Returns	True if code is installed with correct properties
   *		False is code is not installed, or if properties not found
   */
public boolean installMobileCode(String name, String propertiesFile) {
	MCIdentifier id = new MCIdentifier(name);
	String canonicalName = id.getID();
	if (propertiesFile != null) {
		try {
			FileInputStream fis = new FileInputStream(propertiesFile);
			Properties prop = new Properties();
			prop.load(fis);
			fis.close();
			return mobileCodeManager.instantiateWithProperties(canonicalName, prop);
		} catch (Exception e) {
			return mobileCodeManager.instantiate(canonicalName);
		}
	} else {
		return mobileCodeManager.instantiate(canonicalName);
	}
}
/**
   * Add default mobile code JARs, so it allows the installed
   * default mobile code to be shipped to the next destination.
   */
protected boolean installMobileCodeJar(String mcName, String jars[]) {
	// CHERYL : Jar code is installed to allow an-already-installed
	//	  class migrate ie. it doesn't make sense to JUST install
	//	  jar files; instead you only install jar files for those
	//	  classes that you have ALREADY installed AND need to migrate
	//
	// Before I realized this coupling, I thought you used class files
	// for stationary code that needed special access and you used
	// jar files for mobile code. In that light, I couldn't understand
	// why jar files were only stored and NOT instantiated.
	// To make them instantiated, you have to :
	//  1) add     mobileCodeManager.addElementToInstantiator (mcc);
	//	as the last statement before the return
	//  2) Change
	//     MobileCodeContainer mcc = storageManager.putContainer(name);
	//     to
	//     MobileCodeContainer mcc = new MobileCodeContainer (name);
	//     to avoid double storage of the container : here and in the
	//	instantiator.
	String name = (new MCIdentifier(mcName)).getID();
	MobileCodeContainer mcc = storageManager.putContainer(name);
	// CHERYL : MobileCodeContainer mcc = new MobileCodeContainer (name);

	if (cPath == null) {
		// Get the classpath for this application
		Properties properties = System.getProperties();
		String classPath = (String) properties.get("java.class.path");
		// Create a class path object (for easy search for class)
		cPath = new ClassPath(classPath);
	}
	ClassFile jarFile = null;
	for (int i = 0; i < jars.length; i++) {
		try {
			File file = new File(jars[i]);
			FileInputStream fis = new FileInputStream(file);
			byte bytes[] = new byte[fis.available()];
			DataInputStream dis = new DataInputStream(fis);
			dis.readFully(bytes);
			fis.close();
			mcc.getJarTable().put(file.getName(), bytes);
		} catch (Throwable e) {
			try {
				jarFile = cPath.getFile(jars[i]);
				if (jarFile == null)
					throw new FileNotFoundException(jars[i]);
				InputStream is = jarFile.getInputStream();
				byte bytes[] = new byte[is.available()];
				is.read(bytes, 0, is.available());
				is.close();
				mcc.getJarTable().put(jarFile.getName(), bytes);
			} catch (Throwable ex) {
				mct.Console.warning(this, "Abort adding mobile code jars due to errors " + jars[i] + "\n" + ex);
				storageManager.removeContainer(name);
				return false;
			}
		}
	}
	return true;
}
/**
 * This method was created by Tony White.
 * Return whether the system is ready to process mobile
 * code transfers and messages.
 * @return boolean
 */
public boolean isAvailable() {
	return available;
}
/**
   * Setup TCP listener with capability to receive more than one clients
   * simulteneously.
   * @param port the port number where the listener listens to
   * @param multiClients true if enable multi-clients receiving
   */
public void tcpListener(int port) {
	tcpClient = new Listener(port, instantiator, true);
	tcpClient.start();
	mobileCodeManager.setTcpPort(port);
}
/**
   * Setup UDP listener, it will receive a connection one at the time
   * @param port the port number where the listener listen to
   */
public void udpListener(int port) {
	udpClient = new Listener(port, instantiator, false);
	udpClient.start();
	mobileCodeManager.setUdpPort(port);
}
} // End class MobileCodeDaemon