package mct.admin;


/**
 * @(#)MobileCodeManager.java     June 2, 1997, Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.net.InetAddress;
import java.lang.SecurityException;
import java.security.PublicKey;
import java.io.*;
import java.security.IdentityScope;
import java.security.PublicKey;
import java.rmi.RMISecurityManager;
import mct.security.MobileCodeVerifier;
import mct.users.*;
import mct.mediator.Message;
import mct.security.*;
import mct.net.Ctp;
import mct.net.CtpException;
import mct.mediator.CommunicationFacilitatorInterface;
import mct.mediator.MobileCodeRecord;

/**
 * MobileCodeManager keeps track the instantiated mobile codes. This class
 * is a core of the daemon and can be accessible only to the mobile code
 * signed by Admin or other users specified in the daemon's configuration file
 * (i.e., netletdaemon.properties) and to the MobileCodeDaemon itself.<p>
 * 
 * This class also provides an interface between the mobile code
 * and MobileCodeDaemon's services such migration facility, accessing the mobile
 * code's parameters, etc. The interface is conducted through MobileCodeStub class.<p>
 *
 * Note: This class replaces ProcessManager class as found in the PMC package.
 *
 * CHERYL : Adding MCTEventMulticaster for event-monitoring
 *	    Correcting netlet life cycle (stop now kills thread)
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.admin.MobileCodeDaemon
 * @see     mct.admin.MobileCodeStub
 * @see     java.security.IdentityScope
 * @since   MCT1.0
 */

public class MobileCodeManager {
	//**********
	//* Fields *
	//**********
	protected StorageManager storageManager;
	// Hold instances of mobile codes except VMC
	//private Hashtable mcInstances;
	// Hold instances of VMC
	//private Hashtable vmcInstances;
	// Hashtable storing all mobile code by type.
	protected Hashtable mobileCodeInstances;
	// 
	private boolean byPassVerifier = false;

	// The listening port
	private int tcpPort = -1;
	private int udpPort = -1;
	//  private int facPort=-1;

	// Default Destination of Migration
	InetAddress ipTCP = null;
	int portTCP = -1;
	InetAddress ipUDP = null;
	int portUDP = -1;
	boolean TCPflag = true;
	protected CommunicationFacilitatorInterface facilitator = null;
	protected InstantiatorInterface instantiator = null;

	// CHERYL : Hook for forwarding internal MCM events to external observers
	protected MCTEventMulticaster eventDispatcher = null;
	protected MobileCodeThreadGroup mcThreadGroup;



/**
   * Constructor of the MobileCodeManager.
   * @param storageManager instance of the StorageManager class.
   * @param byPassVerifier true if you don't want to have a security
   */
public MobileCodeManager(StorageManager storageManager, boolean byPassVerifier) {
	this.storageManager = storageManager;
	this.byPassVerifier = byPassVerifier;
	mobileCodeInstances = new Hashtable();

	// CHERYL : Initailize event watching
	eventDispatcher = new MCTEventMulticaster();
	mcThreadGroup = new MobileCodeThreadGroup("MobileCode");
}
/**
   * Add a MobileCodeContainer to instantiator. The added element
   * will be instantiated automatically.
   */
public void addElementToInstantiator(MobileCodeContainer mcc) {
	if (instantiator != null)
		instantiator.addElement(mcc);
}
/**
   * This will clone a piece of mobile code, including the byte codes.
   */
public MobileCodeObject clone(MobileCodeObject mobileCode, String name) {
	MobileCodeContainer mcc = storageManager.getContainer(mobileCode.getID());
	if (mcc != null) {
		MCIdentifier cloneID = new MCIdentifier(name, mobileCode.getIdentifier().getClassName());
		MobileCodeContainer cloneMCC = mcc.clone(cloneID);
		addElementToInstantiator(cloneMCC);
		return mobileCode;
	}
	return null;
}
/**
   * Call method onDestory() for mobile code implementing Callback interface
   * to let the mobile code release used resources and the mobile code is removed
   * from the daemon. If the mobile code is active, it will be stopped first.
   * We mark the mobile code object as being garbage collectable; i.e. references
   * to it may be removed from the mobile code manager.
   */
public void destroy(MobileCodeObject name) {
	synchronized (name) {
		MobileCodeStatus mcs = getMobileCodeStatus(name);
		if (mcs != null) {
			synchronized (mcs) {
				mcs.beGarbageCollectable();
				if (name instanceof Callback) {
				 	((Callback) name).onDestroy();
				}
				garbageCollect(name);
				// Have to do this last of all as we
				// may be executing this in this thread.
				// This is probably not a good design
				// but requires substantial changes to
				// improve things.
				mcs.stopThread();
			}
		}
	}
}
/**
   * Call garbageCollect for mobile code implementing Callback interface
   * to let the mobile code release used resources and the mobile code is removed
   * from the daemon. If the mobile code is active, it will be stopped first.
   *
   * This method could be called because a thread has exited. If the mobile code
   * is in the stopped state then nothing will happen. If it has migrated or is
   * in the running state all references to it will be removed.
   *
   * WE CANNOT GARBAGE COLLECT A PIECE OF MOBILE CODE THAT IS IN ANY OTHER STATE.
   * IF AN EXCEPTION OCCURS THAT CAUSES THE THREAD TO EXIT, THE MOBILE CODE SHOULD
   * HAVE BE GARBAGE COLLECTABLE (set in beMigrated() and by onException()).
   */
public void garbageCollect(MobileCodeObject name) {
	if (name != null) {
		synchronized (name) {
			MobileCodeStatus mcs = getMobileCodeStatus(name);
			if (mcs != null) {
				synchronized (mcs) {
					if (mcs.isGarbageCollectable()) {
						String sName = name.getID();
						mobileCodeInstances.remove(sName);
						storageManager.removeContainer(sName);
						if (name instanceof CallbackEventListener)
							eventDispatcher.removeCallbackEventListener((CallbackEventListener) name);
						if (name instanceof MCEventListener)
							eventDispatcher.removeMCEventListener((MCEventListener) name);
						if (name instanceof MigratableEventListener)
							eventDispatcher.removeMigratableEventListener((MigratableEventListener) name);
						// Have to deal with MCTGenericEvents differently
						eventDispatcher.removeMCTGenericEventListener(name);
						// Have to cleanup mailboxes...
						// Remove the mailbox if no pieces of mail remain
						// Again, a timeout for messages should be present in order
						// to prevent messages from lingering forever. If the mobile
						// code has migrated, it means that we should keep the mailbox
						// around if messages remain. If we're garbage collecting because
						// of mobile destruction, then we can remove the mailbox.
						facilitator.stopMbx(name.getID(), !mcs.isMigrated());
						// If the status record exists but we are garbage collecting 
						// an unmigrated mobile code it means that the mobile code
						// is disappearing from the system and so we should force
						// local and regional directories to be updated. THIS IS A
						// CRUCIAL PIECE OF CODE FOR MAINTENANCE OF DIRECTORIES.
						if (!mcs.isMigrated())
							name.resetEventNumber();
						eventDispatcher.removeMobileCode(new MCEvent(name, MCEvent.DESTROYED));
					}
				}
			}
		}
	}
}
public MCTEventMulticaster getEventDispatcher(MobileCodeObject name) throws SecurityException {
	if ((name != null) && (name instanceof MCDInterface)) {
		SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null) {
			if (securityManager instanceof MCDSecurityManager) {
				MobileCodeStatus mcs = (MobileCodeStatus) mobileCodeInstances.get(name.getClass().getName());
				MCDSecurityManager msm = (MCDSecurityManager) securityManager;
				msm.checkGetMobileCodeManager(getSigner(name));
				//System.out.println ("Signer:"+mcs.signer);
			} else
				if (securityManager instanceof RMISecurityManager) {
					mct.Console.message(this, "Ignoring RMISecurityManager");
				} else
					if (!(securityManager instanceof MCDNullSecurityManager)) {
						throw new SecurityException();
					}
		}
		return eventDispatcher;
	}
	return null;
}
/**
   * Get the port number at where the tcp socket for facilitator listening.
   */
public int getFacPort() {
	if (facilitator != null) {
		return facilitator.getPort();
	} else {
		return -1;
	}
}
protected String getID(MobileCodeObject name) {
	if (name == null)
		return null;
	return name.getID();
}
/**
 * Return the IP address used for TCP default migration.
 * @return java.net.InetAddress
 */
public InetAddress getIPTCP() {
	return ipTCP;
}
/**
 * Return the IP address used for UDP default migration.
 * @return java.net.InetAddress
 */
public InetAddress getIPUDP() {
	return ipUDP;
}
/**
 * This method exists for compatibility reasons with the 
 * RMIMobileCodeManager. It will always return a single destination
 * as only one is supported by the MobileCodeManager. I do not expect
 * this to be used. This method is synchronized for compatibility with
 * RMIMobileCodeManager only.
 * @return java.net.URL[]
 */
public synchronized java.net.URL[] getMigrationDestinations() {
	java.net.URL[] urlArray = new java.net.URL[1];
	try {
		if (TCPflag) {
			urlArray[0] = new java.net.URL("file:", ipTCP.getHostName(), portTCP, "");
		} else {
			urlArray[0] = new java.net.URL("file:", ipUDP.getHostName(), portUDP, "");			
		}		
	} catch (java.net.MalformedURLException ex) {
		return null;
	}		
	return urlArray;
}
/**
   * It returns the handle of the instantiated mobile code.
   * @param name the name of the mobile code
   */
public MobileCodeObject getMobileCodeHandle(String name) {
	if (name == null)
		return null;
	MobileCodeStatus mcs = (MobileCodeStatus) mobileCodeInstances.get(name);
	if (mcs != null)
		return mcs.mobileCode;
	return null;
}
/**
   * Get list of the Mobile Code instances installed on the daemon of a given type.
   *
   */
public Enumeration getMobileCodeList(String type) {
	Vector mcInstances = new Vector();
	Enumeration enum = mobileCodeInstances.elements();
	while (enum.hasMoreElements()) {
		MobileCodeStatus mcs = (MobileCodeStatus) enum.nextElement();
		if (mcs.mobileCode.getType().equals(type))
			mcInstances.addElement(mcs.mobileCode.getIdentifier());
	}
	return mcInstances.elements();
}
// Handle mobile code implementing MCDExtension interface
/**
   * Get handle of the MobileCodeManager of the daemon. The mobile code
   * must implement MCDExtension. It throws SecurityException if the mobile
   * code is not signed by the proper signer such as Admin, or other users
   * specified in the daemon configuration file (i.e., netletdaemon.properties).
   * Unless the security feature is disabled.
   *
   * @see mct.NetletDaemon
   * @see mct.users.MCDExtension
   * @see mct.security.MCDSecurityManager
   */
public MobileCodeManager getMobileCodeManager(MobileCodeObject name) throws SecurityException {
	if (name instanceof MCDInterface) {
		SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null) {
			if (securityManager instanceof MCDSecurityManager) {
				MobileCodeStatus mcs = (MobileCodeStatus) mobileCodeInstances.get(name.getClass().getName());
				MCDSecurityManager msm = (MCDSecurityManager) securityManager;
				msm.checkGetMobileCodeManager(getSigner(name));
				//System.out.println ("Signer:"+mcs.signer);
			} else
				if (securityManager instanceof RMISecurityManager) {
					mct.Console.message(this, "Ignoring RMISecurityManager");
				} else
					if (!(securityManager instanceof MCDNullSecurityManager)) {
						throw new SecurityException();
					}
		}
		return this;
	}
	return null;
}
protected MobileCodeStatus getMobileCodeStatus(MobileCodeObject name) {
	if (name == null)
		return null;
	return (MobileCodeStatus) mobileCodeInstances.get(getID(name));
}
/**
   * Get parameter for the mobile code. The mobile code must implement
   * Parameters interface. It returns null if the parameter is not found
   * or the mobile code does not implement Parameters. The objects will be transported through serialization;
   * therefore they must implement interface java.io.Serialization.
   * @see mct.users.Parameters
   */
public Object getParameter(MobileCodeObject name, Object parName) {
	if (name instanceof Parameters) {
		MobileCodeContainer mcc = storageManager.getContainer(getID(name));
		if (mcc != null)
			return mcc.getParameter(parName);
	}
	return null;
}
/**
   * Get a list of parameter keys for the mobile code. The mobile code must implement
   * Parameters interface. It returns null if the parameter is not found
   * or the mobile code does not implement Parameters. 
   * @see mct.users.Parameters
   */
public Enumeration getParameterNames(MobileCodeObject name) {
	if (name instanceof Parameters) {
		MobileCodeContainer mcc = storageManager.getContainer(getID(name));
		if (mcc != null)
			return mcc.getParameterNames();
	}
	return null;
}
/**
   * Get list of parameter values for the mobile code. The mobile code must implement
   * Parameters interface. It returns null if the parameter is not found
   * or the mobile code does not implement Parameters.
   * @see mct.users.Parameters
   */
public Enumeration getParameterValues(MobileCodeObject name) {
	if (name instanceof Parameters) {
		MobileCodeContainer mcc = storageManager.getContainer(getID(name));
		if (mcc != null)
			return mcc.getParameterValues();
	}
	return null;
}
private PublicKey getSigner(MobileCodeObject mobileCode) {
	String name = getID(mobileCode);
	if (name != null) {
		MobileCodeContainer mcc = storageManager.getContainer(name);
		if (mcc != null)
			return mcc.getSignerPublicKey();
	}
	return null;
}
/**
   * Get handle of the StorageManager of the daemon. The mobile code
   * must implement MCDExtension. It throws SecurityException if the mobile
   * code is not signed by the proper signer such as Admin, or other users
   * specified in the daemon configuration file (i.e., netletdaemon.properties).
   * Unless the security feature is disabled.
   *
   * @see mct.NetletDaemon
   * @see mct.users.MCDExtension
   * @see mct.security.MCDSecurityManager
   */
public StorageManager getStorageManager(MobileCodeObject name) throws SecurityException {
	if ((name == null) && !(name instanceof MCDInterface))
		return null;
	SecurityManager securityManager = System.getSecurityManager();
	if (securityManager != null) {
		if (securityManager instanceof MCDSecurityManager) {
			//MobileCodeStatus mcs = (MobileCodeStatus) mobileCodeInstances.get(name.getClass().getName());
			MCDSecurityManager msm = (MCDSecurityManager) securityManager;
			msm.checkGetMobileCodeManager(getSigner(name));
			//System.out.println ("Signer: "+mcs.signer);
		} else
			if (securityManager instanceof RMISecurityManager) {
				mct.Console.message(this, "Ignoring RMISecurityManager");
			} else
				if (!(securityManager instanceof MCDNullSecurityManager)) {
					throw new SecurityException();
				}
	}
	return storageManager;
}
/**
   * Get the port number at where the tcp socket for CTP listening.
   */
public int getTcpPort() {
	return tcpPort;
}
/**
 * This method was created by a SmartGuide.
 * @return mct.admin.MobileCodeThreadGroup
 */
public MobileCodeThreadGroup getThreadGroup() {
	return mcThreadGroup;
}
/**
   * Get the port number at where the udp socket for CTP listening.
   */
public int getUdpPort() {
	return udpPort;
}
// Handle mobile code implementing VMCAccess interface
/**
   * Get handle of the VirtualManagedComponent installed on the daemon. The mobile code
   * must implement VMCAccess. It returns null if the VMC is not found or the mobile code
   * does not implement VMCAccess
   *
   * @see mct.users.VMCAccess
   */
public MobileCodeObject getVirtualManagedComponent(MobileCodeObject mobileCode, String name, boolean className) {
	if (mobileCode instanceof VMCAccess) {
		String aliasName;
		if (className) {
			Enumeration enum = mobileCodeInstances.elements();
			while (enum.hasMoreElements()) {
				MobileCodeStatus mcs = (MobileCodeStatus) enum.nextElement();
				if (className)
					aliasName = mcs.mobileCode.getClass().getName();
				else
					aliasName = mcs.mobileCode.getAlias();
				if (aliasName.equals(name))
					return mcs.mobileCode;
			}
		}
	}
	return null;
}
/**
   * Get list of the VirtualManagedComponent installed on the daemon. The mobile code
   * must implement VMCAccess. It returns null if the VMC is not found or the mobile code
   * does not implement VMCAccess
   *
   * @see mct.users.VMCAccess
   */
public Enumeration getVirtualManagedComponentList(MobileCodeObject mobileCode) {
	if (mobileCode instanceof VMCAccess) {
		Vector vmcInstances = new Vector();
		Enumeration enum = mobileCodeInstances.elements();
		while (enum.hasMoreElements()) {
			MobileCodeStatus mcs = (MobileCodeStatus) enum.nextElement();
			if (mcs.mobileCode instanceof VMCMobileCode)
				vmcInstances.addElement(mcs.mobileCode.getIdentifier());
		}
		return vmcInstances.elements();
	}
	return null;
}
/**
   * To test whether a piece of mobile code has any messages in its
   * mailbox.
   */
public boolean hasMessages(MobileCodeObject mobileCode) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				boolean msgs = ((CommunicationFacilitatorInterface) facilitator).hasMessages(mobileCode.getID());
				mcs.beRunning();
				return msgs;
			} catch (Throwable ex) {
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				};
			}
		}
	}
	return false;
}
// Handle mobile code implementing Callback interface
/**
   * Call method onInit() for the mobile code implementing Callback interface
   * to let the mobile code initialize itself.
   */
public void init(MobileCodeObject name) {
	if ((name != null) && (name instanceof Callback))
		 ((Callback) name).onInit();
}
/**
   * It instantiates the mobile code whose name is specified in the parameter.
   * It seeks the mobile code in the storage manager. If it founds
   * the mobile code in the storage manager, it uses MctClassLoader to instantiate
   * the mobile code. Otherwise, the mobile code is assumed from local system and instantiated
   * by using Class.forName(name).newInstance(). The class' constructor of mobile code must not
   * have parameter(s) because it is instantiated by using Class.newInstance().
   * If the mobile code is not found in the local system, the instantiation is cancelled.<p>
   *
   * As soon as the mobile code is instantiated and the MobileCodeDaemon (MCD) has
   * Facilitator installed (i.e., facilitator!=null), the mobile code will be registered
   * on Facilitator (local database) and on Mediator (central database).<p>
   * 
   * The latter is usefull for the initial mobile code that must be instantiated
   * as soon as the daemon is started.<p>
   *
   * It should be noted that every mobile code must implement interface MobileCode
   * and every virtual managed component must implement interface VMCMobileCode which
   * is subclass of interface MobileCode.
   *
   * @param name the absolute class name of the mobile code
   * @see mct.users.MobileCode
   * @see mct.users.VMCMobileCode
   * @see mct.mediator.Facilitator
   * @see mct.mediator.Mediator
   */
public boolean instantiate(String name) {
	return instantiateWithProperties(name, null);
}
/**
   * It instantiates the mobile code whose name is specified in the parameter.
   * It seeks the mobile code in the storage manager. If it founds
   * the mobile code in the storage manager, it uses MctClassLoader to instantiate
   * the mobile code. Otherwise, the mobile code is assumed from local system and instantiated
   * by using Class.forName(name).newInstance(). The class' constructor of mobile code must not
   * have parameter(s) because it is instantiated by using Class.newInstance().
   * If the mobile code is not found in the local system, the instantiation is cancelled.<p>
   *
   * As soon as the mobile code is instantiated and the MobileCodeDaemon (MCD) has
   * Facilitator installed (i.e., facilitator!=null), the mobile code will be registered
   * on Facilitator (local database) and on Mediator (central database).<p>
   * 
   * The latter is usefull for the initial mobile code that must be instantiated
   * as soon as the daemon is started.<p>
   *
   * It should be noted that every mobile code must implement interface MobileCode
   * and every virtual managed component must implement interface VMCMobileCode which
   * is subclass of interface MobileCode.
   *
   * @param name the absolute name of the mobile code. The name includes
   *	an identity as well as the class name. The @ symbol is used to delimit
   * the mobile code identifier and the 
   * @see mct.users.MobileCode
   * @see mct.users.VMCMobileCode
   * @see mct.mediator.Facilitator
   * @see mct.mediator.Mediator
   */
private boolean instantiateUsingClassLoader(String name, Properties properties) {
	MobileCodeContainer mcc = null;
	boolean persistent = false;
	MobileCodeObject mobileCode = null;
	String className = null;

	// Handle mobile code sent from network and stored in the StorageManager
	if ((mcc = storageManager.getContainer(name)) != null) {
		//System.out.println ("Instiate class in the storageManager");
		try {
			className = MCIdentifier.getClassFromIdentifier(name);
			MctClassLoader mcl = new MctClassLoader(storageManager, name);
			Object obj = null;
			// The mobile code does not have persistent state
			if (!mcc.hasPersistentState()) {
				Class c = mcl.loadClass(className, true);
				if (c == null)
					return false;
				obj = c.newInstance();
				// The mobile code has persistent state
			} else {
				persistent = true;
				ReadObjectStream ois = new ReadObjectStream(mcl, new ByteArrayInputStream(mcc.getPersistentState()));
				obj = ois.readObject();
			}
			if ((obj != null) && (obj instanceof MobileCode)) {
				mobileCode = (MobileCodeObject) obj;
			} else {
				obj = null;
				mct.Console.warning(this, "The mobile code does not implement MobileCode");
				return false;
			}
			// TBD 1 : If registerMobileCode returns false, there
			//	   should be garbage collection to remove the mobileCode
			return registerMobileCode(name, mobileCode, properties);
		} catch (ClassNotFoundException e) {
			mct.Console.warning(this, "Class not found: " + e);
		} catch (InstantiationException e) {
			mct.Console.warning(this, "Fail to instantiate class " + name);
		} catch (IllegalAccessException e) {
			mct.Console.warning(this, "I have no access to your class " + name);
		} catch (Throwable e) {
			mct.Console.warning(this, "Exception occurs");
			e.printStackTrace();
		}
	}
	return false;
}
/**
   * It instantiates the mobile code whose name is specified in the parameter.
   * It seeks the mobile code in the storage manager. If it founds
   * the mobile code in the storage manager, it uses MctClassLoader to instantiate
   * the mobile code. Otherwise, the mobile code is assumed from local system and instantiated
   * by using Class.forName(name).newInstance(). The class' constructor of mobile code must not
   * have parameter(s) because it is instantiated by using Class.newInstance().
   * If the mobile code is not found in the local system, the instantiation is cancelled.<p>
   *
   * As soon as the mobile code is instantiated and the MobileCodeDaemon (MCD) has
   * Facilitator installed (i.e., facilitator!=null), the mobile code will be registered
   * on Facilitator (local database) and on Mediator (central database).<p>
   * 
   * The latter is usefull for the initial mobile code that must be instantiated
   * as soon as the daemon is started.<p>
   *
   * It should be noted that every mobile code must implement interface MobileCode
   * and every virtual managed component must implement interface VMCMobileCode which
   * is subclass of interface MobileCode.
   *
   * @param name the absolute class name of the mobile code
   * @see mct.users.MobileCode
   * @see mct.users.VMCMobileCode
   * @see mct.mediator.Facilitator
   * @see mct.mediator.Mediator
   */
private boolean instantiateUsingClasspath(String name, Properties properties) {
	MobileCodeContainer mcc = null;
	boolean persistent = false;
	MobileCodeObject mobileCode = null;
	String className = null;

	// Instantiate mobile code that resides on CLASSPATH
	try {
		className = MCIdentifier.getClassFromIdentifier(name);
		Class c = Class.forName(className);
		if (c == null)
			return false;
		Object obj = c.newInstance();
		if ((obj == null) || !(obj instanceof MobileCode)) {
			mct.Console.warning(this, "The mobile code does not implement MobileCode interface");
			return false;
		}
		mobileCode = (MobileCodeObject) obj;
		// TBD 1 : If registerMobileCode returns false, there
		//	   should be garbage collection to remove the mobileCode
		return registerMobileCode(name, mobileCode, properties);
	} catch (ClassNotFoundException e) {
		mct.Console.warning(this, "Class not found: " + e);
	} catch (InstantiationException e) {
		mct.Console.warning(this, "Fail to instantiate class " + name);
	} catch (IllegalAccessException e) {
		mct.Console.warning(this, "I have no access to your class " + name);
	} catch (Throwable e) {
		mct.Console.warning(this, "Exception occurs: " + e);
		e.printStackTrace();
	}
	return false;
}
/**
   * It instantiates the mobile code whose name is specified in the parameter.
   * It seeks the mobile code in the storage manager. If it founds
   * the mobile code in the storage manager, it uses MctClassLoader to instantiate
   * the mobile code. Otherwise, the mobile code is assumed from local system and instantiated
   * by using Class.forName(name).newInstance(). The class' constructor of mobile code must not
   * have parameter(s) because it is instantiated by using Class.newInstance().
   * If the mobile code is not found in the local system, the instantiation is cancelled.<p>
   *
   * As soon as the mobile code is instantiated and the MobileCodeDaemon (MCD) has
   * Facilitator installed (i.e., facilitator!=null), the mobile code will be registered
   * on Facilitator (local database) and on Mediator (central database).<p>
   * 
   * The latter is usefull for the initial mobile code that must be instantiated
   * as soon as the daemon is started.<p>
   *
   * It should be noted that every mobile code must implement interface MobileCode
   * and every virtual managed component must implement interface VMCMobileCode which
   * is subclass of interface MobileCode.
   *
   * @param name the absolute class name of the mobile code
   * @see mct.users.MobileCode
   * @see mct.users.VMCMobileCode
   * @see mct.mediator.Facilitator
   * @see mct.mediator.Mediator
   */
public boolean instantiateWithProperties(String name, Properties properties) {
	boolean result;
	if (!(result = instantiateUsingClassLoader(name, properties)))
		result = instantiateUsingClasspath(name, properties);
	return result;
}
/**
   * Check if the mobile code is active.
   * @see #start
   * @see #stop
   */
public boolean isActive(MobileCodeObject name) {
	if (name instanceof Callback) {
		MobileCodeStatus status = getMobileCodeStatus(name);
		if (status != null)
			return status.isRunning();
		else
			return false;
	}
	return false;
}
// Handle mobile code implementing Migratable interface
/**
   * Call method onMigrate() for mobile code implementing Migratable interface to
   * let the mobile code know that it is about to be migrated. At this point,
   * the mobile code may refuse to migrate or allow to migrate with or without
   * persistent states. If the mobile code allows to migrate, it is migrated
   * to the next destination as sprecified in the parameters.<p>
   *
   * It should be noted that the migration with persistent state is done
   * by using serialization. Therefore all non-transient objects must
   * implement java.io.Serializable.
   *
   * @param name the instance of mobile code that will be migrated
   * @param ip the ip address of destination
   * @param port the port of destination
   * @param flag true if migration via TCP; false if migration via UDP
   */
public void migrate(MobileCodeObject name, InetAddress ip, int port, boolean flag) {
	MobileCodeStatus mcs = getMobileCodeStatus(name);
	if (mcs != null && (name instanceof Migratable)) {
		// Set up migration destination stuff
		InetAddress destIp = ip;
		int destPort = port;
		if (ip == null) {
			if (flag == true) {
				destIp = ipTCP;
				destPort = portTCP;
			} else {
				destIp = ipUDP;
				destPort = portUDP;
			}
		}
		// Only allow migration if we're not talking or doing something else
		// It is not possible to migrate while migrating, for example
		if (mcs.isRunning()) {
			mcs.beMigrating();
			try {
				int retValue = ((Migratable) name).onMigrate();
				MobileCodeContainer mcc = storageManager.getContainer(getID(name));
				ByteArrayOutputStream baos = null;
				if (retValue == Migratable.DONT_MIGRATE) {
					return;
				} else
					if (retValue == Migratable.SERIALIZE) {
						baos = new ByteArrayOutputStream();
						ObjectOutputStream oos = new ObjectOutputStream(baos);
						oos.writeObject(name);
					}
				boolean migrationFlag = mcc.getMigrationFlag();
				mcc.setMigrationFlag();
				Ctp ctp = null;
				// Start migrating now
				try {

					/**** BAD CODE HERE ********
					    Ideally, the MIGRATED event should be sent AFTER the code is sent
					    so that the event is sent IF AND ONLY IF the migration is successful.
					
					    Practically, when I sent the event AFTER the code is migrated,
					    the netlet would sometimes startup (and send its STARTUP event)
					    before the MIGRATED event arrived... and the mediator's database
					    would be out-of-order.
					
					    Putting the event BEFORE the migration is a TEMPORARY fix.
					    We must now look at either	
					    1) ordering the events at the mediator
					    2) expanding the number of states maintained by the mediator
					       to account for this "mis-ordering"
					       **********************/
					name.nextEvent();
					eventDispatcher.migrated(new MigratableEvent(name, MigratableEvent.MIGRATED));

					// Send the bytecodes and other bits and pieces.
					ctp = new Ctp(destIp, destPort, flag);
					if (baos != null)
						ctp.send(mcc, baos.toByteArray());
					else
						ctp.send(mcc, null);
					ctp.close();

					// Mark the mobile code as having migrated successfully and
					// destroy all references to it within this MCD. The mobile code threads
					// that we know about are destroyed.

					mcs.beMigrated();
					destroy(name);
				} catch (IOException ex) {
					if (ctp != null)
						ctp.close();
					mct.Console.warning(null, name.getID() + " had I/O problem");
					if (!migrationFlag)
						mcc.clearMigrationFlag();
					mcs.beRunning();
					((Migratable) name).onFailMigrate(destIp, destPort, flag);
					eventDispatcher.netletInitialized(new CallbackEvent(name, CallbackEvent.INITIALIZED));
				} catch (CtpException ex) {
					if (ctp != null)
						ctp.close();
					mct.Console.warning(null, name.getID() + " had CTP problem");
					if (!migrationFlag)
						mcc.clearMigrationFlag();
					mcs.beRunning();
					((Migratable) name).onFailMigrate(destIp, destPort, flag);
					eventDispatcher.netletInitialized(new CallbackEvent(name, CallbackEvent.INITIALIZED));
				}
			} catch (java.lang.ThreadDeath e) {
				// Have to rethrow this in order to ensure that threads are 
				// tidied up properly and zombie threads don't appear.
				throw e;
			} catch (Throwable e) {
				mct.Console.warning(this, "Exception occurs " + e);
				e.printStackTrace();
			}
		} else { // Failed because we were communicating or something
		 	((Migratable) name).onFailMigrate(destIp, destPort, flag);
		}
	}
}
/**
 * This method ensures backward migration capability with the 
 * old mobile code manager. Do nothing, other than post a warning.
 * Migration fails.
 * @param mobileCode mct.users.MobileCodeObject
 * @param ip java.net.InetAddress
 * @param destination mct.users.MCIdentifier
 */
public void migrate(MobileCodeObject mobileCode, InetAddress ip, MCIdentifier destination) {
	mct.Console.error(this, " cannot use RMI migration for "+mobileCode.getID()+" use mct.admin.RMIMobileCodeManager");
	return;
}
//Handle mobile code implementing MessagingAccess interface
/**
   * Notify a mobile code implementing MessagingAccess interface
   * that the a new message arrived.
   */
public void onMessage(MobileCode mobileCode) {
	if (mobileCode instanceof MessagingAccess)
		 ((MessagingAccess) mobileCode).onMessage();
}
/**
   * To retrieve message(s) non-destructively. This method is usually called after
   * the new message notification is raised by calling method onMessage.
   */
public Message peek(MobileCodeObject mobileCode) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				Message msg = ((CommunicationFacilitatorInterface) facilitator).peek(mobileCode.getID());
				mcs.beRunning();
				return msg;
			} catch (Throwable ex) {
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				};
			}
		}
	}
	return null;
}
/**
   * Put a mobile code container in the storage manager.
   * You cannnot add directly to the storage manager because of its protected method.
   * The mobile code container will be verified if the security feature is enabled.
   * @throw SecurityException if the MobileCodeContainer did not pass verifier. 
   */
public void putMobileCodeContainer(MobileCodeContainer mcc) {
	if (verifyMobileCodeContainer(mcc)) {
		storageManager.putContainer(mcc);
	} else
		throw new SecurityException("MobileCodeContainer cannot be verified");
}
// Handle mobile code implementing Parameters interface
/**
   * Put parameter for the mobile code. The mobile code must implement
   * Parameters interface. The objects will be transported through serialization;
   * therefore they must implement interface java.io.Serialization.
   * @see mct.users.Parameters
   */
public void putParameter(MobileCodeObject name, Object parName, Object parValue) {
	if (name instanceof Parameters) {
		MobileCodeContainer mcc = storageManager.getContainer(getID(name));
		if (mcc != null)
			mcc.putParameter(parName, parValue);
	}
}
/**
   * To retrieve message(s). This method is usually called after
   * the new message notification is raised by calling method onMessage.
   */
public Message receive(MobileCodeObject mobileCode, long timer) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				Message msg = ((CommunicationFacilitatorInterface) facilitator).receive(mobileCode.getID(), timer);
				mcs.beRunning();
				return msg;
			} catch (Throwable ex) {
				mct.Console.warning(this, mobileCode.getID()+" has problem in receive: "+ex);
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				};
			}
		}
	}
	return null;
}
public Message[] receiveAll(MobileCodeObject mobileCode, long timer) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				Message[] msg = ((CommunicationFacilitatorInterface) facilitator).receiveAll(mobileCode.getID(), timer);
				mcs.beRunning();
				return msg;
			} catch (Throwable ex) {
				mct.Console.warning(this, mobileCode.getID() + " has problem in receiveAll: " + ex);
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				};
			}
		}
	}
	return null;
}
/*
   * @param properties Properties the properties to be given to the mobile code 
   * @param mobileCodeObject MobileCode the mobile code being registered
   * @param name String the name of the mobile code
   */
protected synchronized boolean registerMobileCode(String name, MobileCodeObject mobileCode, Properties properties) {
	MobileCodeStub stub = null;
	try {
		// Create an identifier for the mobile code based upon name
		// Really this should already have been done.
		mobileCode.setIdentifier(new MCIdentifier(name));

		// Ensure that the mobile code has a stub associated with it.
		// The stub is absolutely essential to the well being of the mobile code object
		mobileCode.setStub(new MobileCodeStubImplementation(this, mobileCode, storageManager));

		// The stub has been initialized and so we send the initialized event
		// to all interested parties.
		mobileCode.nextEvent();
		eventDispatcher.addMobileCode(new MCEvent(mobileCode, MCEvent.INSTALLED));

		// If we have migrated, we use onRestore. Otherwise, we use onInit.
		boolean callInit = true;
		if (mobileCode instanceof Migratable) {
			MobileCodeContainer mcc = storageManager.getContainer(mobileCode.getID());
			if (mcc != null && mcc.hasMigrated()) {
				callInit = false;
				if (!((Migratable) mobileCode).onRestore()) {
					mct.Console.warning(this, "Restoration failed for  " + mobileCode);
					return false; // Abort registration due to restoration errors
				}
			}
		}
		if (callInit) {
			if (!mobileCode.onInit(properties)) {
				mct.Console.warning(this, "Instantiation failed for  " + mobileCode);
				return false; // Abort registration due to initialization errors
			}
		}
		mobileCode.nextEvent();
		eventDispatcher.netletInitialized(new CallbackEvent(mobileCode, CallbackEvent.INITIALIZED));

		// At this point, the code is installed, and 
		// other netlets can get this mobileCode's handle.
		// Mark this event (done in start(mobileCode)).

		mobileCodeInstances.put(name, new MobileCodeStatus(mobileCode));
		start(mobileCode);

		// Set up messaging access for the mobile code
		// This involves accessing the mailbox to allow callbacks
		// to occur on the mobile code.
		
		if (facilitator instanceof mct.mediator.CommunicationFacilitatorInterface)
			facilitator.startMbx(mobileCode.getID());
		return true;
	} catch (Throwable e) {
		mct.Console.warning(this, "Exception occurs during registration: " + e);
		e.printStackTrace();
	}
	return false;
}
/**
   * Remove parameter for the mobile code. The mobile code must implement 
   * Parameters interface. It returns null if the parameter is not found
   * or the mobile code does not implement Parameters. 
   * The objects will be transported through serialization;
   * therefore they must implement interface java.io.Serialization.
   * @see mct.users.Parameters
   */
public Object removeParameter(MobileCodeObject name, Object parName) {
	if (name instanceof Parameters) {
		MobileCodeContainer mcc = storageManager.getContainer(getID(name));
		if (mcc != null)
			return mcc.removeParameter(parName);
	}
	return null;
}
/**
   * This method is called by the Instantiator to let the existing mobile code
   * know that it will be replaced with the new one. So it stops the existing one
   * and let a new mobile code to be instantiated.<p>
   */
public boolean replace(String name) {
	MobileCodeObject mc = getMobileCodeHandle(name);
	if (mc == null)
		return true;
	if (mc instanceof Callback)
		destroy(mc);
	return true;
}
/**
   * Call method onRestore() the for mobile code implementing Callback interface to
   * let the mobile code initialize itself as a result of deserialization.
   */
public void restore(MobileCodeObject name) {
	if ((name != null) && (name instanceof Migratable))
		 ((Migratable) name).onRestore();
}
/**
   * Call method onStop() for mobile code implementing Callback interface to
   * stop executing and the mobile code is marked not active.
   */
public void resume(MobileCodeObject name) {
	synchronized (name) {
		if (name instanceof Callback) {
			MobileCodeStatus mcs = getMobileCodeStatus(name);
			if (mcs != null)
				if (mcs.isSuspended()) {
					mcs.beRunning();
					eventDispatcher.netletResumed(new CallbackEvent(name, CallbackEvent.RESUMED));
				}
		}
	}
}
/**
   * A mobile code implementing interface MessagingAccess may send a
   * message to other mobile code. Can only send a message if I'm running.
   * For example, I can't send a message when I'm migrating!
   */
public boolean send(MobileCodeObject mobileCode, Message mesg) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				boolean status = ((CommunicationFacilitatorInterface) facilitator).send(mesg, false);
				mcs.beRunning();
				return status;
			} catch (Throwable ex) {
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				};
			}
		}
	}
	return false;
}
/**
   * A mobile code implementing interface MessagingAccess may send a
   * message to other mobile code. Can only send a message if I'm running.
   * For example, I can't send a message when I'm migrating!
   */
public boolean sendAsync(MobileCodeObject mobileCode, Message mesg) {
	if (mobileCode instanceof MessagingAccess) {
		MobileCodeStatus mcs = getMobileCodeStatus(mobileCode);
		if ((mcs != null) && mcs.isRunning() && (facilitator instanceof CommunicationFacilitatorInterface)) {
			try {
				mcs.beCommunicating();
				boolean status = ((CommunicationFacilitatorInterface) facilitator).sendAsync(mesg, false);
				mcs.beRunning();
				return status;
			} catch (Throwable ex) {
				mcs.beRunning();
				try {
					throw ex;
				} catch (java.lang.Throwable tex) {
				}
			}
		}
	}
	return false;
}
/**
   * Set the default migration destination for the mobile code.
   * @param ipTCP the ip of the next destination via TCP
   * @param portTCP the port number at which the socket is listening
   * @param ipUDP the ip of the next destination via UDP
   * @param portUDP the port number at which the DatagramSocket is listening
   * @param flag true if the default protocol is TCP otherwise false for UDP
   */
public void setDefaultShippingDestination(InetAddress ipTCP, int portTCP, InetAddress ipUDP, int portUDP, boolean flag) {
	this.ipTCP = ipTCP;
	this.portTCP = portTCP;
	this.ipUDP = ipUDP;
	this.portUDP = portUDP;
	this.TCPflag = flag;
}
/**
   * Facilitator can be set only once, typically during initialization of
   * the mobile code daemon.
   */
public boolean setFacilitator(CommunicationFacilitatorInterface facilitator) {
	if (this.facilitator == null) {
		this.facilitator = facilitator;
		mct.Console.message(this, "Facilitator is installed.");
		return true;
	} else {
		mct.Console.message(this, "Facilitator can be set only once...");
		return false;
	}
}
/**
   * Pass the information of listening port number for Facilitator (TCP).
   */
public void setFacPort(int facPort) {
	if (facilitator != null) {
		facilitator.setPort(facPort);
	}
}
public void setInstantiator(InstantiatorInterface instantiator) {
	if (this.instantiator == null) {
		this.instantiator = instantiator;
		mct.Console.message(this, "Instantiator is installed.");
	} else
		mct.Console.message(this, "Instantiator can be set only once...");
}
/**
   * Pass the information of listening port number for TCP.
   */
public void setTcpPort(int tcpPort) {
	this.tcpPort = tcpPort;
}
/**
   * Pass the information of listening port number for UDP.
   */
public void setUdpPort(int udpPort) {
	this.udpPort = udpPort;
}
/**
   * Call method onStart() for the mobile code implementing Callback interface to
   * determine the mobile code to start executing and the mobile code is marked active.
   * Either the MobileCodeRunner or the MethodRunnerThread classes can be used in order
   * to start the thread for the mobile code. The latter uses reflection in order to 
   * allow execution of any method as the start point for the mobile code. Hmm, perhaps
   * we could pass this information with the mobile code and have it use different 
   * start points depending upon context....
   */
public void start(MobileCodeObject name) {
	synchronized (name) {
		if (name instanceof Callback) {
			MobileCodeStatus mcs = getMobileCodeStatus(name);
			if (mcs != null)
				synchronized (mcs) {
				if (!mcs.hasThread()) {
					mcs.beRunning();
					Thread thread = new MethodRunnerThread(name, "onStart", new Object[0], new MCThreadExceptionHandler(this, storageManager), mcs.getThreadGroup());
					mcs.setThread(thread);

					// We don't have to call onStart, it will be done by either the 
					// MobileCodeRunner or MethodRunnerThread
					// ((Callback) name).onStart();

					eventDispatcher.netletStarted(new CallbackEvent(name, CallbackEvent.STARTED));
				}
			}
		}
	}
}
/**
   * Call method onStop() for mobile code implementing Callback interface to
   * stop executing and the mobile code is marked not active.
   */
public void stop(MobileCodeObject name) {
	synchronized (name) {
		if (name instanceof Callback) {
			MobileCodeStatus mcs = getMobileCodeStatus(name);
			if (mcs != null)
				synchronized (mcs) {
				if (mcs.hasThread()) {
					((Callback) name).onStop();
					eventDispatcher.netletStopped(new CallbackEvent(name, CallbackEvent.STOPPED));
					mcs.beStopped();
				}
			}
		}
	}
}
/**
   * Call method onStop() for mobile code implementing Callback interface to
   * stop executing and the mobile code is marked not active.
   */
public void suspend(MobileCodeObject name) {
	if (name instanceof Callback) {
		MobileCodeStatus mcs = getMobileCodeStatus(name);
		if (mcs != null)
			if (mcs.isRunning()) {
				eventDispatcher.netletSuspended(new CallbackEvent(name, CallbackEvent.SUSPENDED));
				mcs.beSuspended();
			}
	}
}
/**
   * Verify the MobileCodeContainer. If you modify the content (JARs) of 
   * MobileCodeContainer, this method should be called to verify the
   * modified content. The modifiable content (JAR) will be verified
   * and inflated into number of classes that comprise it.
   */
public boolean verifyMobileCodeContainer(MobileCodeContainer mcc) {
	IdentityScope identityScope = IdentityScope.getSystemScope();
	if ((identityScope == null) && (!byPassVerifier))
		return false;
	MobileCodeVerifier mcv = new MobileCodeVerifier(mcc, identityScope);
	if (mcv.verify(byPassVerifier)) {
		mcc.putClassTable(mcv.getClassTable());
		if (mcv.getSigners() != null) {
			if (!mcv.getSigners().isEmpty())
				mcc.putSignerPublicKey((PublicKey) mcv.getSigners().firstElement());
		}
		return true;
	}
	return false;
}
}