package mct.admin;


/**
 * @(#)MobileCodeStatus.java     June 2, 1997, Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import mct.users.MobileCodeObject;
class MobileCodeStatus {
	public static final int ST_UNKNOWN = 0;
	public static final int ST_RUNNING = 1;
	public static final int ST_SUSPENDED = 2;
	public static final int ST_MIGRATING = 3;
	public static final int ST_MIGRATED = 4;
	public static final int ST_COMMUNICATING = 5;
	public static final int ST_STOPPED = 6;

	// Fields for the status object
	protected int state = ST_UNKNOWN;
	protected boolean gc = false;
	protected MobileCodeObject mobileCode = null;
	private transient Thread thread = null;
	private transient ThreadGroup threadGroup = null;



/**
 * This method was created by a SmartGuide.
 * @param object mct.users.MobileCodeObject
 */
public MobileCodeStatus(MobileCodeObject object) {
	mobileCode = object;
	threadGroup = new ThreadGroup(mobileCode.getID());
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beCommunicating() {
	boolean ok = state != ST_COMMUNICATING;
	if (ok) {
		state = ST_COMMUNICATING;
	}
	return ok;
}
/**
 * This marks the status record as representing a garbage collectable
 * piece of mobile code.
 *
 * @return boolean
 */
public boolean beGarbageCollectable() {
	gc = true;
	return gc;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beMigrated() {
	boolean ok = state != ST_MIGRATED;
	if (ok)
		state = ST_MIGRATED;
	beGarbageCollectable();
	return ok;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beMigrating() {
	boolean ok = state != ST_MIGRATED && state != ST_MIGRATING;
	if (ok)
		state = ST_MIGRATING;
	return ok;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beRunning() {
	boolean ok = state != ST_MIGRATED && state != ST_RUNNING;
	if (ok) {
		state = ST_RUNNING;
		resumeThread();
	}
	return ok;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beStopped() {
	boolean ok = state != ST_STOPPED && state != ST_MIGRATED;
	if (ok) {
		state = ST_STOPPED;
		stopThread();
	}
	return ok;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beSuspended() {
	boolean ok = state != ST_MIGRATED && state != ST_SUSPENDED;
	if (ok) {
		state = ST_SUSPENDED;
		suspendThread();
	}
	return ok;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
protected Thread getThread() {
	return thread;
}
/**
 * Accesses the thread group for this threaded piece of mobile code.
 * @return java.lang.ThreadGroup
 */
public ThreadGroup getThreadGroup() {
	return threadGroup;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean hasThread() {
	return thread instanceof Thread;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isCommunicating() {
	return state == ST_COMMUNICATING;
}
/**
 * This method returns true if we can garbage collect
 * a piece of mobile code for which this record maintains
 * status.
 *
 * @return boolean
 */
public boolean isGarbageCollectable() {
	return gc;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isMigrated() {
	return state == ST_MIGRATED;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isMigrating() {
	return state == ST_MIGRATING;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isRunning() {
	return state == ST_RUNNING;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isStopped() {
	return state == ST_STOPPED;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isSuspended() {
	return state == ST_SUSPENDED;
}
/**
 * This method was created by a SmartGuide.
 * @return void
 */
public void resumeThread() {
	if (thread instanceof Thread) {
		thread.resume();
	}
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
protected Thread setThread(Thread thread) {
	return this.thread = thread;
}
/**
 * This method was created by a SmartGuide.
 * @return void
 */
public void stopThread() {
	if (thread instanceof Thread) {
		Thread tmp = thread;
		thread = null;
		tmp.stop();
	}
}
/**
 * This method was created by a SmartGuide.
 * @return void
 */
public void suspendThread() {
	if (thread instanceof Thread) {
		thread.suspend();
	}
}
}