package mct.admin;


/**
 * @(#)MobileCodeStub.java    June 10, 1997 Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.Enumeration;
import java.net.InetAddress;
import mct.users.MobileCodeObject;
import mct.mediator.Message;

/**
 * This is an interface between mobile code and the daemon.
 * The description on each method can be seen on the MobileCodeManager
 *<p>
 * The mobile code (class implementing interface MobileCode)
 * may have any access methods to the MobileCodeManager
 * by implementing one or more interfaces:
 * <ul>
 * <li><b>Callback</b> having methods such as onInit, onRestore, onStart,
 *     onStop, onDestroy as callback of methods init, restore, start, stop,
 *     and destroy respectively, and also method isActive to see if the mobile
 *     code is started or stopped.
 * <li><b>Parameters</b> having methods to access parameters that can
 *     be carried along by the mobile code.
 * <li><b>Migratable</b> having mehtods to allow the mobile code move from
 *     one host to another.
 * <li><b>MCDExtension</b> having methods to get MobileCodeManager and
 *     StorageManager handlers. By this handle, a mobile code can extend services
 *     of the MobileCodeDaemon that relies on mostly to these two classes.
 * <li><b>VMCAccess</b> having methods to access VirtualManagedComponent installed
 *     in the MobileCodeDaemon.
 * <li><b>MessagingAccess</b> having methods to allow a mobile code communicate
 *     with other by passing message.
 * </ul>
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @see     mct.admin.MobileCodeManager
 * @since   MCT1.0
 */
public interface MobileCodeStub {



/**
   * Add a JAR into the MobileCodeContainer of the running mobile code.
   * This is usefull when you want to load more classes that are required
   * by your mobile code.<br>
   * When a JAR is added, it will be verified if the security is enabled.<p>
   * Note: this is not dynamically class loader, but you can anticipate 
   * what classes are going to be needed and you can install it beforehand.<p>
   *
   * Fill with: <br>
   *  if (stub==null) return;<br>
   *  stub.addJAR(name, bytes);
   */
public abstract void addJAR(String name, byte bytes[]);
public void destroy();
public MCTEventMulticaster getEventDispatcher() throws SecurityException;
/**
 * This method was created by a SmartGuide.
 * @return java.net.URL[]
 */
public java.net.URL[] getMigrationDestinations();
/**
   * It returns the MobileCodeContainer handle of the running mobile code.
   * If you add/modify JARs on the MobileCodeContainer, you should
   * call method verifyMobileCodeContainer() to see the classes' changes.<p>
   * Fill with:<br>
   *  if (stub==null) return null;<br>
   *  return stub.getMobileCodeContainer();<br>
   */
public abstract MobileCodeContainer getMobileCodeContainer();
// Handle the MCDExtension if the mobile code implements MCDExtension
public Object getMobileCodeManager() throws SecurityException;
// Handle the Parameters if the mobile code implements Parameters
public Object getParameter(Object paramName);
public Enumeration getParameterNames();
public Enumeration getParameterValues();
public StorageManager getStorageManager() throws SecurityException;
// Handle the VMCAccess if the mobile code implements VMCAccess

public MobileCodeObject getVirtualManagedComponent(String name, boolean className);
public Enumeration getVirtualManagedComponentList();
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean hasMessages();
// Handle the Callback if the mobile code implements Callback
public boolean hasThread();
public void init();
/**
   * This allows the mobile code implementing this interface to install
   * another mobile code.<br>
   * Fill with: <br>
   *  if (stub==null) return;<br> 
   *  stub.instantiate (mcc);<br>
   * @param mcc the container of mobile code to be instantiated.
   */
public abstract void instantiate(MobileCodeContainer mcc);
// Handle the Callback if the mobile code implements Callback
public boolean isActive();
public void migrate(String mobileCodeName, InetAddress ip, int port, boolean flag);
public void migrate(String mobileCodeName, boolean flag);
public void migrate(MobileCodeObject mobileCode, InetAddress ip, int port, boolean flag);
/**
 * This method was created by a SmartGuide.
 * @param mobileCode mct.users.MobileCodeObject
 * @param ip java.net.InetAddress
 * @param destination mct.users.MCIdentifier
 */
public void migrate(MobileCodeObject mobileCode, InetAddress ip, mct.users.MCIdentifier destination);
// Handle the Migration if the mobile code implements Migratable
public void migrate(MobileCodeObject mobileCode, boolean flag);
/**
 * This method was created by a SmartGuide.
 * @return mct.mediator.Message
 */
public Message peek();
public void putParameter(Object paramName, Object paramValue);
public Message receive();
/**
 * This method was created by a SmartGuide.
 * @return mct.mediator.Message
 * @param timer long
 */
public Message receive(long timer);
public Message[] receiveAll();
/**
 * This method was created by a SmartGuide.
 * @return mct.mediator.Message[]
 * @param timer long
 */
public Message[] receiveAll(long timer);
public Object removeParameter(Object paramName);
public void restore();
/**
 * This method was created by a SmartGuide.
 */
public void resume();
// Handle the MessagingAccess if the mobile code implements MessagingAccess
public boolean send(Message mesg);
// Handle the MessagingAccess if the mobile code implements MessagingAccess
public boolean sendAsync(Message mesg);
public void start();
public void stop();
/**
 * This method was created by a SmartGuide.
 */
public void suspend();
/**
   * Return true if the mobile code container is successfully verified;
   * otherwise return false.<p>
   *
   * Fill with:<br>
   *  if (stub==null) return false;
   *  return stub.verify();
   */
public abstract boolean verifyMobileCodeContainer();
}