package mct.admin;


/**
 * The class forwards internal events in the Mobile Code Manager
 * to all registered listeners.
 * Although patterned after AWTEventListener, the central idea
 * in this implementation is to isolate the Mobile Code Manager (MCM)
 * from the listeners : in contrast to the AWT, the MCM's thread 
 * shall not directly invoke the listeners' event handlers 
 * because there are no guarantees that the listeners are written 
 * with the integrity of the MCM in mind. By having the MCM simply 
 * queue events and having a thread within this class, we allow the 
 * MCM to continue on with its own work.
 *
 * The overhead of event-monitoring must be minimized through
 * judicious declaration of events, and perhaps later, some
 * enable/disable methods
 *
 * TBD : I want to use reflection for this class.
 * Q : What happens if a netlet implements >1 listener.  Will it be
 *	registered for >1 type of event ?
 */

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import mct.users.*;

/**
 * QueueEntry simply encapsulates the information about an event
 * for storage in the event queue.
 *  
 * This should be an inner class, but Visual Age does not support them ....
 * TBD : If wanted, could add get/set functions to encapsulate ...
 *
 */

class QueueEntry {
	public String listenerClass;
	// public Method method;
	public String method;
	public Class parameterTypes[];
	public Object arg;



QueueEntry(String listenerClass, String method, Object arg) {
	this.listenerClass = listenerClass;
	this.method = method;
	this.arg = arg;
}
public String toString() {
	return (listenerClass + "::" + method + "(" + arg.toString() + ")");
}
}