package mct.admin;


import java.net.URL;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.awt.*;
import java.util.Properties;
import mct.users.*;
import java.util.BitSet;
/**
 * @(#) RemoteMigrationFacilitator.java	1.0 September-23-1998	Tony White
 * 
 */
public class RemoteMigrationFacilitator extends SuperVirtualManagedComponent implements mct.admin.MigrationInterface {
	private boolean available = false;



/**
 * RemoteMigrationFacilitator constructor comment.
 * @param instantiator Instantiator
 */
public RemoteMigrationFacilitator() throws RemoteException {
	super();
	UnicastRemoteObject.exportObject(this);
}
/**
 * RemoteMigrationFacilitator constructor comment.
 * @param id java.lang.String
 */
public RemoteMigrationFacilitator(String id) throws RemoteException {
	super(id);
	UnicastRemoteObject.exportObject(this);
}
/**
 * This method was created in VisualAge.
 */
public void finalize() {
	available = false;
	try {
		Naming.unbind(getID());
	} catch (Exception e) {
		mct.Console.error(this, "cannot unbind "+getID()+" in RMI registry\nReason: " + e);
	}
}
/**
 * This method returns true when the migration facilitator is ready 
 * for business
 * @return boolean
 */
public boolean isAvailable() {
	return available;
}
/**
 * Bind the object in the RMI registry locally. Maybe we'll extend 
 * this to add the entry in several registries for fault tolerance.
 * @param properties java.util.Properties
 * @return boolean
 */
public boolean onInit(java.util.Properties properties) {
	try {
		Naming.rebind(getID(), this);
		available = true;
	} catch (Exception e) {
		mct.Console.error(this, "cannot rebind "+getID()+" in RMI registry");
		return false;
	}
	return true;
}
/**
 * This method suspends myself. I do this in order to 
 * ensure that I remain registered in the regional directory
 * (if present)
 */
public void onStart() {
	//suspendThread();
	return;
}
/**
 * send method comment.
 */
public boolean send(MCIdentifier id, byte[] state, java.net.URL codebase) throws RemoteException {
	return send(id, new BitSet(8), state, null, codebase);
}
/**
 * send method comment.
 */
public boolean send(MCIdentifier id, java.net.URL codebase) throws RemoteException {
	return send(id, new BitSet(8), null, null, codebase);
}
/**
 * We create a class loader for each piece of mobile code that arrives.
 * However, we use the base mobile code class loader to store the classes
 * and byte codes.
 *
 * @param id MCIdentifier (the identity of the mobile code to be transferred)
 * @param flags BitSet (system flags that pertain to the mobile code)
 * @param state byte[] (serialized form of the mobile code)
 * @param properties Properties (initialization information that's carried with the mobile code)
 * @param codebase URL (the URL to use as a source of class byte codes)
 */
public boolean send(MCIdentifier id, BitSet flags, byte[] state, Properties properties, java.net.URL codebase) throws RemoteException {
	MobileCodeObject mco = null;
	RMIMobileCodeManager mcm = (RMIMobileCodeManager) getMobileCodeManager();
	try {
		// Obtain a base class loader, creating one if necessary
		MCTNetworkClassLoader mcl;
		MCTNetworkClassLoader base_mcl = mcm.obtainClassLoader(id.getAuthority());
		if (base_mcl == null)
			throw new RemoteException("could not allocate class loader for " + id.getAuthority());
		else
			mcl = (MCTNetworkClassLoader) base_mcl.clone();
		mcl.setCodeBase(codebase);
		MobileCodeContainer mcc = new MobileCodeContainer(id);
		mcl.setMobileCodeContainer(mcc);
		Class c = null;
		try {
			c = mcl.loadClass(id.getClassName(), true);
		} catch (ClassNotFoundException ex) {
			throw new RemoteException("could not load class for " + id + "\nReason: " + ex);
		}
		// If class is in cache, we have to check whether we want state or not.
		// Note that the deserialization may also require that we transfer byte
		// codes from a remote device.
		if (c != null) {
			Object obj;
			if (state == null) {
				try {
					obj = c.newInstance();
				} catch (Exception ex) {
					throw new RemoteException("could not instantiate class for " + id + "\nReason: " + ex);
				}
			} else {
				try {
					ReadObjectStream ois = new ReadObjectStream(mcl, new java.io.ByteArrayInputStream(state));
					obj = ois.readObject();
				} catch (Exception ex) {
					throw new RemoteException("could not deserialize " + id + "\nReason: " + ex);
				}
			}
			if (mcc != null && obj instanceof MobileCodeObject) {
				mco = (MobileCodeObject) obj;
				mcc.putClassByteCode(id.getClassName(), mcl.getByteCodes(id.getClassName()));
				mcc.setClassLoader(mcl);
				mcc.putPersistentState(state);
				mcc.setFlags(flags);
				mcc.setParameters(properties);
				StorageManager sm = getStorageManager();
				boolean present = sm.getContainer(id.getID()) != null;
				if (present && mcm.replace(id.getID())) {
					mct.Console.warning(null, "replacing " + id);
					present = false;
				}
				if (!present) {
					sm.putContainer(mcc);
					if (!mcm.registerMobileCode(id.getID(), mco, properties)) {
						throw new RemoteException("could not initiate " + id);
					}
				} else { // Tidyup by removing things from the storage and mobile code managers
					throw new RemoteException("could not replace " + id);
				}
			} else // Throwing the exception will result in removal of anything created
				throw new RemoteException(id.getID() + " does not extend class mct.users.MobileCodeObject");

			// Save parameters that might be useful in the base class
			// This is an extended form of caching and is designed to
			// speed resolution (lookup) of migration facilities considerably.
			base_mcl.setCodeBase(codebase);
			base_mcl.setMigrationFacilitator(mcl.getMigrationFacilitator());
			return obj != null;
		} else
			throw new RemoteException("could not create class " + id.getClassName() + " " + id.getAuthority());
	} catch (Throwable globalEx) {
		// We have to return something to the other end or we will be stuck waiting forever.
		// So, regardless of what happened here, we continue.
		// Ensure that we tidy up in terms of removing references to mobile code
		// that we created. This might be a great deal of work...
		try {
			if (mco != null)
				mcm.destroy(mco);
		} catch (Throwable destroyException) {
		}
		if (globalEx instanceof RemoteException)
			throw (RemoteException) globalEx;
		else
			throw new RemoteException(globalEx.toString());
	}
}
/**
 * transfer method comment.
 */
public byte[] transfer(String className, String authority) throws RemoteException {
	try {
		RMIMobileCodeManager mcm = (RMIMobileCodeManager) getMobileCodeManager();
		MCTNetworkClassLoader mcl = mcm.getClassLoader(authority);
		if (mcl == null)
			throw new RemoteException("unknown class loader: " + authority);
		mct.Console.info(null, getID() + " transferring " + className + " " + authority);
		return mcl.getByteCodes(className);
	} catch (Throwable globalEx) {
		if (globalEx instanceof RemoteException)
			throw (RemoteException) globalEx;
		else
			throw new RemoteException(globalEx.toString());
	}
}
}