package mct.admin;


/**
 * @(#)StorageManager.java      Gatot Susilo February 5, 1997
 *  Revision:
 *    May 24, 1997, Gatot Susilo
 *      - the package is renamed to mct from pmc.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * StorageManager manages the mobile code's data; each is stored in a package
 * MobileCodeContainer.
 *
 * @author Gatot Susilo
 * @see mct.admin.MobileCodeContainer
 * @version 1.0, July 1, 1997
 * @since   MCT1.0
 */

public class StorageManager {
	//**********
	//* Fields *
	//**********

	// Table of MobileCodeContainer
	private Hashtable mcHandler;



//****************
//* Constructors *
//****************
/**
   * The constructor creates a table to store the MobileCodeContainer.
   */
public StorageManager() {

	// Put them together to make management easier
	mcHandler = new Hashtable();
}
/**
   * Get the mobile code whose name is specified in the 
   * parameter. The name should be absolute name.
   * @param name the name of the mobile code
   * @return the MobileCodeContainer if it is found 
   *         otherwise return null
   */
public synchronized MobileCodeContainer getContainer(String name) {
	return (MobileCodeContainer) mcHandler.get(name);
}
/**
   * Return the list of MobileCodeContainer's names
   */
public Enumeration getNames() {
	return mcHandler.keys();
}
/**
   * Create a new MobileCodeContainer if it has not existed yet
   * otherwise just used the existing one.
   * @param name the name of the mobile code
   * @see#getContainer
   */
protected synchronized MobileCodeContainer putContainer(String name) {
	MobileCodeContainer mobileCode;
	// if the package is already in the table just return the handler
	// to the existing package
	if (mcHandler.containsKey(name)) {
		return (MobileCodeContainer) mcHandler.get(name);
	}
	// otherwise create a new one
	mobileCode = new MobileCodeContainer(name);
	mcHandler.put(name, mobileCode);
	return mobileCode;
}
/**
   * Store a mobile code  into  the table
   * @param the mobile code that is going to be stored.
   */
protected synchronized void putContainer(MobileCodeContainer mobileCode) {
	mcHandler.put(mobileCode.getMobileCodeName(), mobileCode);
}
/**
   * Remove the mobile code from the table.
   * @param name the name of the mobile code
   * @param type the type of the mobile code
   * @return the handler of removed mobile code
   * @see#getContainer
   * @see#putContainer
   */
public synchronized MobileCodeContainer removeContainer(String name) {
	return (MobileCodeContainer) mcHandler.remove(name);
}
}