package mct.blackboard;


/**
 *@(#)BlackBoardMessage.java  
 *
 * This class provides the superclass for all messages exchanged 
 * between a blackboard and its users, when using a Facilitator
 * for communication services (hence, it extends Message). 
 * Collectively, subclasses of this
 * class constitute the blackboard protocol
 *
 * All Blackboard messages will minimally have the following format
 *	key     The blackboard is divided into subject areas, 
 *		indexed by a key
 * 	type	Type of message, given by subclasses of this class
 *		Collectively, the message types define the blackboard protocol
 *
 * TBD : Get rid of type, don't need it.
 * @see mct.blackboard.MsgBlackBoard
 */

import java.security.PublicKey;
import java.security.Identity;
import java.security.IdentityScope;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import mct.mediator.Message;
public class BlackBoardMessage extends Message implements Serializable {



/**
   * Constructs a generic blackboard message, filling in all mandatory fields
   */
public BlackBoardMessage(String from, String to, Object key) {
	super(from, to, true);
	// Verify type first ...
	super.addMessage("msgType", toString()); // Message tag
	super.addMessage("key", key); // What types of events to be received
}
/**
   * Returns the key for this message
   */
public Object getKey() {
	Object value[];
	if ((value = super.getMessages("key")) != null) {
		return value[0];
	}
	return null;
}
/**
   * Returns the type of message
   */
public String getType() {
	Object value[];
	if ((value = super.getMessages("msgType")) != null) {
		return (String) value[0];
	}
	return null;
}
/**
   * Returns the string representation of this message
   */
public String toString() {
	return getClass().getName();
}
}