package mct.examples.McTicTacToe;


/**
 * @(#)McTicTacToe.java     June 20 ,1997   Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import mct.users.Netlet;
import java.io.Serializable;

/**
 * This demonstrates the Netlet can keep a persistent state by 
 * implementing interface Serializable. The persistent state is
 * that the states of non transient field of object are kept
 * persistent during migration.<p>
 * In this case, the Netlet plays game tic tac toe in which
 * you will find that it will keep migrating until the game is
 * over (one visit - one move).
 */
public class McTicTacToe extends Netlet implements Serializable {

	// From SUN
	/**
	   * White's current position. The computer is white.
	   */
	int white;

	/**
	   * Black's current position. The user is black.
	   */
	int black;

	/**
	   * The squares in order of importance...
	   */
	final static int moves[] = {4, 0, 2, 6, 8, 1, 3, 5, 7};

	/**
	   * The winning positions.
	   */
	static boolean won[] = new boolean[1 << 9];
	static final int DONE = (1 << 9) - 1;
	static final int OK = 0;
	static final int WIN = 1;
	static final int LOSE = 2;
	static final int STALEMATE = 3;

	/**
	   * Initialize all winning positions.
	   */
	static {
		isWon((1 << 0) | (1 << 1) | (1 << 2));
		isWon((1 << 3) | (1 << 4) | (1 << 5));
		isWon((1 << 6) | (1 << 7) | (1 << 8));
		isWon((1 << 0) | (1 << 3) | (1 << 6));
		isWon((1 << 1) | (1 << 4) | (1 << 7));
		isWon((1 << 2) | (1 << 5) | (1 << 8));
		isWon((1 << 0) | (1 << 4) | (1 << 8));
		isWon((1 << 2) | (1 << 4) | (1 << 6));
	}

	/**
	   * Who goes first in the next game?
	   */
	boolean first = true;
	boolean blackMove = true;
	int count = 0;



/**
   * Compute the best move for white.
   * @return the square to take
   */
int bestMove(int white, int black) {
	int bestmove = -1;
	loop : for (int i = 0; i < 9; i++) {
		int mw = moves[i];
		if (((white & (1 << mw)) == 0) && ((black & (1 << mw)) == 0)) {
			int pw = white | (1 << mw);
			if (won[pw]) {
				// white wins, take it!
				return mw;
			}
			for (int mb = 0; mb < 9; mb++) {
				if (((pw & (1 << mb)) == 0) && ((black & (1 << mb)) == 0)) {
					int pb = black | (1 << mb);
					if (won[pb]) {
						// black wins, take another
						continue loop;
					}
				}
			}
			// Neither white nor black can win in one move, this will do.
			if (bestmove == -1) {
				bestmove = mw;
			}
		}
	}
	if (bestmove != -1) {
		return bestmove;
	}

	// No move is totally satisfactory, try the first one that is open
	for (int i = 0; i < 9; i++) {
		int mw = moves[i];
		if (((white & (1 << mw)) == 0) && ((black & (1 << mw)) == 0)) {
			return mw;
		}
	}

	// No more moves
	return -1;
}
public void draw(String str) {
	System.out.print("\n\nGame status: " + str);
	int i = 0;
	for (int r = 0; r < 3; r++) {
		System.out.print("\n\t");
		for (int c = 0; c < 3; c++, i++) {
			if ((white & (1 << i)) != 0) {
				System.out.print("W");
			} else
				if ((black & (1 << i)) != 0) {
					System.out.print("B");
				} else
					System.out.print("*");
		}
	}
	System.out.println("");
}
// Methods
public String getInfo() {
	return "Tic Tac Toe played by mobile code";
}
/**
   * Mark all positions with these bits set as winning.
   */
static void isWon(int pos) {
	for (int i = 0; i < DONE; i++) {
		if ((i & pos) == pos) {
			won[i] = true;
		}
	}
}
public static void main(String args[]) {
	McTicTacToe mttt = new McTicTacToe();
	mttt.onInit();
	mttt.onRestore();
	mttt.onStart();
}
/**
   * Computer move.
   * White move
   * @return true if legal
   */
boolean myMove() {
	if ((black | white) == DONE) {
		return false;
	}
	int best = bestMove(white, black);
	white |= 1 << best;
	return true;
}
public void onDestroy() {
	System.out.println("METHOD onDestroy is called");
}
public boolean onInit() {
	System.out.println("METHOD onInit is called");
	return true;
}
public int onMigrate() {
	return SERIALIZE;
}
public boolean onRestore() {
	System.out.println("METHOD onRestore is called");
	return true;
}
public void onStart() {
	switch (status()) {
		case WIN :
		case LOSE :
		case STALEMATE :
			white = black = 0;
			if (first) {
				white |= 1 << (int) (Math.random() * 9);
				draw("white move");
				blackMove = true;
			} else {
				black |= 1 << (int) (Math.random() * 9);
				draw("black move");
				blackMove = false;
			}
			first = !first;
	}
	boolean done = false;
	// while(! done) {
	count++;
	if (blackMove) {
		yourMove();
		draw(count + ". black move");
	} else {
		myMove();
		draw(count + ". white move");
	}
	switch (status()) {
		case WIN :
			System.out.println("White won...");
			done = true;
			break;
		case LOSE :
			System.out.println("White lost...");
			done = true;
			break;
		case STALEMATE :
			System.out.println("Game tight...");
			done = true;
			break;
		default :
			blackMove = !blackMove;
	}
	// }
	if (!done)
		migrate(this, true);
	else {
		stop();
		destroy();
	}
}
public void onStop() {
	System.out.println("METHOD onStop is called");
}
/**
   * Figure what the status of the game is.
   */
int status() {
	if (won[white]) {
		return WIN;
	}
	if (won[black]) {
		return LOSE;
	}
	if ((black | white) == DONE) {
		return STALEMATE;
	}
	return OK;
}
/**
   * Black move 
   */
boolean yourMove() {
	int m = -1;
	boolean ok = false;
	while (!ok) {
		ok = true;
		m = (int) (Math.random() * 9);
		if ((m < 0) || (m > 8))
			ok = false;
		if (((black | white) & (1 << m)) != 0)
			ok = false;
	}
	if ((black | white) == DONE) {
		return false;
	}
	//int best = bestMove(black,white);
	//black |= 1 << best;
	black |= 1 << m;
	return true;
}
}