package mct.examples.NetletTester;


/**
 * @(#)NetletTester.java  January 7, 1996 Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import mct.users.Netlet;
import mct.users.VirtualManagedComponent;
import mct.users.VMCException;
import mct.users.MobileCodeObject;

/**
 * The NetletTester demonstrates the capability of mobile code to
 * access the parameters and also VirtualManagedComponent. Furthermore,
 * it demonstrates the migration as well.<p>
 * You should inject the VMC (i.e., <b>SimpleVMC</b> into the daemon
 * before you inject the NetletTester.
 */
public class NetletTester extends Netlet {
	private int numberOfVisitedNodes;
	private VirtualManagedComponent managedObject;



public void onDestroy() {
	System.out.println("BEGIN DESTROY");
	System.out.println("END DESTROY");
}
public boolean onInit() {
	System.out.println("BEGIN INIT");
	System.out.println("\tgetParameter(\"A\"): " + (String) getParameter("A"));
	// I just want to visit a certain number of nodes, hence I want to track
	// how many nodes I have visited by using parameters
	String numberOfNodes = (String) getParameter("Nodes");
	if (numberOfNodes == null) {
		putParameter("Nodes", Integer.toString(1));
		numberOfVisitedNodes = 1;
	} else {
		numberOfVisitedNodes = Integer.parseInt(numberOfNodes);
		numberOfVisitedNodes++;
		putParameter("Nodes", Integer.toString(numberOfVisitedNodes));
	}
	System.out.println("END INIT");
	return true;
}
public void onStart() {
	System.out.println("BEGIN START");
	// Try to put and get parameters
	System.out.println("\tputParameter(fruit,apple)");
	putParameter("fruit", "apple");
	System.out.println("\tgetParameter(fruit)=" + (String) getParameter("fruit"));

	// Try to manage an object group named SimpleMO
	// There are 5 objects: Adapter,Brand,Speed,Address,Link
	MobileCodeObject mCode = getVirtualManagedComponent("mct.examples.NetletTester.SimpleVMC", true);
	if (mCode != null) {
		managedObject = (VirtualManagedComponent) mCode;
		System.out.println("MANAGE OBJECT");
		try {
			System.out.println("\tAdapter: " + managedObject.get("Adapter"));
			System.out.println("\tBrand: " + managedObject.get("Brand"));
			System.out.println("\tI want to change the brand into SMC EtherPower");
			managedObject.set("Brand", "SMC EtherPower");
			System.out.println("\tBrand: " + managedObject.get("Brand"));
			System.out.print("\tI want to change the Adapter into");
			System.out.println(" Modem which is not permissible");
			managedObject.set("Adapter", "Modem");
		} catch (VMCException ex) {
			System.out.println("\t" + ex);
		}
	} else {
		System.out.println("\tObject SimpleVMC is not found");
	}
	System.out.println("END START and CALL METHOD STOP");
	System.out.println("\tIf I have visited two nodes, I don't");
	System.out.println("\tmigrate anymore and I want to destroy myself");
	if (numberOfVisitedNodes > 2) {
		System.out.println("END STOP and CALL METHOD DESTROY");
		destroy();
	} else {
		String isTcp = (String) getParameter("tcp");
		if ((isTcp != null) && (isTcp.equals("true"))) {
			System.out.println("\tmigrate to default destination via TCP");
			migrate(this, true);
		} else {
			System.out.println("\tmigrate to default destination via UDP");
			migrate(this, false);
		}
	}
}
public void onStop() {
	System.out.println("BEGIN STOP");
	System.out.println("END STOP");
}
}