package mct.examples.TelnetDaemon;


/**
 * @(#)TelnetDaemon.java     05-Feb-1996 Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import mct.users.*;
import mct.admin.StorageManager;
import mct.admin.MobileCodeManager;
import mct.admin.MobileCodeContainer;
import mct.net.NVTInputStream;
import mct.net.NVTOutputStream;
import java.util.*;
import java.io.*;
import java.net.*;
class TelnetSession extends Thread {
	private TelnetDaemon td;
	private StorageManager storageManager;
	private MobileCodeManager mobileCodeManager;

	// Implement basic communication of telnet or
	// other internet connection.
	private NVTInputStream nis;
	private NVTOutputStream nos;
	private Socket socket;

	// Special Characters
	private byte CR = 13;
	private byte LF = 10;
	private byte BS = 8;

	// Current Netlet Selected
	private String netlet = "";
	private String localhost;



TelnetSession(TelnetDaemon td, Socket socket) {
	this.td = td;
	this.socket = socket;
	storageManager = this.td.storageManager;
	mobileCodeManager = this.td.mobileCodeManager;
	try {
		nos = new NVTOutputStream(this.socket.getOutputStream());
		nis = new NVTInputStream(this.socket.getInputStream(), nos);
	} catch (IOException e) {
		System.out.println("IOException occurs");
	}
	try {
		localhost = (InetAddress.getLocalHost()).getHostName();
	} catch (UnknownHostException e) {
	}
	start();
}
public void printHelp() {
	nos.println("Valid command:");
	nos.println("\tselect mobile code");
	nos.println("\tlist classes");
	nos.println("\tlist parameters");
	nos.println("\tlist mobile code");
	nos.println("\tstart");
	nos.println("\tstop");
	nos.println("\tdestory");
	nos.println("\tmigrate");
	nos.println("\tquit");
	nos.println("\thelp");
}
public void printPrompt(String netlet) {
	if (netlet.equals("")) {
		nos.print(localhost + "{no netlet selected}% ");
	} else {
		nos.print(localhost + "{" + netlet + "}% ");
	}
}
public String readLine() throws IOException {
	byte command[] = new byte[128];
	int pos = 0;
	boolean done = false;
	do {
		int ch = nis.read();
		if (ch == -1)
			return null;
		byte b = (byte) ch;
		if (b == LF) {
			if (pos > 0) {
				if (command[pos - 1] == CR)
					return new String(command, 0, pos - 1);
			}
		} else
			if (b == BS) {
				if (pos > 0)
					pos--;
				nos.print(' ');
				nos.write(BS);
			} else {
				command[pos] = b;
				pos++;
			}
	} while (!done);
	return null;
}
public void run() {
	try {
		startCmdLine();
		socket.close();
	} catch (IOException e) {
		System.out.println("IOException occurs");
	}
}
public void startCmdLine() {
	String name = "";
	MobileCodeContainer mcc = null;
	boolean done = false;
	nos.println("WELCOME to " + localhost);
	printHelp();
	try {
		while (!done) {
			printPrompt(netlet);
			String cmd = readLine().trim();

			// 1. select netlet
			if (cmd.equals("select mobile code")) {
				nos.print("Enter name: ");
				netlet = readLine().trim();
				mcc = storageManager.getContainer(netlet);
				if (mcc == null) {
					netlet = "";
					nos.println("mobile code not found");
				}
			}

			// 2. list classes 
			else
				if (cmd.equals("list classes")) {
					if (mcc == null) {
						nos.println("ERROR: no netlet selected");
					} else {
						Enumeration enum = mcc.getClassNames();
						while (enum.hasMoreElements()) {
							nos.println((String) enum.nextElement());
						}
					}
				}

				// 3. list parameters
				else
					if (cmd.equals("list parameters")) {
						if (mcc == null) {
							nos.println("ERROR: no netlet selected");
						} else {
							Enumeration enum = mcc.getParameterNames();
							while (enum.hasMoreElements()) {
								String par = (String) enum.nextElement();
								nos.println(par + "=" + mcc.getParameter(par));
							}
						}
					}

					// 4. list netlets 
					else
						if (cmd.equals("list mobile code")) {
							Enumeration enum = storageManager.getNames();
							while (enum.hasMoreElements()) {
								String par = (String) enum.nextElement();
								nos.println(par);
							}
						}

						// 5. start
						else
							if (cmd.equals("start")) {
								if (mcc == null)
									nos.println("ERROR: no netlet selected");
								else
									mobileCodeManager.start(mobileCodeManager.getMobileCodeHandle(netlet));
							}

							// 6. stop
							else
								if (cmd.equals("stop")) {
									if (mcc == null)
										nos.println("ERROR: no netlet selected");
									else
										mobileCodeManager.stop(mobileCodeManager.getMobileCodeHandle(netlet));
								}

								// 7. destroy
								else
									if (cmd.equals("destroy")) {
										if (mcc == null)
											nos.println("ERROR: no netlet selected");
										else {
											mobileCodeManager.destroy(mobileCodeManager.getMobileCodeHandle(netlet));
											netlet = "";
											mcc = null;
										}
									}

									// 8. migrate
									else
										if (cmd.equals("migrate")) {
											if (mcc == null)
												nos.println("ERROR: no netlet selected");
											else {
												nos.println("Migrate via TCP");
												nos.print("Enter host: ");
												String host = readLine().trim();
												nos.print("Enter port: ");
												int port = 0;
												try {
													port = Integer.parseInt(readLine().trim());
												} catch (Throwable ex) {
													nos.println("ERROR: invalid port number");
												}
												if (host.equals("") || (port < 1))
													nos.println("ERROR: host and/or port not specified");
												else {
													mobileCodeManager.migrate(mobileCodeManager.getMobileCodeHandle(netlet), InetAddress.getByName(host), port, true);
													netlet = "";
													mcc = null;
												}
											}
										}

										// 9. quit
										else
											if (cmd.equals("quit")) {
												done = true;
											}

											// 10. help
											else
												if (cmd.equals("help")) {
													printHelp();
												} else {
													nos.println("ERROR: invalid command");
												}
		}
	} catch (IOException ex) {
		System.out.println("IOException occurs....");
	}
}
}