package mct.examples.messaging;


/**
 * @(#)EnquireNetlet.java	1.0 October-13-1997	Cheryl Schramm
 *
 * 	The Netlet periodically polls the blackboard, 
 *	sometimes migrating after the response is received,
 *	sometimes migrating before.
 *
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;
import mct.users.*;
import mct.admin.*;
import mct.net.*;
import mct.mediator.*;
import mct.blackboard.*;
public class EnquireNetlet extends SuperNetlet implements Serializable {
	private Integer last = new Integer(-99);
	private int tries = 0;



public void onDestroy() {
}
public void onFailMigrate(InetAddress ip, int port, boolean isTCP) {
	tries++;
	try {
		Thread.sleep(1000);
	} catch (java.lang.InterruptedException ex) {
	};
	System.err.println("EnquireNetlet::onFailMigrate " + ip.toString() + port + isTCP);
	if (tries < 5)
		migrate(this, true);
}
public boolean onInit() {
	if (getParameter("blackboard") == null)
		putParameter("blackboard", "blackboard[PUBLIC]@mct.blackboard.MsgBlackBoard");
	return true;
}
public boolean onInit(java.util.Properties properties) {
	// Set variables defined as properties
	if (properties != null) {
		String bb = properties.getProperty("EnquireNetlet.blackboard", "blackboard[PUBLIC]@mct.blackboard.MsgBlackBoard");
		putParameter("blackboard", bb);
		return true;
	} else
		return onInit();
}
public void onStart() {
	tries = 0;
	System.out.println(getID() + " : onStart ");
	EnquireMessage msg = new EnquireMessage(getID(), (String) getParameter("blackboard"), "nhops");
	while (!send(msg)) {
		mct.Console.application(null, getID() + " unable to inquire on " + (String) getParameter("blackboard"));
		try {
			Thread.sleep(2000);
			System.out.println(getID() + " sleeping while sending...");
		} catch (InterruptedException ie) {
		}
	}

	// Have to deal with the situation where we are not yet registered
	// with the local facilitator's mobile code directory. A better way
	// to implement the facilitator would be to use the mobile code 
	// manager directly.

	Message[] reply;
	while ((reply = receiveAll()) == null) {
		try {
			Thread.sleep(2000);
			System.out.println(getID() + " sleeping while receiving...");
		} catch (java.lang.InterruptedException ex) {
		}
	}
	try {
		EnquireReplyMessage info = (EnquireReplyMessage) reply[reply.length - 1];
		Integer contents = (Integer) info.getContents();
		if (last.intValue() == contents.intValue())
			try {
			Thread.sleep(1000);
		} catch (java.lang.InterruptedException ex) {
		};
		last = contents;
		System.out.println(" **> " + getID() + "  queries : nhops = " + contents.toString());
	} catch (ClassCastException ce) {
		mct.Console.application(this, "unexpected message contents: " + reply.toString());
	}
	// Sleep in order to let some other thread have a turn
	try {
		Thread.sleep(250);
	} catch (InterruptedException ex) {
	}
	System.out.println(getID() + " MIGRATING ");
	migrate(this, true);
}
public void onStop() {
}
}