package mct.examples.messaging;


/**
 * @(#)NotifyNetlet.java	1.0 October-13-1997	Cheryl Schramm
 *
 * 	The Netlet registers to receive notifications of postings to the BlackBoard.
 *	The netlet periodically migrates, to test whether notifications follow
 *	migrations
 *
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;
import mct.users.*;
import mct.admin.*;
import mct.net.*;
import mct.mediator.*;
import mct.blackboard.*;
public class NotifyNetlet extends SuperNetlet implements Serializable {
	private boolean firstTime = true;
	private int tries;



public void onDestroy() {
}
public void onFailMigrate(InetAddress ip, int port, boolean isTCP) {
	tries++;
	System.err.println("NotifyNetlet::onFailMigrate " + ip.toString() + port + isTCP);
	try {
		Thread.sleep(1000);
	} catch (java.lang.InterruptedException ex) {
	};
	if (tries < 5)
		migrate(this, true);
}
public boolean onInit() {
	if (getParameter("blackboard") == null)
		putParameter("blackboard", "blackboard[PUBLIC]@mct.blackboard.MsgBlackBoard");
	return true;
}
public boolean onInit(java.util.Properties properties) {
	// Set variables defined as properties
	if (properties != null) {
		String bb = properties.getProperty("NotifyNetlet.blackboard", "blackboard[PUBLIC]@mct.blackboard.MsgBlackBoard");
		putParameter("blackboard", bb);
		return true;
	} else
		return onInit();
}
public void onMessage() {
	System.out.println(getID() + " : onMessage ");
}
public void onStart() {
	TestObject o = new TestObject(); // Test network class loader
	tries = 0;
	registerWithBlackboard();
	System.out.println(getID() + " : onStart ");
	Message[] msg;
	while ((msg = receiveAll()) == null) {
		try {
			Thread.sleep(1000);
			System.out.println(getID() + " sleeping...");
		} catch (java.lang.InterruptedException ex) {
		}
	}
	try {
		NotifyMessage nmsg = (NotifyMessage) msg[msg.length-1];
		Integer contents = (Integer) nmsg.getContents();
		System.out.println(" **> " + getID() + " received posting: nhops = " + contents.toString());
	} catch (ClassCastException ce) {
		mct.Console.application(null, getID() + " received unexpected message " + msg.toString());
	} catch (NullPointerException ne) {
		mct.Console.application(null, getID() + " mailbox was empty even though notified");
	}
	System.out.println(getID() + " MIGRATING ");
	migrate(this, true);
}
public void onStop() {
}
public void registerWithBlackboard() {
	if (firstTime) {
		firstTime = false;
		String bb = (String) getParameter("blackboard");
		RegisterMessage rmsg = new RegisterMessage(getID(), bb, "nhops");
		System.out.println(rmsg.getFrom() + " registering with " + rmsg.getTo() + "for key = " + rmsg.getKey().toString());
		int stries = 0;
		while (!send(rmsg) && stries < 5) {
			stries++;
			mct.Console.application(null, getID() + " unable to register with " + bb + "(" + stries + ")");
			try {
				Thread.sleep(2500);
			} catch (Exception ex) {
			}
		};
		if (stries == 5)
			firstTime = true;
	}
}
}