package mct.examples.messaging;


/**
 * @(#)SendNetlet.java	1.0 October-13-1997	Cheryl Schramm
 *
 * 	The Netlet repeatedly sends messages to "ReceiveNetlet".
 *	Between transmissions, the netlet sleeps and migrates.
 *
 *    Netlet Parameters (to be set by the launching process) :
 *    ------------------
 *
 *    M mandatory
 *    O optional
 *
 *    @see ReceiveNetlet
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;
import mct.users.*;
import mct.admin.*;
import mct.net.*;
import mct.mediator.*;
public class SendNetlet extends SuperNetlet implements Serializable {

	//**********
	//* Fields *
	//**********

	private int nhops = 0;
	private String receiver = null;
	private int sleepTime = 0;



public void onDestroy() {
}
public void onFailMigrate(InetAddress ip, int port, boolean isTCP) {
	System.err.println("SendNetlet::onFailMigrate " + ip.toString() + port + isTCP);
}
public boolean onInit() {
	// Retrieve the netlet parameters
	try {
		if ((receiver = (String) getParameter("receiver")) == null) {
			receiver = "ReceiveNetlet";
		}
		String s;
		if ((s = (String) getParameter("sleepTime")) == null) {
			sleepTime = 2500;
		} else {
			sleepTime = Integer.parseInt(s);
		}
	} catch (ClassCastException e) {
		receiver = "ReceiveNetlet";
	}
	return true;
}
public boolean onInit(java.util.Properties properties) {
	String s;
	if (properties != null) {
		s = properties.getProperty("SendNetlet.receiver");
		if (s != null) {
			putParameter("receiver", s);
		} else {
			return false;
		}
		s = properties.getProperty("SendNetlet.sleepTime");
		if (s != null) {
			putParameter("sleepTime", s);
		} else {
			return false;
		}
		return onInit();
	} else
		return onInit();
}
public void onMessage() {
}
public void onStart() {
	System.out.println(getIdentifier().getID() + " : onStart " + nhops);
	try {
		Thread.sleep(sleepTime);
	} catch (InterruptedException ie) {
	}

	// Message will be sent locally first, and then
	// relayed if not present
	// May have to first query the MobileCodeManager to determine
	// if the MonitorExtlet is running ... to avoid message
	// forwarding to the Mediator.

	Message query;
	query = new Message(getIdentifier().getID(), receiver, true);
	System.out.println(query.getFrom() + " sending to " + query.getTo());
	if (send(query)) {
		System.out.println(query.getFrom() + " sent to " + query.getTo());
		Message response = receive();
		System.out.println(response.getTo() + " received from " + response.getFrom());
		// Extract the local information and "compose a report"
		Object[] values = response.getMessages("count");
		int count = ((Integer) values[0]).intValue();
		System.out.println(getIdentifier().getID() + " : Received " + count);
	} else {
		System.err.println(getIdentifier().getID() + " Failed send");
	}
	nhops++;
	System.out.println(getIdentifier().getID() + " Migrating ..." + nhops);
	migrate(this, true);
}
public void onStop() {
}
}