package mct.mediator;


/**
 * @(#) Mailbox.java	1.0 October-13-1997	Cheryl Schramm
 *
 * A mailbox is a synchronization object between threads running
 * in the same JVM. It provides content-independent message repository 
 * for local synchronous message exchanges between two mobileCode
 * threads running on the same MCD.
 * 
 * 
 */
import java.util.Vector;
import mct.admin.Semaphore;
public class Mailbox implements MailboxInterface {
	Vector mailbox = null;
	Vector queue = null;
public Mailbox() {
	mailbox = new Vector();
	queue = new Vector();
}
/**
 * Gives a true/false response depending on whether messages available
 * @return boolean
 */
public boolean hasMessages() {
	return mailbox.size() != 0;
}
/**
   * Peeks at the first message in the mailbox
   */
public Object peek() {
	synchronized (mailbox) {
		if (mailbox.size() == 0)
			return null;
		else
			return mailbox.firstElement();
	}
}
/**
   * Retrieves the next message in the mailbox for this destination
   * Waits forever if no message currently available
   */
public Object receive() {
	return receive(0);
}
/**
   * Retrieves the next message in the mailbox for this destination
   * Waits forever if no message currently available
   */
public Object receive(long timer) {
	if (mailbox.isEmpty()) {
		Thread thread = Thread.currentThread();
		queue.addElement(thread);
		if (timer == 0)
			thread.suspend();
		else {
			try {
				thread.sleep(timer);
			} catch (InterruptedException ex) {
			}
			queue.removeElement(thread);
			if (mailbox.isEmpty())
				return null;
		}
	}
	synchronized (mailbox) {
		Object msg = mailbox.firstElement();
		mailbox.removeElementAt(0);
		return msg;
	}
}
/**
   * Retrieves all messages in the mailbox for this destination
   * Waits forever if no messages are currently available
   */
public Object[] receiveAll() {
	return receiveAll(0);
}
/**
   * Retrieves all messages in the mailbox for this destination
   * Waits for predetermined period (forever if timer == 0).
   */
public Object[] receiveAll(long timer) {
	if (mailbox.isEmpty()) {
		Thread thread = Thread.currentThread();
		queue.addElement(thread);
		if (timer == 0)
			thread.suspend();
		else {
			try {
				thread.sleep(timer);
			} catch (InterruptedException ex) {
			}
			queue.removeElement(thread);			
			if (mailbox.isEmpty())
				return null;
		}
	};
	synchronized (mailbox) {
		Message messages[] = new Message[mailbox.size()];
		mailbox.copyInto(messages);
		mailbox.removeAllElements();
		return messages;
	}
}
/**
   * Sends the message to the destination by putting the message
   * in the respective mailslot, notifying the destination if it
   * is already waiting
   */
public void send(Object message) {
	mailbox.addElement(message);
	synchronized (queue) {
		if (!queue.isEmpty()) {
			Thread thread = (Thread) queue.firstElement();
			queue.removeElementAt(0);
			thread.resume();
		}
	}
}
}