package mct.mediator;


/**
 *@(#)Message.java        July 7, 1997  Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.security.PublicKey;
import java.security.Identity;
import java.security.IdentityScope;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/** 
 * Container of message that is used for inter-agent communication.
 * The content of message uses TLV (Tag Length Value) format.
 */
public class Message implements Serializable {
	private Hashtable message; // Tag that represents the class path fullname
	private String from; // This should be unique ID
	private String to; // This should be unique ID
	private boolean request; // Request, Response
	private boolean async; // Whether asynchronous or not
	private String actualRecepient = ""; // tracking the change of recepient














/**
 * This method was created by Tony White.
 * @param from java.lang.String
 * @param to java.lang.String
 */
public Message(String from, String to) {
	message = new Hashtable();
	this.from = from;
	this.to = to;
	this.request = true;
	isSync();
}
/**
   * Create message.
   * The request is typically without any value.
   * So the recepient can fill out the value of request and send it back
   * to the sender.
   * @param from the id of mobile code as a sender
   * @param to the id of mobile code as a recepient
   * @param request true if the messaging is a request
   */
public Message(String from, String to, boolean request) {
	message = new Hashtable();
	this.from = from;
	this.to = to;
	this.request = request;
	isSync();
}
/**
   * Add a message whose tag and its value are specified in the parameter.
   * To add one or more value, you call this method with the same tag and
   * value that you want to add in.<p>
   * The tag must be: String or Byte. It is recommended that you are consistent
   * in using the tag's object type.<p>
   * The valid value must be in object: Byte, Short, Integer, Long, Float,
   * Double, Character, String, Boolean, PublicKey, Identity, IdentityScope,
   * MCDRecord, MobileCodeRecord.
   */
public boolean addMessage(Object tag, Object value) {
	if (((tag instanceof String) || (tag instanceof Byte)) && ((value instanceof Byte) || (value instanceof Short) || (value instanceof Integer) || (value instanceof Long) || (value instanceof Float) || (value instanceof Double) || (value instanceof Character) || (value instanceof String) || (value instanceof Boolean) || (value instanceof PublicKey) || (value instanceof Identity) || (value instanceof IdentityScope) ||
		// (value instanceof MCDRecord) ||
	 	(value instanceof MobileCodeRecord))) {
		addMessage0(tag, value);
		return true;
	}
	return false;
}
// add a message into container
private void addMessage0(Object tag, Object value) {
	Vector msgVector = (Vector) message.get(tag);
	if (msgVector == null) {
		msgVector = new Vector();
		message.put(tag, msgVector);
	}
	msgVector.addElement(value);
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beAsync() {
	return async = true;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean beSync() {
	return async = false;
}
/**
   * Forward the message to the one as specified in the parameter.
   * To maintain privacy of users, only message for ALL (everyone
   * in the network) can be forwarded.
   * @return true if change is successfully done; otherwise return false.
   */
public boolean forwardTo(String recp) {
	if ((to.equalsIgnoreCase("ALL")) || (actualRecepient.equalsIgnoreCase("ALL"))) {
		actualRecepient = "ALL";
		to = recp;
		return true;
	}
	return false;
}
/**
   * Return the id of the sender
   */
public String getFrom() {
	return from;
}
/**
   * Return the length of messages's values whose tage is specified in the parameter.
   */
public int getLength(Object tag) {
	Vector mesg = (Vector) message.get(tag);
	if (mesg == null)
		return -1;
	return mesg.size();
}
/**
   * Return array of message's values whose tag is specified in the parameter.
   */
public Object[] getMessages(Object tag) {
	Vector mesg = (Vector) message.get(tag);
	if (mesg == null)
		return null;
	Object obj[] = new Object[mesg.size()];
	mesg.copyInto(obj);
	return obj;
}
/**
   * Return enumeration of tags.
   */
public Enumeration getTags() {
	return message.keys();
}
/**
   * Return the id of the recepient
   */
public String getTo() {
	return to;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isAsync() {
	return async;
}
/**
   * Return true if the message is empty.
   */
public boolean isEmpty() {
	return message.isEmpty();
}
/**
   * Return true if message is a request.
   */
public boolean isRequest() {
	return request;
}
/**
 * This method was created by a SmartGuide.
 * @return boolean
 */
public boolean isSync() {
	return !async;
}
}