package mct.mediator;


/**
 *@(#)MessagingProtocol           Gatot Susilo    August 17, 1997
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.net.*;
import java.io.*;
/**
 * The class is used for facilitator and mediator communication.
 * This provides only for transmitting because the receiver has many
 * customized implementation. However, this gives a good template on
 * what is going to be received on receiver.
 */
public class MessagingProtocol {
	//**********
	//* Fields *
	//**********
	//Message content
	public static byte MCD_RECORD = (byte) 0x00;
	public static byte MC_RECORD = (byte) 0x01;
	public static byte LOCATION = (byte) 0x02;
	public static byte MESSAGE = (byte) 0x03;
	public static byte STATUS = (byte) 0x04;
	public static byte ID = (byte) 0x05;
	public static byte EOF = (byte) 0x7F;

	//Message Type (GET/SET/ACK)
	public static byte GET = (byte) 0x80;
	public static byte SET = (byte) 0x81;
	public static byte ACK = (byte) 0x73;
	public static byte ERROR = (byte) 0x74;
	public static byte NO_ERROR = (byte) 0x75;
	public static byte DONT_EXIST = (byte) 0x76;
	private Socket socket = null;
	private DataInputStream dis = null;
	private DataOutputStream dos = null;
	private ObjectOutputStream oos = null;
	private ObjectInputStream ois = null;



/**
   * Create MessagingProtocol with ip, and port specified
   */
public MessagingProtocol(InetAddress ip, int port) throws IOException {
	socket = new Socket(ip, port);
	dis = new DataInputStream(socket.getInputStream());
	dos = new DataOutputStream(socket.getOutputStream());
	oos = new ObjectOutputStream(socket.getOutputStream());
	ois = new ObjectInputStream(socket.getInputStream());
}
//****************
//* Constructors *
//****************
/**
   * Create MessagingProtocol with socket
   */
public MessagingProtocol(Socket socket) throws IOException {
	this.socket = socket;
	dis = new DataInputStream(socket.getInputStream());
	dos = new DataOutputStream(socket.getOutputStream());
	oos = new ObjectOutputStream(socket.getOutputStream());
	ois = new ObjectInputStream(socket.getInputStream());
}
public void close() {
	try {
		if (dis != null) {
			dis.close();
			dis = null;
		}
		if (dos != null) {
			dos.close();
			dos = null;
		}
		if (oos != null) {
			oos.close();
			oos = null;
		}
		if (ois != null) {
			ois.close();
			ois = null;
		}
		if (socket != null) {
			socket.close();
			socket = null;
		}
	} catch (IOException e) {
		e.printStackTrace();
	}
}
/**
   * Return DataInputStream handle
   */
public DataInputStream getDataInputStream() {
	return dis;
}
/**
   * Return DataOutputStream handle
   */
public DataOutputStream getDataOutputStream() {
	return dos;
}
/**
   * Return ObjectInputStream handle
   */
public ObjectInputStream getObjectInputStream() {
	return ois;
}
/**
   * Return ObjectOutputStream handle
   */
public ObjectOutputStream getObjectOutputStream() {
	return oos;
}
/**
   * Return Socket handle
   */
public Socket getSocket() {
	return socket;
}
/**
   * Send the status of the mobile code with specified id.<br>
   * Format: <br>
   * SET/GET      (byte)<br>
   * STATUS       (byte)<br>
   * value if SET (String)<br>
   * EOF          (byte)<br>
   */
public synchronized void send(String id, byte status, boolean isSET) throws IOException {
	System.out.println("1");
	if (isSET)
		dos.writeByte(SET);
	else
		dos.writeByte(GET);
	System.out.println("3");
	dos.writeByte(STATUS);
	System.out.println("4");
	dos.writeUTF(id);
	System.out.println("5");
	if (isSET)
		dos.writeByte(status);
	dos.writeByte(EOF);
}
/**
   * Send the curent location of the mobile code with specified id.<br>
   * location: ip_address:tcp_port:udp_port:fac_port<br>
   * Format: <br>
   * SET/GET      (byte)<br>
   * LOCATION     (byte)<br>
   * value if SET (String)<br>
   * EOF          (byte)<br>
   */
public synchronized void send(String id, String location, boolean isSET) throws IOException {
	if (isSET) {
		dos.writeByte(SET);
	} else {
		dos.writeByte(GET);
	}
	dos.writeByte(LOCATION);
	dos.writeUTF(id);
	if (isSET) {
		dos.writeUTF(location);
	}
	dos.writeByte(EOF);
	dos.flush();
}
/**
   * The message is always pushed into either the Facilitator or the Mediator<br>
   * Format: <br>
   * SET      (byte)<br>
   * MESSAGE  (byte)
   * value    (Message)<br>
   * EOF      (byte)<br> 
   */
public synchronized void send(Message message) throws IOException {
	dos.writeByte(SET);
	dos.writeByte(MESSAGE);
	oos.writeObject(message);
	dos.writeByte(EOF);
	dos.flush();
}
/**
   * Sending MobileCodeRecord.<br>
   * Format: <br>
   * SET/GET      (byte)<br>
   * MC_RECORD    (byte)<br>
   * value if SET (MobileCodeRecord)<br>
   * EOF          (byte)<br> 
   */
public synchronized void send(MobileCodeRecord mcRecord, boolean isSET) throws IOException {
	if (dos == null)
		System.out.println("Protocol ERROR :send with no dos");
	if (isSET)
		dos.writeByte(SET);
	else
		dos.writeByte(GET);
	dos.writeByte(MC_RECORD);
	oos.writeObject(mcRecord);
	dos.writeByte(EOF);
	dos.flush();
}
/**
   * Sending Acknowledgement.<br>
   * Format: <br>
   * ACK          (byte)<br>
   * tag          (byte)<br>
   * EOF          (byte)<br> 
   */
public synchronized void sendAck(byte tag) throws IOException {
	dos.writeByte(ACK);
	dos.writeByte(tag);
	dos.writeByte(EOF);
	dos.flush();
}
  /** 
   * Return tag of acknowledgement.
   */
  public synchronized byte waitACK() throws IOException {
	if (dis.readByte()!=ACK) throw new IOException ("Invalid tag");
	byte tag=dis.readByte();
	dis.readByte();//discard the EOF;
	return tag;
  }  
}