package mct.mediator;


/**
 *@(#)MobileCodeDatabase.java       July 21, 1997  Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.*;

/**
 * It's database for the mobile code that contains information
 * about its attribute (status, identity, location,etc) and its mailbox.
 */
public class MobileCodeDatabase {
	//**********
	//* Fields *
	//**********
	//Status
	/**
	   * The status of mobile code that does not exist yet in the network
	   */
	public static final byte DONT_EXIST = 0x00;
	/**
	   * The status of mobile code that has been instantiated.
	   */
	public static final byte RUNNING = 0x01;
	/**
	   * The status of mobile code that is about migrating
	   */
	public static final byte MIGRATING = 0x02;

	//database
	private Hashtable table; // this table does not store messages
	private Hashtable mailbox; // special table for a vector of messages













//****************
//* Constructors *
//****************
/**
   * Create a table for mobile code's attribute and a table for mailbox.
   */
public MobileCodeDatabase() {
	table = new Hashtable();
	mailbox = new Hashtable();
}
/**
   * Add an entry for the mobile code, if it already exists,
   * then it will be replaced.
   */
public void addMobileCodeRecord(String id, String name, String alias, String type, String info, String location, byte status, boolean hasMessagingAccess) {
	MobileCodeRecord mcRecord = new MobileCodeRecord();
	mcRecord.id = id;
	mcRecord.className = name;
	mcRecord.alias = alias;
	mcRecord.type = type;
	mcRecord.info = info;
	mcRecord.location = location;
	mcRecord.status = status;
	mcRecord.hasMessagingAccess = hasMessagingAccess;
	addMobileCodeRecord(mcRecord);
}
/**
   * Add an entry for the mobile code, if it already exists,
   * then it will be replaced.
   */
public void addMobileCodeRecord(MobileCodeRecord mcRecord) {
	if (mcRecord == null)
		return;
	if (table.containsKey(mcRecord.id)) {
		String tName = ((MobileCodeRecord) table.get(mcRecord.id)).className;
		String tAlias = ((MobileCodeRecord) table.get(mcRecord.id)).alias;
		mct.Console.warning(this, "the mobile code id: " + mcRecord.id + " name: " + tName + " alias: " + tAlias + " already exists and will be replaced with" + " a mobile code name: " + mcRecord.className + " alias: " + mcRecord.alias);
	}
	table.put(mcRecord.id, mcRecord);
}
/**
   * Get messages of a mobile code whose id is specified in the parameter
   *
   * @param id the id of mobile code.
   * @return array of messages if any; otherwise return null
   */
public Message[] getMessages(String id) {
	if (mailbox.containsKey(id)) {
		Vector vector = (Vector) mailbox.remove(id);
		if (vector.isEmpty())
			return null;
		Message messages[] = new Message[vector.size()];
		vector.copyInto(messages);
		return messages;
	}
	return null;
}
/**
   * Get a mobile code record whose id is specified in the parameter
   *
   * @param id the id of mobile code.
   */
public MobileCodeRecord getMobileCodeRecord(String id) {
	return (MobileCodeRecord) table.get(id);
}
/**
   * Return true if the mobile code whose id is specified in 
   * the parameter has messaging access
   */
public boolean hasMessagingAccess(String id) {
	if (table.get(id) == null)
		return false;
	MobileCodeRecord mcRecord = (MobileCodeRecord) table.get(id);
	return mcRecord.hasMessagingAccess;
}
/**
   * Return registered mobile code. Enumeration of MobileCodeRecord
   */
public Enumeration mobileCodeElements() {
	return table.elements();
}
/**
   * Return IDs of mobile code. Enumeration of String
   */
public Enumeration mobileCodeIDs() {
	return table.keys();
}
/**
   * Put a message of a mobile code whose id is specified in the parameter
   *
   * @param mesg the message
   * @param id the id of mobile code.
   * @return array of messages if any; otherwise return null
   */
public void putMessage(String id, Message mesg) {
	if (mailbox.get(id) == null)
		mailbox.put(id, new Vector());
	Vector vector = (Vector) mailbox.get(id);
	vector.addElement(mesg);
}
/**
   * Remove a mobile code record whose id is specified in the parameter
   *
   * @param id the id of mobile code.
   */
public void removeMobileCodeRecord(String id) {
	table.remove(id);
	mailbox.remove(id);
}
/**
   * To enable/disable the messaging access of mobile code.
   * To enable the messaging access, the mobile code must implement
   * interface mct.users.MessagingAccess.
   * @param id the id of mobile code
   * @param hasMessagingAccess true if the mobile code implements interface MessagingAccess
   * @see mct.users.MessagingAccess
   */
public void setMessagingAccess(String id, boolean hasMessagingAccess) {
	if (table.get(id) == null)
		return;
	MobileCodeRecord mcRecord = (MobileCodeRecord) table.get(id);
	mcRecord.hasMessagingAccess = hasMessagingAccess;
}
}