package mct.mediator;


/**
 * @(#) MobileCodeDirectory.java	1.0 October-13-1997	Cheryl Schramm
 *
 *  A MobileCodeDirectory is a database of MobileCodeRecords,
 *  identifying a set of netlets and their location.
 *
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import mct.users.*;
import mct.admin.*;
import mct.net.*;
public class MobileCodeDirectory extends VirtualManagedComponent {

	public static final int MCD_FORCE = -1;

	//**********
	//* Fields *
	//**********

	private Hashtable table;
	private Hashtable events;



/**
   * Construct an empty directory
   */
public MobileCodeDirectory() {
	table = new Hashtable();
	events = new Hashtable();
}
/**
   * Add an entry for a mobile code, replacing the current entry
   * if it exists
   * @param	mcr	The entry for the mobileCode
   */
protected synchronized boolean addMobileCodeRecord(MobileCodeRecord mcr, int event, boolean force) {
	if (mcr == null)
		return false;
	if (table.containsKey(mcr.id)) {
		String tName = ((MobileCodeRecord) table.get(mcr.id)).className;
		String tAlias = ((MobileCodeRecord) table.get(mcr.id)).alias;
		mct.Console.debug(this, "the mobile code id: " + mcr.id + " name: " + tName + " alias: " + tAlias + "location: " + mcr.location + " already exists and will be replaced with" + " a mobile code name: " + mcr.className + " alias: " + mcr.alias + "location : " + mcr.location);
	}
	if (!events.containsKey(mcr.id))
		events.put(mcr.id, new Integer(-1));
	Integer eventNumber = (Integer) events.get(mcr.id);
	mct.Console.debug(this, "ADD:" + mcr.id + " o(" + eventNumber.intValue() + ") n(" + event + ")");
	if (eventNumber.intValue() < event || event == MCD_FORCE || force) {
		table.put(mcr.id, mcr);
		events.put(mcr.id, new Integer(event));
		return true;
	} else
		return false;
}
/**
   * Get the MobileCodeRecord for the given mobileCode
   *
   * @param id	Name of the mobilecode, as stored in the MobileCodeRecord
   * @returns	The associated MobileCodeRecord, if present
   *		Null, if not present
   */
public synchronized MobileCodeRecord getMobileCodeRecord(String id) {
	return (MobileCodeRecord) table.get(id);
}
/**
 * This method was created by a SmartGuide.
 * @return java.util.Hashtable
 */
public Hashtable getTable() {
	return table;
}
/**
   * Indicates whether the given mobile code implements the toolkit's
   * MessagingAccess interface, meaning that it can send/receive messages
   * via the Facilitator
   * @param	id	Name of the mobile code, as stored in the MobileCodeRecord
   * @returns	True if netlet supports messaging
   */
public synchronized boolean hasMessagingAccess(String id) {
	if (table.get(id) == null)
		return false;
	MobileCodeRecord mcRecord = (MobileCodeRecord) table.get(id);
	return mcRecord.hasMessagingAccess;
}
/**
 * The basic MobileCodeDirectory is always available
 * @return boolean
 */
public boolean isAvailable() {
	return true;
}
/**
   * Returns all entries in the directory
   * @returns	Enumeration of MobileCodeRecord
   */
public synchronized Enumeration mobileCodeElements() {
	return table.elements();
}
/**
   * Returns a listing of all ids present in the directory
   * @returns	Enumeration of String ids, from all MobileCodeRecords in the directory
   */
public synchronized Enumeration mobileCodeIDs() {
	return table.keys();
}
/**
   * Remove the MobileCodeRecord for the given mobileCode
   * @param	id	Name of mobilecode, as stored in the MobileCodeRecord
   * @param event int
   * @param force boolean, if true we force the removal of the record
   */
protected synchronized boolean removeMobileCodeRecord(String id, int event, boolean force) {
	MobileCodeRecord mcr = getMobileCodeRecord(id);
	Integer eventNumber = (Integer) events.get(id);
	if (mcr != null) {
		if (event > eventNumber.intValue() || event == MCD_FORCE || force) {
			events.put(id, new Integer(event));
			table.remove(id);
			mct.Console.debug(this, "REM:" + mcr.id + " o(" + eventNumber + ") n(" + event + ")");
			return true;
		} else
			return false;
	} else {
		events.put(id, new Integer(event));
		return false;
	}
}
/**
   * To enable/disable the messaging access of mobile code.
   * To enable the messaging access, the mobile code must implement
   * interface mct.users.MessagingAccess.
   * @param id the id of mobile code
   * @param hasMessagingAccess true if the mobile code implements interface MessagingAccess
   * @see mct.users.MessagingAccess
   */
protected synchronized void setMessagingAccess(String id, boolean hasMessagingAccess) {
	if (table.get(id) == null)
		return;
	MobileCodeRecord mcRecord = (MobileCodeRecord) table.get(id);
	mcRecord.hasMessagingAccess = hasMessagingAccess;
}
/**
 * This method was created by a SmartGuide.
 * @param title java.lang.String
 */
public void setTitle(String title) {
	return;
}
}