package mct.mediator;


import mct.users.MobileCode;
import mct.users.MobileCodeObject;
import mct.users.MCIdentifier;
import mct.users.MessagingAccess;
import java.net.InetAddress;

/**
 *@(#)MobileCodeRecord.java       July 21, 1997 Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.io.Serializable;

/**
 * Record of Mobile Code.
 */
public class MobileCodeRecord implements Serializable {

	// CHERYL : The state space of the lifecycle of 
	//	a mobile code needs to be expanded.
	//	We must also include the notion of message orders
	//	to handle misordered updates due to network delays
	//
	/**
	   * The status of mobile code that does not exist yet in the network
	   */
	public static final byte DONT_EXIST = 0x00;
	/**
	   * The status of mobile code that has been instantiated.
	   */
	public static final byte RUNNING = 0x01;
	/**
	   * The status of mobile code that is about migrating
	   */
	public static final byte MIGRATING = 0x02;


	//Fields
	/**
	   * The unique id of mobile code. The id should never duplicate with others
	   * in the network.
	   */
	public String id = null;

	/**
	   * The absolute class name of mobile code
	   */
	public String className = null;

	/**
	   * Alias name of mobile code
	   */
	public String alias = null;

	/**
	   * The type of mobile code
	   */
	public String type = null;

	/**
	   * Description about the mobile code
	   */
	public String info = null;

	/**
	   * Current location of mobile code that has format as following:<br>
	   * ip_address:tcp_port:udp_port:fac_port<p>
	   * ip_address=the ip address of the mobile code daemon<br>
	   * tcp_port=the port number for tcp at which the daemon listening<br>
	   * udp_port=the port number for udp at which the daemon listenting<br>
	   * fac_port=the port number for facilitator of the daemon.<br>
	   */
	public String location = null;

	/**
	   * The status of mobile code.
	   */
	public byte status = 0x00;

	/**
	   * Return true if mobile code implements interface MessagingAccess 
	   */
	public boolean hasMessagingAccess = false;




/** Default constructor initializes all fields to null values
   */
public MobileCodeRecord() {
	this.id = null;
	this.alias = null;
	this.className = null;
	this.type = null;
	this.info = null;
	this.location = null;
	this.status = 0x00;
	this.hasMessagingAccess = false;
}
/** Constructs a filled MobileCodeRecord, extracting fields from
   *  the MobileCode, and constructing the location from the other parameters
   *  @param	mc
   *  @param	tcp_port=the port number for tcp at which the daemon listening
   *  @param	udp_port=the port number for udp at which the daemon listenting
   *  @param	fac_port=the port number for facilitator of the daemon.
   */
public MobileCodeRecord(MobileCodeObject mc, int tcpPort, int udpPort, int facPort) {
	this(mc, null);
	this.location = getLocation(tcpPort, udpPort, facPort);
}
/** Constructs a filled MobileCodeRecord, extracting fields from
   *  the MobileCode
   *  @param	 mc
   *  @param location     Current location with the following format : 
   *         ip_address:tcp_port:udp_port:fac_port
   */
public MobileCodeRecord(MobileCodeObject mc, String location) {
	if (mc != null) {
		this.id = mc.getID();
		this.info = mc.getInfo();
		this.className = mc.getClass().getName();
		this.type = mc.getType();
		this.alias = mc.getAlias();
		this.location = location;
		this.status = MobileCodeRecord.DONT_EXIST;
		if (mc instanceof MessagingAccess) {
			this.hasMessagingAccess = true;
		} else {
			this.hasMessagingAccess = false;
		}
	}
}
public static String getLocation(int tcpPort, int udpPort, int facPort) {
	String location;
	try {
		byte bytes[] = InetAddress.getLocalHost().getAddress();
		String host = (bytes[0] & 0xFF) + "." + (bytes[1] & 0xFF) + "." + (bytes[2] & 0xFF) + "." + (bytes[3] & 0xFF);
		location = host + ":" + tcpPort + ":" + udpPort + ":" + facPort;
	} catch (Throwable e) {
		location = null;
	}
	return location;
}
public String toString() {
	return className + "." + id + "@" + location;
}
}