package mct.net;


/**
 * @(#)Ctp.java  December 26, 1996 Gatot Susilo
 *  Revision:
 *    Oct 5, 1997, Gatot Susilo
 *      - raise exception if any error occurs.
 *    May 24, 1997, Gatot Susilo
 *      - the package is renamed to mck from pmc
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.io.*;
import java.net.*;
import java.util.*;
import mct.admin.MobileCodeContainer;
import mct.admin.StorageManager;
import mct.admin.Instantiator;

/**
 * The Ctp is a Code Transfer Protocol that handles the codes'
 * shipment from one Netlet Daemon (i.e. instance of Agent Daemon)
 * to another. The protocol is made as simple and fast as possible
 * to do its job. A Ctp may use either UDP or TCP as underlying 
 * protocol. Both share a common protocol data format (i.e. Protocol
 * Data Unit (PDU)) that is encoded and decoded by the class CtpEncode 
 * and the class CtpDecode respectively. For now, there are only 
 * two types of PDUs:i) REQUEST to set codes and/or parameters on 
 * the designated Netlet Daemon, and ii) RESPONSE to send either 
 * positive or negative acknowledment to the sender (the Netlet 
 * Daemon) and/or to the origin (the Simulator, the Network Management 
 * Station)/<p>
 *
 * Whenever possible, the TCP stream is desireable. This is simply 
 * because the TCP stream is much more reliable than the UDP. 
 * Moreover, the TCP stream can tranfer a big PDU at once. For now
 * the UDP can only tranfer a PDU with maximum size MAX_PDU bytes.<p>
 * 
 * The following is the format of PDU for REQUEST and RESPONSE:
 * <pre>
 ***********
 * REQUEST *
 ***********
 *    magic_number    = u2; 0xC0CA
 *    major_version   = u1; 0x01
 *    minor_version   = u1; 0x00
 *    pdu_type        = u1; (0x00-0x0F=response, 0x10-0xFF=request)
 *    request_id      = u4; (random number)
 *    package_name    = <String>; write as UTF-8;
 *    ack_tag         = u1; 
 *    ip              = u4; (32-bit ip number).
 *    port            = u4; (integer).
 *
 *    persistent_state_tag = u1; 
 *    size            = u4; (size of bytes)
 *    bytes               ;
 *    mobile_code_container_tag = u1;
 *    size            = u4; (size of bytes)
 *    bytes               ;
 *    identity_tag    = u1;
 *    count           = u2; (number of identities)
 *    size            = u4; (size of bytes)
 *    bytes
 *    eof_tag         = u1;
 *
 * pdu_type:
 *    0x10 = set
 *    0x11 = set+overwrite (not implemented yet)
 *
 * tag:
 *    ACK_TCP_TAG   = u1; 0x01; request ack. via TCP
 *    ACK_UDP_TAG   = u1; 0x02; request ack. via UDP
 *    PARAMETER_TAG = u1; 0x03; followed by the parameter entries
 *    CLASS_TAG     = u1; 0x04; followed by the class entries 
 *    EOF_TAG       = u1; 0x05; end of REQUEST PDU
 *
 ************
 * RESPONSE *
 ************
 *    magic_number  = u2; 0xC0CA
 *    major_version = u1; 0x01
 *    minor_version = u1; 0x00
 *    pdu_type      = u1; (0x00-0x0F=response, 0x10-0xFF=request)
 *    request_id    = u4; (correspond to (Request) request_id)
 *    mobileCode_name   = <String>; write as UTF-8
 *    message_id    = u1;
 *    eof_tag       = u1;
 *
 * pdu_type:
 *    0x00 = response
 *
 * message_id:  
 *    0x00 = no errors
 *    0x01 = agent exists, overwrite
 *    0x02 = agent exists, cancel
 *    0x03 = passive bytecode exists, overwrite
 *    0x04 = passive bytecode exists, cancel
 *    0x05 = error(s)
 * </pre>
 **************************************************************
 *
 * When the changes of the CTP behaviour are required, you 
 * should modify this class to suit your application need.
 * However, the class CtpEncode and CtpDecode should remain the
 * same, unless you define a new data type to transport.
 */

public class Ctp implements CtpConstants {
	// The default is using a TCP
	private boolean TCP = true;

	// TCP stream and its socket handler
	private Socket tcpSocket = null;
	private InputStream inStream = null;
	private OutputStream outStream = null;

	// UDP packet and its socket handler
	private DatagramSocket udpSocket = null;

	// The destination of the code to ship to
	private InetAddress ip;
	private int port;

	// Others/Miscelleneous
	int requestId;
	CtpEncode encodePdu = null;
	CtpDecode decodePdu = null;

	// Specify mobileCodeName
	String mobileCodeName = "";



/**
   * Similar to the constructor Ctp(Socket socket), but
   * it uses the UDP as underlying transport protocol.
   * @param socket the datagram socket
   */
public Ctp(DatagramSocket socket) {
	udpSocket = socket;
	TCP = false;
}
/**
   * Similar to the previous constructor, but you provide
   * the host destination via TCP that will be created
   * by this constructor.
   * @param ip the ip of the host destination
   * @param port the port number of the host destination
   * @throw IOException if connection cannot be made. 
   */
public Ctp(InetAddress ip, int port) throws IOException {
	this(ip, port, true);
}
/**
   * Similar to the previous one, but you may choose
   * the underlying transport protocol either TCP or UDP.
   * @param ip the ip of the host destination
   * @param port the port number of the host destination
   * @param TCP true if using tcp; false if using UDP.
   * @throw IOException if connection cannot be made.
   */
public Ctp(InetAddress ip, int port, boolean TCP) throws IOException {
	this.TCP = TCP;
	this.ip = ip;
	this.port = port;
	if (TCP) {
		tcpSocket = new Socket(ip, port);
		inStream = tcpSocket.getInputStream();
		outStream = tcpSocket.getOutputStream();
	} else {
		try {
			udpSocket = new DatagramSocket();
		} catch (SocketException ex) {
			mct.Console.error(this, "Socket cannot bind to port");
			throw new IOException();
		}
	}
}
/**
   * This constructor is used when you want either to send
   * or to receive a PDU to/from the Socket connection that you 
   * have made. This Ctp will used the TCP stream as underlying
   * transport protocol.
   * @param socket the TCP socket connection that you have made
   * @throw IOException if connection cannot be made.
   */
public Ctp(Socket socket) throws IOException {
	tcpSocket = socket;
	TCP = true;
	inStream = tcpSocket.getInputStream();
	outStream = tcpSocket.getOutputStream();
}
/**
 * This method was created by a SmartGuide.
 */
public void close() throws IOException {
	if (TCP) {
		if (tcpSocket != null) {
			tcpSocket.close();
			tcpSocket = null;
		}
	} else {
		if (udpSocket != null) {
			udpSocket.close();
			udpSocket = null;
		}
	}
}
// Just print the string to standard output
private void printSocketDisconnected(Exception ex) {
	mct.Console.warning(this, ex.toString());
	try {
		close();
	} catch (IOException e) {
	};
}
/**
   * Receive a netlet and store it on the corresponding
   * netlet container that is managed by the storage manager.
   * This will be handled automatically by Instantiator.
   * @param instantiator the instantiator handler.
   * @return the handle of MobileCodeContainer
   */
public MobileCodeContainer receive(Instantiator instantiator) throws CtpException {
	if (TCP)
		return receiveTcp(instantiator);
	else
		return receiveUdp(instantiator);
}
/**
   * To receive a PDU at a certain port of UDP.
   * This might be useful when you want to receive the packet
   * at different port.
   * @param instantiator the instantiator handler
   * @param port the port number where the UDP opens
   * @return the handle of MobileCodeContainer
   */
public MobileCodeContainer receive(Instantiator instantiator, int port) throws CtpException {
	DatagramSocket dSocket = null;
	try {
		dSocket = new DatagramSocket(port);
		return receiveUdp(instantiator);
	} catch (SocketException ex) {
		mct.Console.error(this, "Socket cannot bind to port");
	} finally {
		dSocket.close();
	}
	return null;
}
// Receive a netlet via TCP and send either positive or
// negative acknowledgment to sender and/or to origin
// (i.e. the simulator or the network management station)
private MobileCodeContainer receiveTcp(Instantiator instantiator) throws CtpException {
	MobileCodeContainer mc = null;
	//System.out.println ("Ctp: receiving TCP and decode it");

	try {
		// Decode the PDU from the underlying TCP stream
		decodePdu = new CtpDecode(inStream);
		if (decodePdu == null)
			throw new CtpException("PDU decoding error");

		// Decoding has been done so get the contents
		// If netlet already exists, so just use it and override
		// existing parameters and classes
		// mobileCodeName = decodePdu.getMobileCodeName();
		mc = decodePdu.getMobileCodeContainer();
		if (mc != null)
			instantiator.addElement(mc);

		// Send the positive acknowledgment to sender 
		requestId = decodePdu.getRequestId();
		encodePdu = new CtpEncode(outStream, RESPONSE, mobileCodeName);
		encodePdu.send(NO_ERRORS, requestId);
	} catch (EOFException ex) {
		mct.Console.warning(this, "PDU is received incomplete...");

		// Send the negative acknowledgement to sender
		try {
			encodePdu = new CtpEncode(outStream, RESPONSE, mobileCodeName);
			encodePdu.send(ERRORS, requestId);
		} catch (IOException e) {
			printSocketDisconnected(e);
			throw new CtpException("Encode NAK error " + e.toString());
		}
	} catch (IOException ex) {
		printSocketDisconnected(ex);
		throw new CtpException("TCP error" + ex.toString());
	}

	// Send acknowledgement to origin(corntrol) if requested
	if (decodePdu.isAck())
		sendAckToOrigin();
	return mc;
}
// Receive a netlet via UDP and send either positive or
// negative acknowledgment to sender and/or to origin
// (i.e. the simulator or the network management station)
private MobileCodeContainer receiveUdp(Instantiator instantiator) throws CtpException {
	MobileCodeContainer mc = null;

	// The maximum UDP packet size is MAX_PDU bytes
	byte buffer[] = new byte[MAX_PDU];
	DatagramPacket packet = new DatagramPacket(buffer, MAX_PDU);
	try {
		// Receive the UDP packet
		udpSocket.receive(packet);

		// Convert the packet (i.e., a byte array) as input stream
		// hence the CtpDecode can decode the PDU as if it is a TCP
		// stream.
		ByteArrayInputStream bais = new ByteArrayInputStream(packet.getData());
		decodePdu = new CtpDecode(bais);
		if (decodePdu == null)
			throw new CtpException("PDU decoding error");
		// Decoding has been done so get the contents

		// If netlet already exists, so just use it and override
		// existing parameters and classes
		// mobileCodeName = decodePdu.getMobileCodeName();
		mc = decodePdu.getMobileCodeContainer();
		if (mc != null)
			instantiator.addElement(mc);

		// Send a positive acknowledgment to sender via a UDP. 
		requestId = decodePdu.getRequestId();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		encodePdu = new CtpEncode(baos, RESPONSE, mobileCodeName);
		encodePdu.send(NO_ERRORS, requestId);
		DatagramPacket ackPacket = new DatagramPacket(baos.toByteArray(), baos.size(), packet.getAddress(), packet.getPort());
		udpSocket.send(ackPacket);
	} catch (EOFException ex) {
		mct.Console.warning(this, "PDU is received incomplete...");

		// Send a negative acknowledment to sender
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			encodePdu = new CtpEncode(baos, RESPONSE, mobileCodeName);
			encodePdu.send(ERRORS, requestId);
			DatagramPacket ackPacket = new DatagramPacket(baos.toByteArray(), baos.size(), packet.getAddress(), packet.getPort());
			udpSocket.send(ackPacket);
		} catch (IOException e) {
			printSocketDisconnected(e);
			throw new CtpException("Encode response error " + e.toString());
		}
	} catch (IOException ex) {
		printSocketDisconnected(ex);
		throw new CtpException("UDP I/O error " + ex.toString());
	}

	// Send acknowledgement to origin(corntrol) if requested
	if (decodePdu.isAck())
		sendAckToOrigin();
	return mc;
}
/**
   * Send a mobileCodeContainer as one PDU either via TCP or
   * UDP depending on the constructor being used.
   * @param mobileCodeContainer a container that holds classes and parameters
   * @throw CtpException if any errors occur.
   */
public void send(MobileCodeContainer mobileCodeContainer, byte state[]) throws CtpException {
	if (TCP)
		sendTcp(mobileCodeContainer, state);
	else
		sendUdp(mobileCodeContainer, state);
}
// Send acknowledgement either via TCP or via UDP
private void sendAckToOrigin() throws CtpException {
	// Send a positive acknowledgement to origin(control)
	//**
	//** A negative acknowledgement should be handled as well
	//**
	try {
		if (decodePdu.isAckTcp()) {
			Socket ackSocket = new Socket(decodePdu.getAckAddress(), decodePdu.getAckPort());
			encodePdu = new CtpEncode(ackSocket.getOutputStream(), RESPONSE, mobileCodeName);
			encodePdu.send(NO_ERRORS, requestId);
		} else {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			encodePdu = new CtpEncode(baos, RESPONSE, mobileCodeName);
			encodePdu.send(NO_ERRORS, requestId);
			DatagramSocket datagramSocket = new DatagramSocket();
			DatagramPacket packet = new DatagramPacket(baos.toByteArray(), baos.size(), decodePdu.getAckAddress(), decodePdu.getAckPort());
			datagramSocket.send(packet);
		}
	} catch (IOException ex) {
		mct.Console.error(this, "IO error while sending acknowledment to origin...");
		throw new CtpException("ACK error " + ex.toString());
	}
}
// Send a netlet via TCP
private void sendTcp(MobileCodeContainer mobileCodeContainer, byte state[]) throws CtpException {
	// Create CtpEncode and set a PDU type SET (i.e. REQUEST)
	encodePdu = new CtpEncode(outStream, SET);

	// Encode and send the netlet through the TCP stream
	try {
		requestId = encodePdu.send(mobileCodeContainer, state);
	} catch (IOException ex) {
		printSocketDisconnected(ex);
		throw new CtpException("sendTcp: IO error");
	}
}
// Send a netlet via UDP
private void sendUdp(MobileCodeContainer mobileCodeContainer, byte state[]) throws CtpException {
	// Write the output stream as a byte array. The following
	// class is dynamically expandable. This class is used
	// because the CtpEncode is writing into the output stream.
	// From this output stream, it will be converted as 
	// a byte array that complies a PDU format and ready to
	// go as a PDU packet.
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	encodePdu = new CtpEncode(baos, SET);
	try {
		// Encode the netlet into a PDU format
		requestId = encodePdu.send(mobileCodeContainer, state);

		// Prepare a datagram packet
		DatagramPacket packet = new DatagramPacket(baos.toByteArray(), baos.size(), ip, port);

		// Send the packet
		udpSocket.send(packet);
	} catch (IOException ex) {
		mct.Console.error(this, "IO error...");
		throw new CtpException("sendUdp: IO error");
	}
}
}