package mct.net;


/**
 * @(#)CtpConstants.java December 25, 1996 Gatot Susilo
 *  Revision:
 *    May 24, 1997, Gatot Susilo
 *      - it is renamed to mct from pmc.
 *      - it is changed as interface instead of class.
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
/**
 * Consists of CTP's constants.
 */
public interface CtpConstants {
	/**
	   * A magic number to identify CTP packet: 0xC0CA
	   */
	final public byte MAGIC_NUMBER[] = {(byte) 0xC0, (byte) 0xCA};

	/**
	   * A major version of the protocol
	   */
	final public byte MAJOR_VERSION = 0x01;

	/**
	   * A minor version of the protocol
	   */
	final public byte MINOR_VERSION = 0x00;

	/**
	   * Maximum size of datagram packet.
	   */
	final public int MAX_PDU = 100000; // bytes
	//************
	//* PDU Type *
	//************
	/**
	   * Request the AgentDaemon to set a netlet that is carried by the CTP
	   */
	final public byte SET = 0x10;

	/**
	   * Request the Agent Daemon to set a netlet whether the netlet already
	   * exists or not.
	   */
	final public byte SET_OVERWRITE = 0x11;

	/**
	   * To mask the PDU type to determine whether the PDU is REQUEST or RESPONSE
	   * If ((pduType & RESPONSE_MASK)==0) then the PDU type is a RESPONSE
	   */
	final public byte RESPONSE_MASK = (byte) 0xF0;

	/**
	   * A PDU type of RESPONSE
	   */
	final public byte RESPONSE = 0x00; // valid response id: 0x00-0x0F

	//**************
	//* Message id *
	//**************
	/**
	   * Message Id: no error(s) occur(s)
	   */
	final public byte NO_ERRORS = 0x00;

	/**
	   * Message Id: The mobile code exists and overwrites exisiting mobile code.<p>
	   * This message occurs when the REQUEST pdu type is SET_OVERWRITE.
	   */
	final public byte MOBILE_CODE_EXIST_OVERWRITE = 0x01;

	/**
	   * Message id: The mobile code exists and cancels setting the agent.<p>
	   * This message occurs when the REQUEST pdu type is SET.
	   */
	final public byte MOBILE_CODE_EXIST_CANCEL = 0x02;

	/**
	   * Message Id: General errors occur
	   */
	final public byte ERRORS = 0x05;

	//****************
	//* Content TAGS *
	//****************
	/**
	   * The content type of each messages that constitutes the CTP packet 
	   * is marked with its appropriate tag. 
	   * @see mct.net.Ctp for further detail of the Ctp format.
	   */
	// Specify to where the acknowlegment should go to:
	final public byte ACK_TCP_TAG = 0x01;
	// followed by: ip number (InetAddress), and port number (int)

	final public byte ACK_UDP_TAG = 0x02;
	// followed by: ip number (InetAddress), and 4-byte port number (int)

	final public byte PERSISTENT_STATE_TAG = 0x03;
	// followed by: size and bytes

	final public byte IDENTITY_TAG = 0x04;
	// followed by: name, size, and bytes
	//    This refers to Public Key of the signer

	final public byte MOBILE_CODE_CONTAINER_TAG = 0x05;
	// followed by: size, and bytes

	final public byte MESSAGE_TAG = 0x06;
	// followed by: number of strings, String array

	final public byte EOF_TAG = 0x0A;



}