package mct.net;


/**
 * @(#)CtpDecode.java December 26, 1996 Gatot Susilo
 *  Revision:
 *    May 24, 1997, Gatot Susilo
 *      - the package is renamed to mck from pmc
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.EOFException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.security.Identity;
import java.net.InetAddress;
import mct.admin.MobileCodeContainer;
import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;

/**
 * This class decodes the Ctp PDU format that is shipped
 * via either TCP or UDP. The CtpDecode treats the PDU as
 * InputStream.<p>
 * If it is used for TCP, the inStream is typically 
 * Socket.getInputStream(). If it is used for UDP,
 * the byte array from UDP is converted into InputStream
 * by using class ByteArrayInputStream.<p>
 *
 * @see mct.net.CtpEncode
 * @author Gatot Susilo
 * @version 1.0
 */
public class CtpDecode implements CtpConstants {
	// magicNumber and version of CTP
	private byte magicNumber[] = new byte[2];
	private byte majorVersion;
	private byte minorVersion;

	// the PDU type
	private byte pduType;

	// the request id
	private int requestId;

	// if it is a RESPONSE pdu format
	private int messageId;
	private String mobileCodeName = "";

	// if the client asks for acknowledgement to designated host
	// either via TCP or UDP
	private boolean ack;
	private boolean tcp;
	private InetAddress ip;
	private int port;

	// Data from Decoding
	private MobileCodeContainer mobileCodeContainer = null;
	private Vector identities = null;
	private byte state[] = null;

	// Read the data with DataInputStream
	private DataInputStream dis;



/**
   * Contruct the class and start to decode the inStream
   * @throw CtpException
   * @throw IOException
   */
public CtpDecode(InputStream inStream) throws IOException, CtpException {
	dis = new DataInputStream(inStream);
	tcp = false;
	ack = false;
	decode();
}
/**
   * Decode the PDU into corresponding information.
   * @throw CtpException
   * @throw IOException
   */
private void decode() throws IOException, CtpException {
	//System.out.println ("CTPDecode: start decoding and waiting magic number");

	// read magic number, and version
	dis.readFully(magicNumber);
	//System.out.println ("Receive: MN:"+ magicNumber);
	majorVersion = dis.readByte();
	minorVersion = dis.readByte();
	//System.out.println ("Receive: Version: "+majorVersion+"."+minorVersion);

	// read pdu type
	pduType = dis.readByte();
	//System.out.println ("Receive PDU: "+pduType);

	// read request id
	requestId = dis.readInt();
	//System.out.println ("Receive req. ID: "+ requestId);

	// read netlet name
	mobileCodeName = dis.readUTF();

	//System.out.println ("Magic:"+majorVersion+"."+minorVersion);
	//System.out.println ("PDU Type: "+pduType+" RequestID: "+requestId);
	//System.out.println ("MobileCodeName: "+mobileCodeName);
	// if it's response PDU, just read messageId and stop
	if ((pduType & RESPONSE_MASK) == (byte) 0x00) {
		messageId = dis.readByte();
	}
	// this is request PDU
	else {
		boolean eof = false;
		while (!eof) {
			try {
				byte tag = dis.readByte();

				// handle the ACK request
				if ((tag == ACK_TCP_TAG) || (tag == ACK_UDP_TAG)) {
					//System.out.println ("CtpDecode: ACK_TAG");
					ack = true;
					if (tag == ACK_TCP_TAG)
						tcp = true;

					// Get the ip address
					byte ipBytes[] = new byte[4];
					dis.readFully(ipBytes);
					String host = Integer.toString(ipBytes[0]) + "." + Integer.toString(ipBytes[1]) + "." + Integer.toString(ipBytes[2]) + "." + Integer.toString(ipBytes[3]);
					ip = InetAddress.getByName(host);
					// Get the port number
					port = dis.readInt();
					//System.out.println ("CtpDecode: host:"+host+" port:"+port);
				}
				/*          // Get parameters
				  else if (tag==PARAMETER_TAG) {
				short count=dis.readShort();
				parameterTable=new Hashtable();
				for(short i=0;i<count;i++) {
				  String string=dis.readUTF();
				  int pos=string.indexOf('=');
				  if (pos > 0) {
				String name=string.substring(0,pos);
				String value=string.substring(pos+1);
				parameterTable.put(name,value);
				  } else {
				throw new CtpException();
				  }
				}
				  } 
				  // Get classes (bytecodes)
				  else if (tag==CLASS_TAG) {
				short count=dis.readShort();
				classTable=new Hashtable();
				for (short i=0;i<count;i++) {
				  String name=dis.readUTF();
				  int size = dis.readInt();
				  byte bytes[] = new byte[size];
				  try {
				dis.readFully(bytes);
				  } catch (EOFException ex) {
				throw new CtpException();
				  }
				  // String name = getClassName(bytes);
				  classTable.put(name,bytes);
				}
				  }
				  */
				// Get Identity
				else
					if (tag == IDENTITY_TAG) {
						int length = dis.readShort();
						int size = dis.readInt();
						byte bytes[] = new byte[size];
						dis.readFully(bytes);
						try {
							ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(bytes));
							if (identities == null)
								identities = new Vector();
							for (int i = 0; i < length; i++) {
								Identity identity = (Identity) ois.readObject();
								if (identity != null)
									identities.addElement(identity);
							}
						} catch (Throwable ex) {
							throw new CtpException("Cannot decode Identity");
						}
					}
					// Get Persitent State
					else
						if (tag == PERSISTENT_STATE_TAG) {
							//System.out.println ("PersistentState Tag");
							int size = dis.readInt();
							state = new byte[size];
							dis.readFully(state);
						}

						// Get MobileCodeContainer
						else
							if (tag == MOBILE_CODE_CONTAINER_TAG) {
								//System.out.println ("MCC TAG");
								int size = dis.readInt();
								byte bytes[] = new byte[size];
								dis.readFully(bytes);
								try {
									ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(bytes));
									MobileCodeContainer mcc = (MobileCodeContainer) ois.readObject();
									if (mcc == null)
										throw new CtpException("Cannot decode Mobile Code Container");
									mobileCodeContainer = mcc;
								} catch (Throwable ex) {
									//ex.printStackTrace();
									throw new CtpException("General error on decoding Mobile Code Container");
								}
							} else
								if (tag == EOF_TAG)
									eof = true;
			} catch (EOFException ex) {
				eof = true;
				throw new CtpException("Data is corrupted while decoding...");
			}
		} // end while
		if ((mobileCodeContainer != null) && (state != null))
			mobileCodeContainer.putPersistentState(state);
	} // end if
}
/**
   * Where the acknowledge goes to
   */
public InetAddress getAckAddress() {
	return ip;
}
/**
   * The port number where the acknowledge goes to
   */
public int getAckPort() {
	return port;
}
/**
   * Get the parameters as the result of decoding.
   * @return the parameterTable's handler
   */
public Vector getIdentities() {
	return identities;
}
/**
   * Get the classes as the result of decoding.
   * @return the classTable's handler
   */
public MobileCodeContainer getMobileCodeContainer() {
	return mobileCodeContainer;
}
/**
   * the name of the netlet
   */
public String getMobileCodeName() {
	return mobileCodeName;
}
/**
   * Get the PduType
   */
public byte getPduType() {
	return pduType;
}
/**
   * Get the request id of the received pdu
   */
public int getRequestId() {
	return requestId;
}
/**
   * return true if client requests to acknowledge to 
   * designated host.
   */
public boolean isAck() {
	return ack;
}
/**
   * Acknowledge either via TCP (true) or UDP (false)
   */
public boolean isAckTcp() {
	return tcp;
}
}