package mct.net;


/**
 * @(#)CtpEncode.java December 26, 1996 Gatot Susilo
 *  Revision:
 *    May 24, 1997, Gatot Susilo
 *      - the package is renamed to mck from pmc
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.io.DataOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Random;
import java.util.Enumeration;
import java.net.InetAddress;
import java.security.Identity;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import mct.admin.MobileCodeContainer;

/**
 * Class CtpEncode writes the CTP's PDU request/response format that may be
 * transported by either TCP or UDP. If it is used with TCP,
 * the parameter outStream is typically Socket.getOutputStream().
 * While it is used with UDP, the parameter outStream is 
 * typically ByteArrayOutputStream that may convert the OutputStream
 * as a byte array<p>
 *
 * At the other end, CtpDecode should be used to receive the CTP's PDU
 * request/response format.<p>
 * @see CtpDecode
 * @author Gatot Susilo
 * @version 1.0
 */

public class CtpEncode implements CtpConstants {
	// It's convinience to use the DataOutputStream to write the data
	private DataOutputStream dos;

	// PDU type: 0x00-0x0F response, 0x10-0xff request
	private byte pduType;

	// The mobile code name
	private String mobileCodeName = "";

	// Acknowledge to designated destination via either TCP or UDP
	private boolean ack;
	private boolean tcp;
	private InetAddress ip;
	private int port;

	// The requestID is used to track the PDU being sent
	private int requestId = -1;



/**
   * Construct the CtpEncode to send a whole NetletContainer.
   * The pdu type is typically REQUEST type.
   * @param outStream the designated output stream
   * @param pduType response: 0x00-0x0F; request: 0x10-0xFF
   * @see CtpConstants
   */
public CtpEncode(OutputStream outStream, byte pduType) {
	this(outStream, pduType, null);
}
/**
   * Contstruct the CtpEncode to send any parameters, classes, or both.
   * The pdu type may be a REQUEST or RESPONSE type.
   */
public CtpEncode(OutputStream outStream, byte pduType, String mobileCodeName) {
	dos = new DataOutputStream(outStream);
	this.pduType = pduType;
	this.mobileCodeName = mobileCodeName;
	ack = false;
	tcp = false;
}
public int send(Identity identity[]) throws IOException, CtpException {
	if ((pduType & RESPONSE_MASK) == 0)
		throw new CtpException();
	else {
		try {
			Random random = new Random();
			requestId = random.nextInt();
			sendHeader();
			dos.writeByte(IDENTITY_TAG);
			dos.writeShort(identity.length);
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(baos);
			for (int i = 0; i < identity.length; i++)
				oos.writeObject(identity);
			dos.writeInt(baos.size());
			dos.write(baos.toByteArray());
			dos.writeByte(EOF_TAG);
			dos.flush();
			return requestId;
		} catch (Throwable e) {
			throw new CtpException("Encoding error...");
		}
	}
}
/**
   * Send RESPONSE pdu format.
   * @param messageId type of message
   * @param requestId it corresponds to requestId of REQUEST pdu format.
   * @throw CtpException if pdu type is invalid.
   * @throw IOException.
   */
public void send(byte messageId, int requestId) throws IOException, CtpException {
	if ((pduType & RESPONSE_MASK) != 0)
		throw new CtpException();
	else {
		this.requestId = requestId;
		sendHeader();
		dos.writeByte(messageId);
		dos.writeByte(EOF_TAG);
		dos.flush();
	}
}
/**
   * Send the MobileCodeContainer without persistent state.
   */
public int send(MobileCodeContainer mcc) throws IOException, CtpException {
	return send(mcc, null);
}
/**
   * Send the whole data in NetletContainer. The data is not copied.
   * @see #send
   * @param MobileCodeContainer the handle only
   * @return requestId this will be used on RESPONSE pdu type.
   * @throw CtpException
   * @throw IOException
   */
public int send(MobileCodeContainer mcc, byte state[]) throws IOException, CtpException {
	mobileCodeName = mcc.getMobileCodeName();
	Enumeration enum = mcc.getParameterNames();
	while (enum.hasMoreElements()) {
		String name = (String) enum.nextElement();
	}
	if ((pduType & RESPONSE_MASK) == 0)
		throw new CtpException();
	else {
		// Set random request ID
		Random random = new Random();
		requestId = random.nextInt();

		// Send common header
		//System.out.println ("CtpEncode: sending header");
		sendHeader();

		// If you want to acknowledge to designated host
		if (ack) {
			//System.out.println ("I am sending Ack");
			if (tcp)
				dos.writeByte(ACK_TCP_TAG);
			else
				dos.writeByte(ACK_UDP_TAG);
			dos.write(ip.getAddress(), 0, 4);
			dos.writeInt(port);
		}
		if (state != null) {
			dos.writeByte(PERSISTENT_STATE_TAG);
			dos.writeInt(state.length);
			dos.write(state);
		}
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(mcc);
		dos.writeByte(MOBILE_CODE_CONTAINER_TAG);
		dos.writeInt(baos.size());
		dos.write(baos.toByteArray());
		dos.writeByte(EOF_TAG);
		dos.flush();
		return requestId;
	}
}
/**
   * Send common header used for REQUEST/RESPONSE format
   */
private void sendHeader() throws IOException {
	dos.write(MAGIC_NUMBER, 0, 2);
	dos.writeByte(MAJOR_VERSION);
	dos.writeByte(MINOR_VERSION);
	dos.writeByte(pduType);
	dos.writeInt(requestId);
	dos.writeUTF(mobileCodeName);
}
/**
   * Before you call any <tt>send</tt> methods, you must call this method first 
   * if you want to acknowlege to the designated
   * address either via TCP or UDP upon receipt of this PDU.
   * Moreover, <p>
   * @param tcp true if use TCP transport or false if use UDP instead
   * @param ip the 32-bit Ip address of designated host.
   * @param port the port number of designated host.
   */
public void setAck(boolean tcp, InetAddress ip, int port) {
	ack = true;
	this.tcp = tcp;
	this.ip = ip;
	this.port = port;
}
}