package mct.security;


/**
 * @(#)MCDSecurityManager.java       June 15, 1997   Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.security.PublicKey;
import java.util.Vector;
import java.net.InetAddress;
import java.io.FileDescriptor;
import java.security.*;

/**
 * This is an implementation of security manager of mobile code
 * daemon. It provides a few rules of restrictions that may guard
 * the system from malicious mobile code. With this security manager
 * enabled, the system is considerably safe from attack of the
 * dangereous mobile code.<br>
 * The implemented restrictions are:
 * <ul>
 * <li>Mobile code does not allow to read/write file system
 * <li>Mobile code does not allow to create class loader
 * <li>Only authorized mobile code (i.e., signed by a proper signer) can
 *     access StorageManager and MobileCodeManager
 * <li>etc.
 * </ul>
 */
public class MCDSecurityManager extends java.lang.SecurityManager {
	//**********
	//* Fields *
	//**********
	private String root;
	private Vector mcmUsers, smUsers; // Vector of string
	private IdentityScope iScope = null;



//****************
//* Constructors *
//****************
/**
   * 
   */
public MCDSecurityManager(String root, Vector mcmUsers, Vector smUsers) {
	super();
	this.root = root;
	this.mcmUsers = mcmUsers;
	this.smUsers = smUsers;
	iScope = IdentityScope.getSystemScope();
}
/**
   * There is no restriction.
   */
public void checkAccept(String host, int port) {
}
/**
   * There is no restriction.
   */
public void checkAccess(Thread g) {
}
/**
   * There is no restriction.
   */
public void checkAccess(ThreadGroup g) {
}
/**
   * Nobody allows to access AWT Event Queue
   */
public void checkAwtEventQueueAccess() {
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public void checkConnect(String host, int port) {
}
/**
   * There is no restriction.
   */
public void checkConnect(String host, int port, Object context) {
}
/**
   * Only the daemon can create class loader.
   * can be created.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkCreateClassLoader() {
	if (currentClassLoader() == null)
		return;
	printError("checkCreateClassLoader: " + currentClassLoader());
	throw new SecurityException();
}
/**
   * Only the daemon can delete the file system.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkDelete(String file) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkDelete String " + file);
	throw new SecurityException();
}
/**
   * Only the daemon can execute the command.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkExec(String cmd) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkExec " + cmd);
	throw new SecurityException();
}
/**
   * Only the daemon itself can terminate itself.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkExit(int status) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkExit " + status);
	throw new SecurityException();
}
//***********
//* Methods *
//***********

/**
   * Only authorized users including root can access MobileCodeManager.
   * @param key the public key of authorized user.
   * @throw SecurityException if the user is not authorized.
   */
public void checkGetMobileCodeManager(PublicKey key) {
	if (getThreadGroup().getName().equals("main"))
		return;
	if (iScope != null) {
		Identity identity = iScope.getIdentity(key);
		if (identity != null) {
			String name = identity.getName();
			if (mcmUsers.contains(name) || name.equals(root))
				return;
		}
	}
	printError("checkGetMobileCodeManager PublicKey " + key);
	throw new SecurityException();
}
/**
   * Only authorized users including root can access StorageManager.
   * @param key the public key of authorized user.
   * @throw SecurityException if the user is not authorized.
   */
public void checkGetStorageManager(PublicKey key) {
	if (getThreadGroup().getName().equals("main"))
		return;
	if (iScope != null) {
		Identity identity = iScope.getIdentity(key);
		if (identity != null) {
			String name = identity.getName();
			if (smUsers.contains(name) || name.equals(root))
				return;
		}
	}
	printError("checkGetStorageManager PublicKey " + key);
	throw new SecurityException();
}
/**
   * Only the daemon can access the link of DLL.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkLink(String lib) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkLink " + lib);
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public void checkListen(int port) {
}
/**
   * Only the daemon can access Member whose class is specified in the parameter.
   */
public void checkMemberAccess(Class clazz, int which) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkMemeberAccess Class Int " + clazz + ":" + which);
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public void checkMulticast(InetAddress maddr) {
}
/**
   * There is no restriction.
   */
public void checkMulticast(InetAddress maddr, byte ttl) {
}
/**
   * Nobody allows to access any Package
   */
public void checkPackageAccess(String pkg) {
	printError("checkPackageAccess String " + pkg);
	throw new SecurityException();
}
/**
   * Nobody allows to access any Package
   */
public void checkPackageDefinition(String pkg) {
	printError("checkPackageDefinition String " + pkg);
	throw new SecurityException();
}
/**
   * Nobody allows to print.
   */
public void checkPrintJobAccess() {
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public void checkPropertiesAccess() {
	printMessage("checkPropertiesAccess ");
	return;
}
/**
   * There is no restriction.
   */
public void checkPropertyAccess(String key) {
	printMessage("checkPropertyAccess String " + key);
	return;
}
/**
   * There is no restriction.
   */
public void checkPropertyAccess(String key, String def) {
	printMessage("checkPropertyAccess String String" + key + ":" + def);
	return;
}
/**
   * There is no resctriction because open-socket requires this
   */
public void checkRead(FileDescriptor fd) {
}
/**
   * Only the daemon can read the file system.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkRead(String file) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkRead String " + file);
	throw new SecurityException();
}
/**
   * Only the daemon can read the file system.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkRead(String file, Object context) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkRead String Object " + file + context);
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public void checkSecurityAccess(String provider) {
}
/**
   * There is no restriction.
   */
public void checkSetFactory() {
}
/**
   * Nobody allows to access system clipboard
   */
public void checkSystemClipboardAccess() {
	throw new SecurityException();
}
/**
   * There is no restriction.
   */
public boolean checkTopLevelWindow(Object window) {
	printMessage("checkTopLevelWindow Object " + window);
	return true;
}
/**
   * There is no resctriction because open-socket requires this
   */
public void checkWrite(FileDescriptor fd) {
}
/**
   * Only the daemon can write to the file system.
   *
   * @throw SecurityException if it violates the security restriction(s).
   */
public void checkWrite(String file) {
	if (getThreadGroup().getName().equals("main"))
		return;
	printError("checkWrite String " + file);
	throw new SecurityException();
}
/**
   * Return the current ThreadGroup.
   */
public ThreadGroup getThreadGroup() {
	ThreadGroup tg = Thread.currentThread().getThreadGroup();
	return tg;
}
//******************************************************************************
// Standard from SecurityManager
private void printError(String s) {
	mct.Console.error(this, "Security violation on " + s);
}
private void printMessage(String s) {
	mct.Console.message(this, "Accessed security method: " + s);
}
private void printWarning(String s) {
	mct.Console.warning(this, s);
}
}