package mct.security;


/**
 * @(#) MobileCodeVerifier.java       June 15, 1996 Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import sun.security.pkcs.PKCS7; // implements signed siganture file
import sun.tools.jar.*;
import sun.security.provider.IdentityDatabase;
import sun.security.provider.SystemSigner;
import sun.security.provider.SystemIdentity;
import mct.admin.MobileCodeContainer;
import java.util.zip.*;
import java.security.*;
import java.util.*;
import java.io.*;

/**
 * This is basically a JAR verifier to ensure that it's signed by
 * proper signer. The implementation is based on standard JAR and
 * security provided from SUN's JDK 1.1.x distribution. 
 */
public class MobileCodeVerifier {
	//**********
	//* Fields *
	//**********
	private MobileCodeContainer mcc;
	private IdentityScope identityScope;
	private Hashtable byteCode;
	private boolean passVerifier = false;
	private Vector signers;




/**
   * Constructor.
   * @param mcc the MobileCodeContainer containing JARs will be verified
   * @param identityScope the scope of identity
   */
public MobileCodeVerifier(MobileCodeContainer mcc, IdentityScope identityScope) {
	this.mcc = mcc;
	this.identityScope = identityScope;
	byteCode = new Hashtable();
	signers = new Vector();
}
/**
   * Get the table of classes. This should be called after method
   * verify is called.
   *
   * see #verify
   */
public Hashtable getClassTable() {
	return byteCode;
}
/**
   * Return the valid signers of the JARs if passVerifier is false.
   */
public Vector getSigners() {
	return signers;
}
private String slashToDot(String str) {
	String temp = str.replace('/', '.');
	temp = temp.replace('\\', '.');
	if (temp.charAt(0) == '.')
		temp = temp.substring(1);
	return temp.trim();
}
/**
   * Verify the MobileCodeContaner. It does also to explote the JAR
   * into classes.
   *
   * @param passVerifier true if verification is skipped.
   */
public boolean verify(boolean passVerifier) {
	this.passVerifier = passVerifier;
	try {
		verifyJars();
		return true;
	} catch (Throwable e) {
		mct.Console.error(this, "Exception occurs....");
		e.printStackTrace();
	}
	return false;
}
/**
   * JAR's verification in detail.
   */
public byte[] verifyJar(String jarName, boolean passVerifier) throws IOException, SecurityException {
	byte[] jarBytes = (byte[]) mcc.getJarTable().get(jarName);
	byte[] classByteCodes = null;
	byte buffer[] = new byte[512];
	if (jarBytes == null)
		return classByteCodes;
	JarVerifierStream jvs = new JarVerifierStream(new ByteArrayInputStream(jarBytes));
	ZipEntry entry;
	while ((entry = jvs.getNextEntry()) != null) {
		if (entry.getName().endsWith(".class")) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			int count;
			while ((count = jvs.read(buffer, 0, 512)) != -1) {
				baos.write(buffer, 0, count);
			}
			String name = slashToDot(entry.getName());
			classByteCodes = baos.toByteArray();
			byteCode.put(name, classByteCodes);
		}
	}
	if (!passVerifier)
		verifyJar0(jvs);
	return classByteCodes;
}
/**
   * JAR's verification in further detail.
   */
private void verifyJar0(JarVerifierStream jvs) throws IOException, SecurityException {
	if (jvs == null)
		return;

	// String name (key), {BigInteger SHA, BigInteger MD5} (value)
	Hashtable nameToHash = jvs.getNameToHash();
	// Manifest File (MANIFEST.MF)
	Manifest manifest = jvs.getManifest();
	// String name (key) and Vector(Identity) (value)
	Hashtable verifiedSignatures = jvs.getVerifiedSignatures();
	// PKCS7 (value)
	Enumeration enumPKCS7 = jvs.getBlocks();

	/* For now, I just assume to use persistent IdentityDatabase provided
	   by SUN that comes together with JDK1.1.1 */
	Enumeration enum = verifiedSignatures.keys();
	boolean trusted = false;
	while (enum.hasMoreElements()) {
		Vector vector = (Vector) verifiedSignatures.get((String) enum.nextElement());
		Enumeration enumVector = vector.elements();
		while (enumVector.hasMoreElements()) {
			Identity identity = (Identity) enumVector.nextElement();
			//System.out.println ("Check this identity: "+identity);
			PublicKey publicKey = identity.getPublicKey();
			signers.addElement(publicKey);
			Identity id = identityScope.getIdentity(publicKey);
			if (id instanceof SystemSigner) {
				SystemSigner systemSigner = (SystemSigner) id;
				//System.out.println ("Signer: "+systemSigner);
				trusted = systemSigner.isTrusted();
				//System.out.println ("The signer is trusted ?"+trusted);
			} else
				if (id instanceof SystemIdentity) {
					SystemIdentity systemIdentity = (SystemIdentity) id;
					trusted = systemIdentity.isTrusted();
				} else
					trusted = false;
		}
	}
	//System.out.println ("out of loop: trusted ?"+trusted);
	if (!trusted) {
		signers = null;
		throw new SecurityException("The mobile code is not trusted");
	}
	if (verifiedSignatures.size() == 0) {
		signers = null;
		throw new SecurityException("No verified signatures returned");
	}
}
/**
   * JAR's verification in detail.
   */
private void verifyJars() throws IOException, SecurityException {
	Hashtable jar = mcc.getJarTable();
	Enumeration keys = jar.keys();
	while (keys.hasMoreElements()) {
		verifyJar((String) keys.nextElement(), passVerifier);
	}
}
}