package mct.users;


/**
 * @(#)Callback.java    June 2, 1997  Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
/**
 * This interface allows the mobile code to be executed based on
 * event as found in the Applet.
 */
public interface Callback {



/**
   * Sends the message to the MobileCodeDaemon to stop the mobile code.<br>
   * Fill with:<br>
   *  if (stub!=null) stub.destroy ();<br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract void destroy();
/**
   * Sends the message to the MobileCodeDaemon to re-initialize 
   * the mobile code.<br>
   * Fill with:<br>
   *  if (stub!=null) stub.init ();<br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract void init();
/**
   * return true if the mobile code is started; otherwise return false.<br>
   * Fill with:<br>
   *  if (stub!=null) return stub.isActive(); <br>
   *  return false;  <br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract boolean isActive();
/**
   * Cleans up whatever resources are being held. 
   * @see #onInit
   * @see #onStart
   * @see #onStop
   */
public void onDestroy();
/**
   * This method is called, once the class is instantiated.
   * Hence, you should put anything that has to be called
   * at beginning after class instantiation.
   */
public boolean onInit();
/**
   * The Mobile Code will be started.
   */
public void onStart();
/**
   * The Mobile Code will be stoped.
   */
public void onStop();
/**
   * Sends the message to the MobileCodeDaemon to start the mobile code.<br>
   * Fill with:<br>
   *  if (stub!=null) stub.start (); <br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract void start();
/**
   * Sends the message to the MobileCodeDaemon to stop the mobile code.<br>
   * Fill with:<br>
   *  if (stub!=null) stub.stop ();<br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract void stop();
}