package mct.users;


/**
 * @(#)Migratable.java    June 2, 1997    Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.io.IOException;
import java.net.InetAddress;

/**
 * This interface provides an interface between the mobile code and the daemon's
 * migration service.
 *
 * @author Gatot Susilo
 * @version 1.0, July 1, 1997
 * @since   MCT1.0
 */
public interface Migratable extends Callback {
	//**********
	//* Fields *
	//**********
	/**
	   * Inform the daemon not migrate this mobile code.
	   */
	final public int DONT_MIGRATE = -1;
	/**
	   * Inform the daemon to serialize this mobile code for migration.
	   * Keep persistent state.
	   */
	final public int SERIALIZE = 0;
	/**
	   * Inform the daemon not to serialize this mobile code for migration.
	   * Do not keep persistent state. 
	   */
	final public int DONT_SERIALIZE = 1;



/**
 * This method returns an array of URLs constituting migration
 * destinations
 * @return java.net.URL[]
 */
public java.net.URL[] getMigrationDestinations();
/**
   * Migrate the mobile code specified in the parameter to the destination specified
   * in the parameters.
   * Fill with: <br>
   *  if (stub!=null) stub.migrate(name, ip, port, flag);<br>
   * @param name the name of mobile code
   * @param ip the ip address of the destination
   * @param port the port number of the destination
   * @param flag true if via TCP; false if via UDP
   */
public abstract void migrate(String name, InetAddress ip, int port, boolean flag);
/**
   * Migrate the mobile code specified in the parameter to the default destination
   * of the daemon.<br>
   * Fill with: <br>
   *  if (stub!=null) stub.migrate(name, flag);<br>
   * @param name the name of mobile code
   * @param flag true if via TCP; false if via UDP
   */
public abstract void migrate(String name, boolean flag);
/**
   * Migrate the mobile code specified in the parameter to the destination specified
   * in the parameters.
   * Fill with: <br>
   *  if (stub!=null) stub.migrate(mobileCode, ip, port, flag);<br>
   * @param mobileCodeObject the instance of mobile code
   * @param ip the ip address of the destination
   * @param port the port number of the destination
   * @param flag true if via TCP; false if via UDP
   */
public abstract void migrate(MobileCodeObject mobileCode, InetAddress ip, int port, boolean flag);
/**
 * This method allows a piece of mobile code to 
 * migrate to a given destination as represented by
 * the identifier of a MigrationInterface object.
 * This has the added advantage of allowing the mobile
 * code to migrate to a logical destination which might be
 * mobile itself. Cool!
 * @param mobileCode mct.users.MobileCodeObject
 * @param ip java.net.InetAddress;
 * @param destination mct.users.MCIdentifier
 */
public abstract void migrate(MobileCodeObject mobileCode, java.net.InetAddress ip, MCIdentifier destination);
/**
   * Migrate the mobile code specified in the parameter to the default destination
   * of the daemon.<br>
   * Fill with: <br>
   *  if (stub!=null) stub.migrate(mobileCode, flag);<br>
   * @param mobileCode the instance of mobile code
   * @param flag true if via TCP; false if via UDP
   */
public abstract void migrate(MobileCodeObject mobileCode, boolean flag);
/**
   * This method will be called by the daemon when the migration fails
   * for some reasons. So the mobile agent can decide what to do next.
   * @param ip the ip destination of migration
   * @param port the port number of migration
   * @param isTCP true if migration via TCP; false if via UDP
   */
public abstract void onFailMigrate(InetAddress ip, int port, boolean isTCP);
/**
   * This method is called when the mobile code is about to be migrated.
   * It returns one of the fields above: DONT_MIGRATE, SERIALIZE, or DONT_SERIALIZE
   */
public int onMigrate();
/**
   * The mobile code is restored.
   */
public boolean onRestore();
/**
   * Sends the message to the MobileCodeDaemon to restore 
   * the persistent states of the mobile code.<br>
   * Fill with:<br>
   *  if (stub!=null) return stub.restore ();<br>
   * The stub is set by the daemon by calling method setStub(stub). 
   * (i.e., the method is in the MobileCode interface)
   * @see mct.users.MobileCode
   */
public abstract void restore();
}