package mct.users;


/**
 * @(#)MobileCode.java    June 2, 1997, Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import mct.admin.MobileCodeStub;

/**
 * All mobile code but VirtualManagedComponent (VMC) must implement this interface.
 * VMC must implement VMCMobileCode instead (see VMCMobileCode).<p>
 *
 * The other components that you may installed independently in your
 * you mobile code are:
 * <ul>
 * <li><b>Callback</b> having methods such as onInit, onRestore, onStart,
 *     onStop, onDestroy as callback of methods init, restore, start, stop,
 *     and destroy respectively, and also method isActive to see if the mobile
 *     code is started or stopped.
 * <li><b>Parameters</b> having methods to access parameters that can
 *     be carried along by the mobile code.
 * <li><b>Migratable</b> having mehtods to allow the mobile code move from
 *     one host to another.
 * <li><b>MCDExtension</b> having methods to get MobileCodeManager and
 *     StorageManager handlers. By this handle, a mobile code can extend services
 *     of the MobileCodeDaemon that relies on mostly to these two classes.
 * <li><b>VMCAccess</b> having methods to access VirtualManagedComponent installed
 *     in the MobileCodeDaemon.
 * <li><b>MessagingAccess</b> having methods to allow a mobile code communicate
 *     with other by passing message.
 * </ul>
 * <p>
 * 
 * The examples of making use of those components are: 
 * <ul>
 * <li>class Netlet that implements interface MobileCode, Parameters, VMCAccess,
 *     Migratable, Callback. This is a typical mobile code that can carry
 *     parameters, access to VMC, and travel from one host to another (migratable).
 * <li>class Extlet that implements interface MobileCode, Parameters, Callback,
 *     MCDExtension. The Extlet is used to extend the daemon with some services.
 *     In most cases, the extlet requires to access the MobileCodeManager or
 *     StorageManager that are accessible through interface MCDExtension.
 *     See the example of TelnetDaemon.
 * </ul>
 *
 * The mobile code is by-default executed as a thread by the daemon.
 * To add thread granularity, the mobile code may implement java.lang.Runnable and
 * invoke a thread somewhere in its method. On migration, the mobile code by-default
 * does not maintain persistent state, unless it implements java.io.Serialazable.
 *
 * @author Gatot Susilo
 * @see mct.users.VMCMobileCode
 * @see java.lang.Runnable
 * @see java.io.Serializable
 * @version 1.0, July 1, 1997
 * @since   MCT1.0
 */
public interface MobileCode {



/**
   * It returns the type of the mobile code.
   */
public abstract String getType();
/**
   * This method is called, once the class is instantiated.
   * Hence, you should put anything that has to be called
   * at beginning after class instantiation.
   */
public boolean onInit(java.util.Properties properties);
}