package mct.users;


/**
 * @(#) VMCManagedResources.java    June 2, 1997  Gatot Susilo
 */
/**************************************************************************
 *                    COPYRIGHT of MOBILE CODE TOOLKIT                    *
 *                                                                        *
 * This program is a free software and follows terms and conditions as    *
 * described on the GNU General Public License                            *
 * (see http://www.sce.carleton.ca/netmanage/gatot/gpl.html).             *
 * This software is freely distributed; however we recommend that         *
 * everybody should download a free copy directly from our site           *
 * http://www.sce.carleton.ca/netmanage/gatot.                            *
 * Therefore, we can keep a track who has got a copy and what for what    *
 * the purpose is. This will allow us to notify you when there are changes* 
 * or bugs fixed on the next release.                                     *                                   
 *                                                                        *
 * This program is a part of work in the Perpetuum Mobile Procura project.*
 * URL: http://www.sce.carleton.ca/netmanage/perpetuum.shtml.             *
 *                                                                        * 
 * Copyright 1997, Gatot Susilo                                           *
 * Department of Systems and Computer Engineering,                        *
 * Carleton University                                                    *
 *                                                                        *
 * Author: Gatot Susilo (susilo@sce.carleton.ca)                          *  
 * Date: October 5, 1997                                                  *
 * Contact Person:                                                        *
 *  Andrzej Bieszczad, Associate Professor, Systems & Computer Engineering*
 *  1125 Colonel By Drive, Carleton University, Ottawa, Canada K1S 5B6    *
 *  Email: andrzej@sce.carleton.ca    Tel:(613)520-5734  Fax:(613)520-5727*
 *  URL: http://www.sce.carleton.ca/faculty/bieszczad.html                *
 **************************************************************************
 *                                NO WARRANTY                             *
 *                                                                        *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY   *
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN*
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES *
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER       *
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE*
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH   *
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL    *
 * NECESSARY SERVICING, REPAIR OR CORRECTION.                             *
 *                                                                        *    
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING  *
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR    *
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR      *
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL   *
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM         *
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED      *
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF *
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR*
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 **************************************************************************/
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This is the basic component of the Virtual Managed Component (VMC) to access
 * the managed resources of the network component. Hence, all VMC must implement
 * MobileCode interface and this interface.<p> 
 * The mobile code must be aware of the managed resources that are being accessed. 
 * An encoding rule such as BER or DER may be implemented to encode the objects
 * being accessed. To allow mobile code know what an encoding rule used by the VMC,
 * the mobile code should check it by accessing Encoding method. If there is no
 * encoding rule at all, the Encoding method must return null.<p>
 *
 * @author Gatot Susilo
 * @see java.lang.Runnable
 * @see mct.users.MobileCode
 * @version 1.0, July 1, 1997
 * @since   MCT1.0
 */
public interface VMCManagedResources {



/**
   * It deletes the object whose name is specified in the parameter.
   */
public abstract void delete(Object name);
/**
   * Return type of encoding rule implemented by the VMC. If there is no encoding
   * rule being implemented, it must return null. The mobile code should be aware
   * of the objects that are being accessed.<p>
   * Typical encoding rules that can implemented for object representation are
   * Basic Encoding Rule (BER), Distinquished Encoding Rule (DER;i.e., subset of BER), etc.
   * If the VMC interfaces to the MIB data, the BER or DER encoding rule should be implemented.
   */
public abstract String Encoding();
/**
   * It gets the object whose name is specified in the parameter.
   * If it is MIB, the name is typically ObjectIdentifier.
   */
public abstract Object get(Object name);
/**
   * It return object's values whose names are specified in the parameter.<br>
   * If no encoding rule implemented, typically it return Hashtable filled with
   * the names and values of objects that are successfully accessed.
   */
public abstract Object getBulk(Object names[]);
/**
   * It returns next object as typically found on the SNMP.<br>
   * If it is MIB, the name is typically ObjectIdentifier.
   */
public abstract Object getNext(Object name);
/**
   * It return the list of the object names.<br>
   * If no encoding rule implemented, typically it returns Enumeration of the object names.
   */
public abstract Object getObjectList();
/**
   * It adds or modifies the object whose name and its value are specified in 
   * the parameters.<br>
   * If is MIB, the name is typically ObjectIdentifier.
   */
public abstract void set(Object name, Object value);
}