package mct.users;


import java.awt.AWTEvent;

/**
 * MCTEvent is the superclass for all events that are defined within
 * the Mobile Code Toolkit. Application netlets may register to 
 * receive events that allow them to track the progress of other
 * mobile code on this MobileCodeDaemon.  
 *
 * This class may be extended as new events are called for.
 * So far, I've structured events according to the user interfaces,
 * namely MobileCode and Callback.
 * 
 * Note that this MCTevent classes are patterned after the AWTevent classes
 * and in particular, I've ensured that the event ids do not 
 * re-use the AWT event ids. Likewise, extensions of this class must 
 * ensure that their event ids do not conflict with those of other 
 * existing subclasses.
 *
 */

public class MCTEvent {
	public final static long MC_EVENT_MASK = 0x01;
	public final static long CALLBACK_EVENT_MASK = 0x02;
	public final static int RESERVED_ID_MIN = AWTEvent.RESERVED_ID_MAX;
	public final static int RESERVED_ID_MAX = RESERVED_ID_MIN + 1999;

	//**********
	//* Fields *
	//**********

	/* 
	   * The Mobile Code that triggered this event
	   */
	Object source; // Should this be of type MCIdentifier

	/* 
	   * The type of event 
	   */
	int id;



public MCTEvent(Object source, int id) {
	this.source = source;
	this.id = id;
}
public int getId() {
	return this.id;
}
public Object getSource() {
	return this.source;
}
public String paramString() {
	return "";
}
public void setSource(Object newSource) {
	this.source = newSource;
}
public String toString() {
	String srcName = null;
	return getClass().getName() + "[" + paramString() + "] on " + (srcName != null ? srcName : source);
}
}